<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Dotenv\Dotenv;

class Bootstrap_Cli extends Zend_Application_Bootstrap_Bootstrap
{

    protected function _initEnvironment()
    {
        $env = new Dotenv(APPLICATION_PATH . '/../');
        $env->load();
    }

    protected function _initConfig()
    {
        $config = [];

        /** @var SplFileInfo $file */
        foreach (new FilesystemIterator(APPLICATION_PATH . '/../config', FilesystemIterator::SKIP_DOTS) as $file) {
            $config[$file->getBasename('.' . $file->getExtension())] = require $file->getPathname();
        }

        Zend_Registry::set('Config', $config);

        enforce_valid_env(app_env());
    }

    protected function _initDatabaseConnection()
    {
        $database = config('databases.mysql');

        MySqlRepository::setDefaultConnection([
            'host'     => $database['host'],
            'user'     => $database['username'],
            'password' => $database['password'],
            'database' => $database['database'],
        ]);

        Zend_Registry::set('Zend_Db_Adapter', new Zend_Db_Adapter_Pdo_Mysql([
            'host'     => $database['host'],
            'username' => $database['username'],
            'password' => $database['password'],
            'dbname'   => $database['database'],
        ]));

        if (Identity::exists()) {
            $user = 'user' . Identity::getUserId();
        } else {
            $user = 'user';
        }

        $userGroups = new UserGroups($database);
        $userGroups->login($user);
        Zend_Registry::set('RC_DILPS_AUTH', $userGroups);
    }

    protected function _initLanguage()
    {
        $path = APPLICATION_PATH . '/../resources/lang/';

        if (Identity::exists() && file_exists(sprintf($path . '/%s/main.php', Identity::getUser()->getSettings()->getLanguage()))) {
            $locale = Identity::getUser()->getSettings()->getLanguage();
        } else {
            $locale = config('app.lang');
        }

        $translate = new Zend_Translate([
            'adapter' => 'array',
            'content' => sprintf($path . '/%s/', $locale),
            'MyLocal' => $locale,
            'scan'    => Zend_Translate::LOCALE_DIRECTORY,
        ]);

        Zend_Registry::set('Zend_Locale', new MyLocal($locale));
        Zend_Registry::set('Zend_Translate', $translate);
    }

    protected function _initRouter()
    {
        $this->bootstrap('frontcontroller');

        $this->getResource('frontcontroller')
            ->setDispatcher(new Dispatcher_Cli())
            ->setResponse(new Zend_Controller_Response_Cli())
            ->setRouter(new Router_Cli())
            ->setRequest(new Zend_Controller_Request_Simple())
            ->setControllerDirectory(config('cli.resources.frontController.controllerDirectory'));
    }

}

