<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

class BackupController_Cli extends AbstractController_Cli
{
    const BACKUP_DATE_FORMAT = 'Y-m-d_H:i';

    public function init()
    {
        $this->_helper->viewRenderer->setNoRender(true);
    }

    public function backupAction()
    {
        $this->backupMysqlAction();

        $this->backupMongoDbAction();
    }

    public function backupMysqlAction()
    {
        $backupDir = $this->getBackupPath();
        $outputFile = $backupDir . date(self::BACKUP_DATE_FORMAT) . '.sql';
        $config = config('databases');

        // backup
        shell_exec('mysqldump ' . $config['database'] . ' ' . implode(' ', [
                '--host=' . $config['host'],
                '--user=' . $config['username'],
                '--password=' . $config['password'],
                '--result-file="' . $outputFile . '"',
            ]));

        // compress
        shell_exec('tar czfP "' . $outputFile . '.gz" "' . $outputFile . '"');

        // cleanup
        unlink($outputFile);
    }

    public function backupMongoDbAction()
    {
        $backupDir = $this->getBackupPath();
        $outputFile = $backupDir . date(self::BACKUP_DATE_FORMAT) . '.bjson';

        // backup
        shell_exec('mongodump --db=' . config('databases.mongodb.database') . ' --archive=' . $outputFile . ' --quiet');

        // compress
        shell_exec('tar czfP "' . $outputFile . '.gz" "' . $outputFile . '"');

        // cleanup
        unlink($outputFile);
    }

    public function garbageAction()
    {
        exec('find ' . $this->getBackupPath(true) . ' -type f -mtime +100 -iname *_04:00.* -prune ! -iname "*-01_*" -exec rm -f {} \;');
    }

    /**
     * @param bool $root
     * @return string
     */
    protected function getBackupPath($root = false)
    {
        $directory = '/home/backup/';

        if (!$root) {
            $directory .= config('app.id') . '/' . date('Y') . '/';
            @mkdir($directory, 0755, true);
        }

        return $directory;
    }

}