<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Portal\Issue;
use Rc\Domain\Repository\Portal\IssueRepository;
use Rc\Domain\Repository\PortalRepository;
use Rc\Domain\Repository\Research\ExpositionRepository;

class IssueMigrationController_Cli extends AbstractController_Cli
{

    public function executeAction()
    {
        Zend_Registry::get('RC_DILPS_AUTH')->login('admin');

        foreach (PortalRepository::findAll() as $portal) {
            if ($portal->isDefault()) {
                continue;
            }

            $issues = [];
            foreach ($portal->getIssueList() as $issueNumber) {
                if (is_null($issueNumber) || $issueNumber === '') {
                    continue;
                }

                $issue = IssueRepository::findOneByPortalAndNumber($portal, $issueNumber);
                if (!isset($issue)) {
                    $issue = new Issue();
                    $issue->setNumber($issueNumber);
                    $issue->setPortal($portal);
                    IssueRepository::persist($issue);
                }

                $issues[$issueNumber] = $issue;
            }
            IssueRepository::flush();

            foreach (ExpositionRepository::findPublishedByPortal($portal) as $research) {
                if (!isset($issues[$research->getPortalEdition()])) {
                    continue;
                }

                $research->setPortalIssue($issues[$research->getPortalEdition()]);

                ExpositionRepository::persist($research);
            }

            ExpositionRepository::flush();
        }
    }
}
