<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use ForceUTF8\Encoding;
use Rc\Domain\Repository\MongoRepository;
use Rc\Domain\Repository\PortalRepository;
use Rc\Domain\Repository\Research\ExpositionRepository;
use Rc\Service\CrossRefService;

class ResearchController_Cli extends AbstractController_Cli
{

    public function updateFullTextAction()
    {
        ini_set('memory_limit', '8192M');
        ini_set('max_execution_time', 86400);

        Zend_Registry::get('RC_DILPS_AUTH')->login('admin');

        foreach (ExpositionRepository::findAll() as $exposition) {
            if ($exposition->getLastModified() < time() - 86400 * 7) {
                continue;
            }

            $fullText = [];
            foreach ($exposition->getWeaves() as $weave) {
                foreach ($weave->getItems() as $tool) {
                    $fullText[] = strip_tags($tool->getDescription() . $tool->getTextContent());
                }
            }
            $fullText = implode(' ', $fullText);
            if ($exposition->getFullText() == $fullText) {
                continue;
            }

            $exposition->setFullText($fullText);
            ExpositionRepository::persist($exposition);
        }
        ExpositionRepository::flush();
    }

    public function depositDoiAction()
    {
        Zend_Registry::get('RC_DILPS_AUTH')->login('admin');

        $forceDeposit = (bool)$this->getParam('force', false);
        $expositionId = $this->getParam('exposition');
        if (!empty($expositionId)) {
            $exposition = ExpositionRepository::findById($expositionId);
            CrossRefService::deposit($exposition, $forceDeposit);
            ExpositionRepository::persist($exposition);
        } else {
            foreach (PortalRepository::findAll() as $portal) {
                if (!$portal->getFeature($portal::FEATURE_EXPOSITION_DOI_DEPOSIT)) {
                    continue;
                }

                $expositions = ExpositionRepository::findPublishedByMainPortal($portal);
                foreach ($expositions as $exposition) {
                    CrossRefService::deposit($exposition, $forceDeposit);
                    ExpositionRepository::persist($exposition);
                }

                ExpositionRepository::persistCollection($expositions);
            }
        }

        MongoRepository::flush();
    }

    public function exportPublishedAction()
    {
        $portal = PortalRepository::findById(2);

        $export = [];
        foreach (ExpositionRepository::findPublishedByPortal($portal) as $research) {
            $export[] = Encoding::toUTF8(
                $research->getPortalEdition() . ';' .
                $research->getAuthor()->getTitle() . ';' .
                $research->getTitle() . ';' .
                implode(';', (array)$research->getKeywords())
            );
        }

        file_put_contents(
            temp_path($portal->getShortName() . '-published-researches.csv'),
            chr(255) . chr(254) . mb_convert_encoding(implode(PHP_EOL, $export), 'UTF-16LE', 'UTF-8')
        );
    }
}