<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Message;
use Rc\Domain\Model\Research\Degree;
use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Repository\PortalRepository;
use Rc\Domain\Repository\Research\DegreeRepository;
use Rc\Domain\Repository\UserRepository;

class DegreeController extends BasicControllerAction
{

    public function addAction()
    {
        if (Identity::getUser()->isLimitedUser()) {
            throw new InsufficientRightsException();
        }

        $degree = new Degree();
        $degree->setOwner(Identity::getUser());
        $form = $this->_getForm($degree)->setDefaultsFrom($degree);

        if ($form->isPostAndValid()) {
            $degree->setValues($form->getValues());
            DegreeRepository::persist($degree);

            KeywordRepository::addKeywords($degree->getKeywords());

            $this->_setSuccessHeader();
            $this->forward('show-own', 'profile');
        } else {
            $this->view->form = $form;
            $this->render('profile-entity/add', null, true);
        }
    }

    public function editAction()
    {
        $degree = $this->_getRequestedDegree();
        $form = $this->_getForm($degree, Identity::getUser())->setDefaultsFrom($degree);

        if ($form->isPostAndValid()) {
            $degree->setValues($form->getValues());
            $degree->setStatus(Degree::STATUS_PROGRESS);
            $degree->setPortalId(null);

            DegreeRepository::persist($degree);

            EntityChangeRepository::addChange($degree, EntityChange::TYPE_EDIT);
            KeywordRepository::addKeywords($degree->getKeywords());

            $this->_setSuccessHeader();
            $this->forward('show-own', 'profile');
        } else {
            $this->view->form = $form;
            $this->render('profile-entity/edit', null, true);
        }
    }

    public function publishAction()
    {
        if (!config('app.allow.publishing')) {
            throw new InsufficientRightsException();
        }

        $degree = $this->_getRequestedDegree();
        if (!$degree->isOwner(Identity::getUser()) || !$degree->hasStatus(Exposition::STATUS_PROGRESS)) {
            throw new InsufficientRightsException();
        }

        $form = $this->_getForm($degree);
        if ($form->isPostAndValid()) {
            $degree->setStatus(Exposition::STATUS_REVIEW);

            $portal = PortalRepository::findById($form->getValue('portalid'));
            // create notification for portal admins
            foreach ($portal->getAdminIds() as $adminId) {
                $message = new Message();
                $message->setSenderId(UserRepository::findSystemAccount()->getId());
                $message->setReceiverId($adminId);
                $message->setType(Message::TYPE_SUBMIT_TO_PORTAL);
                $message->setContent([
                    'user'   => Identity::getUserId(),
                    'entity' => $degree->getId(),
                    'type'   => 'Degree',
                    'portal' => $form->getValue('portalid'),
                ]);
                $this->_sendMessage($message);
            }
            $degree->setPortalId($form->getValue('portalid'));
            DegreeRepository::persist($degree);

            $this->view->portal = $portal ?: null;
        } else {
            $this->view->form = $form;
        }
    }

    public function searchAction()
    {
        $this->_disableView();

        $retVal = [];
        foreach (DegreeRepository::suggest($this->_getParam('property'), $this->_getParam('value'), ['limit' => 10]) as $suggest) {
            $retVal[] = [
                'value' => $suggest->getEntityId(),
                'label' => $suggest->getMatch(),
            ];
        }

        header('Content-Type: application/json');
        echo json_encode($retVal);
    }

}