<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

class FeedsController extends BasicControllerAction
{

    public function init()
    {
        $this->initPortal();
    }

    public function indexAction()
    {
        $this->forward('list');
    }

    public function listAction()
    {
        $page = $this->_getParam('page', 0);
        $changes = [];
        foreach (EntityChangeRepository::findByUser(Identity::getUser(), ['page' => $page]) as $change) {
            if (!$change->getEntity()) {
                continue;
            }

            $changes[] = $change;
        }
        $this->view->changes = $changes;
        $this->view->page = $page;
    }

    public function settingsAction()
    {
        $form = $this->_getForm();
        if ($form->isPostAndValid()) {
            $values = $form->getValues(false);

            if (!empty($values['entities'])) {
                foreach ($values['entities'] as $entityId => $receiveOptions) {
                    $feed = EntityFeedRepository::findOneByUserAndTargetId(Identity::getUser(), $entityId);
                    if (!isset($feed)) {
                        continue;
                    }

                    $feed->setValues($receiveOptions);
                    EntityFeedRepository::update($feed);
                }
            }

            if (!empty($values['persons'])) {
                foreach ($values['persons'] as $userId => $receiveOptions) {
                    $feed = UserFeedRepository::findOneByUserAndTargetId(Identity::getUser(), $userId);
                    if (!isset($feed)) {
                        continue;
                    }

                    $feed->setValues($receiveOptions);
                    UserFeedRepository::update($feed);
                }
            }

            if (!empty($values['portals'])) {
                foreach ($values['portals'] as $userId => $receiveOptions) {
                    $feed = PortalFeedRepository::findOneByUserAndTargetId(Identity::getUser(), $userId);
                    if (!isset($feed)) {
                        continue;
                    }

                    $feed->setValues($receiveOptions);
                    PortalFeedRepository::update($feed);
                }
            }

            $this->_setSuccessHeader();
        } else {
            $this->view->assign('form', $form);
        }
    }

    public function followUserAction()
    {
        $this->_disableView();

        $user = $this->_getRequestedUser();

        Identity::getUser()->followUser($user);
    }

    public function unfollowUserAction()
    {
        $this->_disableView();

        $user = $this->_getRequestedUser();

        Identity::getUser()->unfollowUser($user);
    }

    public function followEntityAction()
    {
        $this->_disableView();

        $entity = $this->_getRequestedEntity();

        Identity::getUser()->followEntity($entity);
    }

    public function unfollowEntityAction()
    {
        $this->_disableView();

        $entity = $this->_getRequestedEntity();

        Identity::getUser()->unfollowEntity($entity);
    }

    public function followPortalAction()
    {
        $this->_disableView();

        $portal = $this->_getRequestedPortal();

        Identity::getUser()->followPortal($portal);
    }

    public function unfollowPortalAction()
    {
        $this->_disableView();

        $portal = $this->_getRequestedPortal();

        Identity::getUser()->unfollowPortal($portal);
    }
}