<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

class ReportController extends BasicControllerAction
{

    public function addAction()
    {
        $form = $this->_getForm();
        $research = $this->_getRequestedResearch();
        $portal = $this->_getRequestedPortal();

        if ($form->isPostAndValid()) {
            $report = new Report();
            $report->setValues($form->getValues());
            $report->setUserId(Identity::getUserId());
            $report->setResearchId($research->getId());
            $report->setPortalId($portal->getId());
            ReportRepository::add($report);

            $this->_setSuccessHeader();

            $this->forward('list');
        } else {
            $this->view->form = $form;
        }
    }

    public function removeAction()
    {
        $report = $this->_getRequestedReport();
        $form = $this->_getForm($report);

        if (!Identity::getUser()->isAdmin()) {
            throw new InsufficientRightsException();
        }

        if ($form->isPostAndValid()) {
            if (Identity::getUser()->isAdmin() && $form->getValue('completely')) {
                ReportRepository::remove($report);
            } else {
                $report->setValues($form->getValues());
                ReportRepository::update($report);
            }

            $this->_setSuccessHeader();
            $this->forward('list', null, null, ['research' => $report->getResearchId()]);
        } else {
            $this->view->form = $form;
        }
    }

    public function convertAction()
    {
        $report = $this->_getRequestedReport();
        $form = $this->_getForm($report);

        if ($report->isDeleted() || !Identity::getUser()->isAdmin()) {
            throw new InsufficientRightsException();
        }

        if ($form->isPostAndValid()) {
            // convert report to comment
            $comment = unserialize('O:7:"Comment"' . substr(serialize($report), 12));
            $comment->setId(null);

            CommentRepository::add($comment);
            ReportRepository::remove($report);

            $this->_disableView();
        } else {
            $this->view->form = $form;
        }
    }

    public function listAction()
    {
        $this->view->isAdmin = Identity::exists() && Identity::getUser()->isAdmin();
        $this->view->research = $this->_getRequestedResearch();
        $this->view->portals = [];
        $this->view->reports = [];
        $this->view->showReports = $this->_getParam('showReports');
        foreach ($this->_getRequestedResearch()->getPublishedInPortals() as $publishedInPortal) {
            $this->view->portals[$publishedInPortal->getId()] = $publishedInPortal;
            $this->view->reports[$publishedInPortal->getId()] = ReportRepository::findByResearchAndPortal($this->_getRequestedResearch(), $publishedInPortal, 0);
        }
    }

}

?>