<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Research\Snapshot;
use Rc\Domain\Model\Translation;
use Rc\Domain\Repository\Research\SnapshotRepository;
use Rc\Domain\Repository\ResearchRepository;
use Rc\Utility\DuplicateFacade;

class SnapshotController extends BasicControllerAction
{

    /**
     * @throws Exception
     * @throws InsufficientRightsException
     */
    public function addAction()
    {
        if (Identity::getUser()->isLimitedUser()) {
            throw new InsufficientRightsException();
        }

        $research = $this->_getRequestedResearch();

        $form = $this->_getForm();
        if ($form->isPostAndValid()) {
            $serializer = new \Rc\Serializer\Serializer();

            $snapshot = new Snapshot($research, $serializer->serialize($research));
            $snapshot->setValues($form->getValues());

            SnapshotRepository::persist($snapshot);

            $this->_setSuccessHeader();
            $this->_disableView();
        } else {
            $this->view->form = $form;
        }
    }

    /**
     * @throws Exception
     * @throws InsufficientRightsException
     */
    public function restoreAction()
    {
        $snapshot = $this->_getRequestedSnapshot();
        $currentResearch = $snapshot->getResearch();
        $serializer = new \Rc\Serializer\Serializer();
        $duplicateFacade = new DuplicateFacade();

        if (!$currentResearch->isMetaEditableBy(Identity::getUser())) {
            throw new InsufficientRightsException();
        }

        $form = $this->_getForm($snapshot);
        if ($this->getRequest()->isPost()) {
            $formToUse = $this->_getSubmittedTab($form);
        } else {
            $formToUse = $form;
        }

        if ($formToUse->isPostAndValid()) {
            $options = [
                'collaboration' => $formToUse->getValue('collaboration'),
            ];

            switch ($formToUse->getName()) {
                case 'tabduplicate':
                    $research = $duplicateFacade->duplicate($currentResearch, $snapshot->getSnapshot(), $options);
                    break;
                case 'tabreplace':
                    $research = $duplicateFacade->restore($currentResearch, $snapshot->getSnapshot(), $options);

                    $snapshotBackup = new Snapshot($currentResearch, $serializer->serialize($currentResearch));
                    SnapshotRepository::persist($snapshotBackup);

                    $currentResearch->deleteRelations();
                    break;
                default:
                    throw new InvalidArgumentException('Invalid restore action!');
            }

            if ($formToUse->getValue('title')) {
                $research->setTitle(new Translation($formToUse->getValue('title')));
            }

            ResearchRepository::persist($research);

            $this->_setSuccessHeader();
            $this->_disableView();
        } else {
            $this->view->form = $formToUse;
        }
    }

    /**
     * @throws Exception
     * @throws InsufficientRightsException
     */
    public function removeAction()
    {
        $snapshot = $this->_getRequestedSnapshot();

        if (!$snapshot->getResearch()->isMetaEditableBy(Identity::getUser())) {
            throw new InsufficientRightsException();
        }

        $form = $this->_getForm($snapshot);
        if ($form->isPostAndValid()) {
            SnapshotRepository::remove($snapshot);

            $this->_setSuccessHeader();
            $this->_disableView();
        } else {
            $this->view->form = $form;
        }
    }

    /**
     * @throws \Rc\Domain\Repository\Exception
     */
    public function listAction()
    {
        $research = $this->_getRequestedResearch();

        $this->view->entity = $research;
        $this->view->snapshots = SnapshotRepository::findByResearch($research);
    }

}
