<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Model\Research\Exposition\Weave\GraphicalWeave;
use Rc\Domain\Repository\Research\Exposition\WeaveRepository;
use Rc\Domain\Repository\Research\ExpositionRepository;

class WeaveController extends BasicControllerAction
{

    /**
     * @return void
     */
    public function thumbAction()
    {
        $this->view->thumb = $this->_getRequestedWeave()->getThumb($this->_getParam('w'), $this->_getParam('h'));
    }

    public function addAction()
    {
        $exposition = $this->_getRequestedResearch();
        $form = $this->_getForm($exposition);

        if ($form->isPostAndValid()) {
            $weave = new GraphicalWeave();
            $weave->setOwner(Identity::getUser())
                ->setExposition($exposition)
                ->setValues($form->getValues());
            WeaveRepository::persist($weave);

            $this->updateResearchLastModified($exposition);

            $this->_setSuccessHeader();

            $this->view->weave = $weave;
        } else {
            $this->view->form = $form;
        }
    }

    public function editAction()
    {
        $weave = $this->_getRequestedWeave();
        $form = $this->_getForm($weave->getExposition(), $weave)->setDefaultsFrom($weave);

        if ($form->isPostAndValid()) {
            $weave->setValues($form->getValues());
            WeaveRepository::persist($weave);

            $this->updateResearchLastModified($weave->getExposition());

            $this->_disableView();
        } else {
            $this->view->form = $form;
            $this->view->weave = $weave;
        }
    }

    public function removeAction()
    {
        $weave = $this->_getRequestedWeave();
        if (Identity::getUser()->isPortalAdmin()) {
            Zend_Registry::get('RC_DILPS_AUTH')->addTemporaryEditGroupFor($weave->getExposition()->getId());
        }
        $exposition = $weave->getExposition();
        $form = $this->_getForm($weave);

        if (!$exposition->isOwner(Identity::getUser()) && !$exposition->isCoAuthor(Identity::getUser())
            && !($exposition->isStatus(Exposition::STATUS_REVIEW) && Identity::getUser()->isPortalAdmin())
        ) {
            throw new InsufficientRightsException();
        } else if (count($exposition->getWeaves()) == 1) {
            $this->view->lastWeave = true;
        } else if ($form->isPostAndValid()) {
            $this->updateResearchLastModified($exposition);

            // remove weave entry from chapterlist
            $chapterList = $exposition->getChapterList();
            foreach ($chapterList as $key => $value) {
                if ($value['weaveid'] == $weave->getId()) {
                    unset($chapterList[$key]);
                }
            }
            $exposition->setChapterList($chapterList);

            // remove weave if it is start site
            if ($exposition->getDefaultWeaveId() === $weave->getId()) {
                $exposition->setDefaultWeave(null);
            }

            ExpositionRepository::persist($exposition);
            WeaveRepository::remove($weave);

            $this->_disableView();
        } else {
            $this->view->form = $form;
            $this->view->weave = $weave;
        }
    }

    public function setDefaultAspectAction()
    {
        $weave = $this->_getRequestedWeave();
        $weave->setDefaultAspect($this->_getParam('x'), $this->_getParam('y'));

        WeaveRepository::persist($weave);

        $this->updateResearchLastModified($weave->getExposition());

        $this->view->weave = $weave;
    }

    public function showCurrentAspectAction()
    {
        $this->view->weave = $this->_getRequestedWeave();
        $this->view->currentAspect = ['x' => $this->_getParam('x'), 'y' => $this->_getParam('y')];
    }

}

?>