<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

class CommentFormGenerator extends BasicFormGenerator
{

    public static function getAdd()
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.comment.add');

        $form->addElement('hidden', 'author', [
            'required' => true,
            'value'    => Identity::getUser()->getName(),
            'readonly' => 'readonly',
        ])
            ->addElement('textarea', 'content', [
                'label'       => 'field.content',
                'required'    => true,
                'data-editor' => true,
            ]);

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    public static function getRemove()
    {
        $form = parent::getRemove();
        $form->setTitle('dialog.comment.remove');

        $form->addElement('hidden', 'deleted', [
            'value'    => time(),
            'readonly' => true,
            'required' => true,
        ])
            ->addElement('hidden', 'deletedById', [
                'value'    => Identity::getUserId(),
                'readonly' => true,
                'required' => true,
            ]);

        if (Identity::getUser()->isAdmin()) {
            $form->addElement('checkbox', 'completely', [
                'order'    => 1,
                'value'    => 0,
                'required' => true,
                'label'    => 'field.removePermanent',
            ]);
        }

        return $form;
    }

    public static function getConvert(Comment $comment = null)
    {
        foreach ($comment->getResearch()->getPublishedInPortals() as $publishedInPortal) {
            $portals[$publishedInPortal->getId()] = $publishedInPortal->getName();
        }

        $form = new DefaultForm();
        $form->setTitle('dialog.comment.convert', $comment);

        $form->addElement('html', 'info', [
            'value' => static::_getTranslate()->translate('info.comment.convert'),
        ])
            ->addElement('select', 'portalid', [
                'label'        => 'field.portal',
                'required'     => true,
                'multiOptions' => ['' => ''] + $portals,
            ])
            ->addElement('hidden', 'confirmation', [
                'value'    => 'confirmation',
                'required' => true,
            ]);

        $form->addCancelButton('no')->addSubmitButton('yes');

        return $form;
    }

}

?>