<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Model\Research\Exposition\SimpleMedia;
use Rc\Domain\Model\Research\Exposition\Weave;
use Rc\Domain\Model\Research\Exposition\Weave\Tool;
use Rc\Domain\Model\Research\Exposition\Weave\Tool\AudioTool;
use Rc\Domain\Model\Research\Exposition\Weave\Tool\PictureTool;
use Rc\Domain\Model\Research\Exposition\Weave\Tool\ShapeTool;
use Rc\Domain\Model\Research\Exposition\Weave\Tool\SlideshowTool;
use Rc\Domain\Model\Research\Exposition\Weave\Tool\TextTool;
use Rc\Domain\Model\Research\Exposition\Weave\ToolInterface;
use Rc\Domain\Repository\Core\RevisionRepository;
use Rc\Domain\Repository\Research\Exposition\Weave\ToolRepository;

class ItemFormGenerator extends BasicFormGenerator
{

    public static function getAdd(ToolInterface $tool, Exposition $exposition)
    {
        $form = new DefaultForm();
        $form->setTitle(
            sprintf(
                self::_getTranslate()->translate('dialog.item.edit'),
                self::_getTranslate()->translate('tool.' . $tool->getName())
            ),
            $tool
        );

        $tab = $form->addTab('common');
        $tab->addElement(
            'text',
            'title',
            [
                'label' => 'field.name',
                'required' => true,
            ]
        );

        $tab = $form->addTab('media');
        if ($tool->getName() === ToolFactory::TOOL_EMBED) {
            $tab->addElement(
                'html',
                'info-publishing',
                [
                    'value'    => '<br/><span style="float: left; margin-right: .3em;" class="ui-icon ui-icon-info"></span>Please note that some RC based journals/portals do not allow use of externally hosted materials. If you are making an exposition to be published in such a portal, please check their policy first.',
                ]
            );
            $tab->addElement(
                'html',
                'info',
                [
                    'value' => '<br/><span style="float: left; margin-right: .3em;" class="ui-icon ui-icon-info"></span>The RC supports the following external content providers: Madek, Youtube, Vimeo, Soundcloud, Sketchfab. Please copy the URL of the media item you want to embed.',
                ]
            );
            $tab->addElement(
                'text',
                'url',
                [
                    'label'    => 'field.url',
                    'required' => true,
                    'onchange' => 'Editor.processEmbedUrl(this)',
                    'validators' => [
                        ['EmbedUrl'],
                     ],
                ]
            );
            $tab->addElement(
                'hidden',
                'urlApi',
                [
                    'required' => true,
                ]
            );
            $tab->addElement(
                'hidden',
                'previewUrl'
            );
            $tab->addElement(
                'html',
                'info-data-changed',
                [
                    'value' => '<span style="float: left; margin-right: .3em" class="ui-icon ui-icon-info"></span>The tool dimension and tool options were updated to fit your current input. You can review the changes now or simply submit.',
                    'style' => 'display: none; font-weight: bold',
                ]
            );
        } elseif ($tool->getName() === ToolFactory::TOOL_COMMENT) {
            $tab->addElement(
                'textarea',
                'textcontent',
                [
                    'class' => 'documentwork',
                ]
            );
        } elseif ($tool->getName() === Tool::MEDIA_TYPE_TEXT) {
            $tab->addElement(
                'button',
                'load',
                [
                    'icon'    => 'ui-icon-plus',
                    'label'   => 'button.resource.load',
                    'onclick' => 'Dialog.loadResource(' . $tool->getId() . ', {context:this});return false',
                ]
            )
                ->addElement(
                    'textarea',
                    'textcontent',
                    [
                        'data-editor'           => true,
                        'data-background-color' => $tool->getWeave() ? $tool->getWeave()->getBackgroundColor() : '',
                        'class'                 => 'documentwork',
                    ]
                )
                ->addElement(
                    'button',
                    'save',
                    [
                        'icon'    => 'ui-icon-plus',
                        'label'   => 'button.resource.save',
                        'onclick' => 'Dialog.addTemplate($("#frm-media-textcontent").val());return false',
                    ]
                )
                ->addElement(
                    'html',
                    'clear',
                    [
                        'style' => 'clear:both',
                    ]
                );
        } elseif ($tool->getName() === Tool::MEDIA_TYPE_SIMPLE_TEXT) {
            $tab->addElement(
                'textarea',
                'textcontent',
                [
                    'data-simpleeditor' => true,
                    'id'                => 'wysihtml5-editor',
                    // the id is set in DialogForm.js too, because zf1 ignore this configuration
                ]
            );
        } elseif ($tool->getName() === Tool::MEDIA_TYPE_SHAPE) {
            foreach (ShapeFactory::getAvailableShapes() as $shapeId) {
                $activeClass = ($tool->getShapeContent() === $shapeId ? 'active' : null);
                $shape = ShapeFactory::createShapeByType(
                    $shapeId,
                    50,
                    50,
                    [
                        'fill'   => 'black',
                        'stroke' => [
                            'color'     => 'black',
                            'width'     => 3,
                            'dashArray' => '',
                        ],
                    ]
                );
                $value[] = '<li class="' . $activeClass . '" data-id="' . $shapeId . '" data-options=",' . implode(
                        ',',
                        $tool->getValidOptions($shapeId)
                    ) . ',">' . $shape . '</li>';
            }

            $tab->addElement(
                'hidden',
                'shapecontent',
                [
                    'value' => '',
                ]
            );
            $tab->addElement(
                'html',
                'shapes',
                [
                    'value'   => '<ul class="shapelist">' . implode(null, $value) . '</ul>',
                    'onclick' => '
                if (!$(event.target).closest("li").length) return false;
                var target = $(event.target).closest("li");
                $(this).find("li").removeClass("active");
                target.addClass("active");
                $("#frm-media-shapecontent").val(target.attr("data-id"));
                $("[id*=frm-options]").closest("p").show();
                (target.attr("data-options").indexOf(",stroke,") === -1 ? $("[id*=frm-options-stroke]").closest("p").hide() : null);
                (target.attr("data-options").indexOf(",fill,") === -1 ? $("[id*=frm-options-fill]").closest("p").hide() : null);',
                ]
            );
            $tab->addElement(
                'html',
                'shapeoptions',
                [
                    'value' => '',
                ]
            );
        }

        $tab = $form->addTab('style');

        if (in_array($tool->getName(), [Tool::MEDIA_TYPE_AUDIO, Tool::MEDIA_TYPE_VIDEO, Tool::MEDIA_TYPE_EMBED])) {
            $tab->addAccordion(
                'layout',
                [
                    $tab->createElement(
                        'number',
                        'left',
                        [
                            'label'    => 'field.left',
                            'required' => true,
                            'min'      => -1,
                        ]
                    ),
                    $tab->createElement(
                        'number',
                        'top',
                        [
                            'label'    => 'field.top',
                            'required' => true,
                            'min'      => -1,
                        ]
                    ),
                    $tab->createElement(
                        'number',
                        'width',
                        [
                            'label'    => 'field.width',
                            'required' => true,
                            'min'      => -1,
                        ]
                    ),
                    $tab->createElement(
                        'number',
                        'height',
                        [
                            'label'    => 'field.height',
                            'required' => true,
                            'min'      => -1,
                        ]
                    ),
                ]
            );
        } else {
            $tab->addAccordion(
                'layout',
                [
                    $tab->createElement(
                        'number',
                        'left',
                        [
                            'label'    => 'field.left',
                            'required' => true,
                            'min'      => -1,
                        ]
                    ),
                    $tab->createElement(
                        'number',
                        'top',
                        [
                            'label'    => 'field.top',
                            'required' => true,
                            'min'      => -1,
                        ]
                    ),
                    $tab->createElement(
                        'number',
                        'width',
                        [
                            'label'    => 'field.width',
                            'required' => true,
                            'min'      => -1,
                        ]
                    ),
                    $tab->createElement(
                        'number',
                        'height',
                        [
                            'label'    => 'field.height',
                            'required' => true,
                            'min'      => -1,
                        ]
                    ),
                    $tab->createElement(
                        'number',
                        'rotate',
                        [
                            'label'    => 'field.rotate',
                            'required' => true,
                            'step'     => '0.01',
                        ]
                    ),
                ]
            );
        }

        $tab->addAccordion(
            'padding',
            [
                $tab->createElement(
                    'number',
                    'padding-left',
                    [
                        'label' => 'field.left',
                        'min'   => -1,
                    ]
                ),
                $tab->createElement(
                    'number',
                    'padding-top',
                    [
                        'label' => 'field.top',
                        'min'   => -1,
                    ]
                ),
                $tab->createElement(
                    'number',
                    'padding-right',
                    [
                        'label' => 'field.right',
                        'min'   => -1,
                    ]
                ),
                $tab->createElement(
                    'number',
                    'padding-bottom',
                    [
                        'label' => 'field.bottom',
                        'min'   => -1,
                    ]
                ),
            ]
        );
        $tab->addAccordion('border', [
            $tab->createElement('select', 'border-style', [
                'label'        => 'field.style',
                'multioptions' => Tool::getBorderStyles(),
            ]),
            $tab->createElement('number', 'border-width', [
                'label' => 'field.strength',
                'min'   => -1,
            ]),
            $tab->createElement('color', 'border-color', [
                'label' => 'field.color',
            ]),
            $tab->createElement('number', 'border-radius', [
                'label' => 'field.radius',
            ]),
        ]);
        $tab->addAccordion(
            'background',
            [
                $tab->createElement(
                    'color',
                    'background-color',
                    [
                        'label' => 'field.color',
                    ]
                ),
                $tab->createElement(
                    'select',
                    'background-image-file-id',
                    [
                        'label'        => 'field.image',
                        'multiOptions' => ['' => ''] + self::_getSupportedFiles(new PictureTool(), $exposition),
                    ]
                ),
                $tab->createInlineElement(
                    [
                        $tab->createElement(
                            'select',
                            'background-image-style',
                            [
                                'label'        => 'field.image.properties',
                                'style'        => 'width:90px',
                                'multioptions' => Tool::getBackgroundImageStyles(),
                            ]
                        ),
                        $tab->createElement(
                            'select',
                            'background-image-position',
                            [
                                'style'        => 'width:112px',
                                'multioptions' => Tool::getBackgroundImagePositions(),
                            ]
                        ),
                        $tab->createElement(
                            'select',
                            'background-image-size',
                            [
                                'style'        => 'width:90px',
                                'multioptions' => Tool::getBackgroundImageSizes(),
                            ]
                        ),
                    ]
                ),
            ]
        );
        $tab->addAccordion(
            'shadow',
            [
                $tab->createElement(
                    'number',
                    'shadow-margin-left',
                    [
                        'label' => 'field.margin.left',
                    ]
                ),
                $tab->createElement(
                    'number',
                    'shadow-margin-top',
                    [
                        'label' => 'field.margin.top',
                    ]
                ),
                $tab->createElement(
                    'number',
                    'shadow-unschaerfe',
                    [
                        'label' => 'field.unschaerfe',
                    ]
                ),
                $tab->createElement(
                    'color',
                    'shadow-color',
                    [
                        'label' => 'field.color',
                    ]
                ),
            ]
        );
        $tab->addAccordion(
            'opacity',
            [
                $tab->createElement(
                    'range',
                    'opacity',
                    [
                        'label' => 'field.opacity',
                        'min'   => -1,
                        'max'   => 100,
                        'step'  => 1,
                    ]
                ),
            ]
        );

        $tab = $form->addTab('options');
        self::_appendOptions($tab, $tool);

        $form->addCancelButton()->addSubmitButton()->addElement(
            'button',
            'remove',
            [
                'label'   => 'button.tool.remove',
                'class'   => 'button-tool-remove',
                'onclick' => 'var button=this;new DialogForm({
		url: "/item/remove?" + Dialog.toParam([' . $tool->getId() . '], "item") + "&research=" + Editor.getResearchId(),
		eAfterValidSubmit: function() {
		    Editor.removeItems([' . $tool->getId() . ']);
		    $("[name=cancelbutton]",$(button).parent()).click();
		}
	    });return false',
            ]
        );

        return $form;
    }

    /**
     * @param Tool $tool
     * @param Exposition $exposition
     * @return DefaultForm
     */
    public static function getEdit(Tool $tool, Exposition $exposition)
    {
        $form = self::getAdd($tool, $exposition);

        $history = [];
        foreach (RevisionRepository::findByDocument($tool, ['limit' => 25]) as $revision) {
            $row = '<tr>
		        <td>' . $revision->getCreated('d/m/y - H:i:s') . '
		        <td>' . $revision->getUser()->getName() . '
		        <td>' . $revision->getLabel() . '
		        <td><button class="ui-button ui-widget ui-state-default ui-corner-all ui-button-text-icon-primary" style="margin:0"
		        onclick="Dialog.restoreTool(' . $revision->getDocument()->getId() . ',\'' . $revision->getUuid() . '\'); return false;">Restore</button>';

            $history[] = $row;
        }

        $form->addTab('history')
            ->addElement(
                'html',
                'history',
                [
                    'value' => '<br/><table style="width:569px"><tr><th>modified at<th>modifier<th>modification<th>' . implode(
                            null,
                            $history
                        ) . '</table>',
                ]
            );

        return $form;
    }

    public static function getRestore()
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.item.restore');

        $form->addElement(
            'html',
            'info',
            [
                'value' => static::_getTranslate()->translate('info.entry.restore'),
            ]
        )
            ->addElement(
                'hidden',
                'confirmation',
                [
                    'value'    => 'confirmation',
                    'required' => true,
                ]
            );

        $form->addCancelButton('no')->addSubmitButton('yes');

        return $form;
    }

    public static function getRestoreDeleted(Weave $weave)
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.item.restore.deleted');

        $history = [];
        foreach (RevisionRepository::findDeletedToolsByWeave($weave, ['limit' => 25]) as $revision) {
            $history[] = '<tr>
		        <td>' . $revision->getCreated('d/m/y - H:i:s') . '
		        <td>' . $revision->getUser()->getName() . '
		        <td>' . $revision->getLabel() . '
		        <td><button  class="ui-button ui-widget ui-state-default ui-corner-all ui-button-text-icon-primary" style="margin:0"
		        onclick="Dialog.restoreTool(' . $revision->getDocument()->getId() . ',\'' . $revision->getUuid() . '\',true); return false;">Restore</button>';
        }

        if ($history) {
            $form->addElement(
                'html',
                'history',
                [
                    'value' => '<table style="width:569px"><tr><th>deleted at<th>modifier<th>title<th>' . implode(
                            null,
                            $history
                        ) . '</table>',
                ]
            );
        } else {
            $form->addElement(
                'html',
                'history',
                [
                    'value' => 'No deleted tools found.',
                ]
            );
        }

        $form->addCancelButton('ok');

        return $form;
    }

    public static function getConvert()
    {
        $form = new DefaultForm();

        $form->setTitle('dialog.item.convert')
            ->addElement('html', 'info', [
                'value' => static::_getTranslate()->translate('info.entry.convert'),
            ])
            ->addElement('hidden', 'confirmation', [
                'value'    => 'confirmation',
                'required' => true,
            ]);

        $form->addCancelButton('no')->addSubmitButton('yes');

        return $form;
    }

    /**
     * @param Tool $item
     * @return DefaultForm
     * @throws Zend_Exception
     */
    public static function getRemove(Tool $item = null)
    {
        $form = parent::getRemove();
        $form->setTitle('dialog.item.remove');
        if ($item->isReviewing() && $item->getExposition()->isStatus(Exposition::STATUS_REVIEW)) {
            $form->getElement('info')->setValue(static::_getTranslate()->translate('info.item.remove.finally'));
        } else {
            $form->getElement('info')->setValue(static::_getTranslate()->translate('info.item.remove'));
        }

        return $form;
    }

    public static function getPasteError()
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.items.paste.error');

        $form->addElement(
            'html',
            'info',
            [
                'value' => static::_getTranslate()->translate('info.items.paste.error'),
            ]
        );

        $form->addSubmitButton('ok');

        return $form;
    }

    public static function getSynchronize()
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.item.synchronize');

        $form->addElement(
            'html',
            'info',
            [
                'value' => static::_getTranslate()->translate('info.item.synchronize'),
            ]
        )
            ->addElement(
                'hidden',
                'confirm',
                [
                    'value'    => 'item.synchronize',
                    'required' => true,
                ]
            );

        $form->addCancelButton('no')->addSubmitButton('yes');

        return $form;
    }

    public static function getSelectResource(ToolInterface $tool, Exposition $exposition)
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.resource.add');

        $form->addElement(
            'multiselect',
            'file',
            [
                'label'            => 'field.files',
                'data-multiselect' => true,
                'multiOptions'     => self::_getSupportedFiles($tool, $exposition),
                'data-maxcount' => $tool->getResourceLimit(),
            ]
        );

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    public static function getSelectAudio(ToolInterface $tool, Exposition $exposition)
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.resource.add');

        $form->addElement(
            'select',
            'audio',
            [
                'label'        => 'field.audio',
                'multioptions' => [null] + self::_getSupportedFiles(new AudioTool(), $exposition),
            ]
        );

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    public static function getSelectImage(ToolInterface $tool, Exposition $exposition)
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.resource.add');

        $form->addElement(
            'select',
            'image',
            [
                'label'        => 'field.image',
                'multioptions' => [null] + self::_getSupportedFiles(new PictureTool(), $exposition),
            ]
        );

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    public static function getEditResource(ToolInterface $tool, SimpleMedia $simpleMedia)
    {
        $form = SimpleMediaFormGenerator::getEdit($simpleMedia);
        $form->setTitle('dialog.resource.edit', $tool);

        return $form;
    }

    public static function getLoadResource(ToolInterface $tool, Exposition $exposition)
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.resource.load');

        $form->addElement(
            'select',
            'file',
            [
                'label'        => 'field.files',
                'multioptions' => self::_getSupportedFiles($tool, $exposition),
            ]
        );

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    public static function getRemoveResource(SimpleMedia $simpleMedia)
    {
        $form = parent::getRemove();
        $form->setTitle('dialog.file.remove', $simpleMedia);
        $form->getElement('info')->setValue(static::_getTranslate()->translate('info.entry.item.remove'));
        return $form;
    }

    protected static function _appendOptions(DefaultForm $tab, ToolInterface $tool)
    {
        $method = '_getOptionsFor' . ucfirst($tool->getName()) . 'Tool';
        self::$method($tab, $tool);
    }

    protected static function _getOptionsForPictureTool(DefaultForm $tab)
    {
        $tab->addAccordion(
            'position',
            [
                $tab->createElement(
                    'select',
                    'size',
                    [
                        'label'        => 'field.size',
                        'multioptions' => PictureTool::getSizes(),
                    ]
                ),
                $tab->createElement(
                    'select',
                    'position',
                    [
                        'label'        => 'field.position',
                        'multioptions' => PictureTool::getPositions(),
                    ]
                ),
            ]
        );
        $tab->addAccordion(
            'settings',
            [
                $tab->createElement(
                    'checkbox',
                    'fix-aspect-ratio',
                    [
                        'label' => 'field.fix-aspect-ratio',
                    ]
                ),
                $tab->createElement(
                    'checkbox',
                    'open-picture-in-popup',
                    [
                        'label' => 'field.open-picture-in-popup',
                    ]
                ),
                $tab->createElement(
                    'text',
                    'open-link',
                    [
                        'label' => 'field.open-link',
                    ]
                ),
                $tab->createElement(
                    'select',
                    'hover-popup',
                    [
                        'label'        => 'field.hover-popup',
                        'multioptions' => Tool::getHoverPopupOptions(),
                    ]
                ),
                $tab->createElement(
                    'textarea',
                    'hover-plain-text',
                    [
                        'label' => 'field.hover-plain-text',
                        'class' => 'documentwork',
                    ]
                ),
            ]
        );
    }

    protected static function _getOptionsForSlideshowTool(DefaultForm $tab, ToolInterface $tool)
    {

        $tab->addAccordion(
            'position',
            [
                $tab->createElement(
                    'select',
                    'size',
                    [
                        'label'        => 'field.size',
                        'multioptions' => PictureTool::getSizes(),
                    ]
                ),
                $tab->createElement(
                    'select',
                    'position',
                    [
                        'label'        => 'field.position',
                        'multioptions' => PictureTool::getPositions(),
                    ]
                ),
            ]
        );
        $tab->addAccordion(
            'settings',
            [
                $tab->createElement(
                    'checkbox',
                    'loop',
                    [
                        'label' => 'field.loop',
                    ]
                ),
                $tab->createElement(
                    'checkbox',
                    'shownavbar',
                    [
                        'label' => 'field.show-navbar',
                    ]
                ),
                $tab->createElement(
                    'checkbox',
                    'open-picture-in-popup',
                    [
                        'label' => 'field.open-picture-in-popup',
                    ]
                ),
                $tab->createElement(
                    'select',
                    'hover-popup',
                    [
                        'label'        => 'field.hover-popup',
                        'multioptions' => Tool::getHoverPopupOptions(),
                    ]
                ),
                $tab->createElement(
                    'textarea',
                    'hover-plain-text',
                    [
                        'label' => 'field.hover-plain-text',
                        'class' => 'documentwork',
                    ]
                ),
            ]
        );
        $tab->addAccordion(
            'automate',
            [
                $tab->createElement(
                    'select',
                    'play-option',
                    [
                        'label'        => 'field.play-option',
                        'multioptions' => SlideshowTool::getPlayOptions(),
                    ]
                ),
                $tab->createElement(
                    'number',
                    'default-slide-speed',
                    [
                        'label' => 'field.default-speed',
                    ]
                ),
                $tab->createElement(
                    'text',
                    'slides-speed',
                    [
                        'label'          => 'field.speed',
                        'data-resources' => json_encode(self::getSlideshowResourceList($tool)),
                    ]
                ),
            ]
        );
        $tab->addAccordion(
            'audioSlideshow',
            [
                $tab->createElement(
                    'text',
                    'crossfade',
                    [
                        'label' => 'field.crossfade',
                    ]
                ),
                $tab->createElement(
                    'number',
                    'fade-duration',
                    [
                        'label' => 'field.fade-duration',
                    ]
                ),
                $tab->createElement(
                    'number',
                    'timeout-duration',
                    [
                        'label' => 'field.timeout-duration',
                        'min'   => 749,
                    ]
                ),
            ]
        );
    }

    protected static function _getOptionsForObjectViewerTool(DefaultForm $tab, Tool $tool)
    {
        $tab->addAccordion(
            'settings',
            [
                $tab->createElement(
                    'number',
                    'levels',
                    [
                        'label' => 'field.levels',
                    ]
                ),
                $tab->createElement(
                    'text',
                    'startlevel',
                    [
                        'label'          => 'field.startlevel',
                        'data-resources' => json_encode(self::getObjectViewerResourceList($tool)),
                    ]
                ),
                $tab->createElement(
                    'checkbox',
                    'autorotate',
                    [
                        'label' => 'field.autorotate',
                    ]
                ),
            ]
        );
    }

    protected static function _getOptionsForVideoTool(DefaultForm $tab)
    {
        $tab->addAccordion(
            'settings',
            [
                $tab->createElement(
                    'checkbox',
                    'fix-aspect-ratio',
                    [
                        'label' => 'field.fix-aspect-ratio',
                    ]
                ),
                $tab->createElement(
                    'checkbox',
                    'loop',
                    [
                        'label' => 'field.loop',
                    ]
                ),
                $tab->createElement(
                    'checkbox',
                    'autoplay',
                    [
                        'label' => 'field.autoplay',
                    ]
                ),
                $tab->createElement(
                    'checkbox',
                    'displayVolume',
                    [
                        'label' => 'field.display.volume',
                        'value' => false,
                    ]
                ),
                $tab->createElement(
                    'checkbox',
                    'displayMinimal',
                    [
                        'label' => 'field.display.minimal',
                        'value' => false,
                    ]
                ),
                $tab->createElement(
                    'checkbox',
                    'hidePlayer',
                    [
                        'label' => 'field.hide.player',
                        'value' => false,
                    ]
                ),
                $tab->createElement(
                    'checkbox',
                    'stopOtherPlayers',
                    [
                        'label' => 'field.stop.other.players',
                        'value' => false,
                    ]
                ),
                $tab->createElement(
                    'select',
                    'hover-popup',
                    [
                        'label'        => 'field.hover-popup',
                        'multioptions' => Tool::getHoverPopupOptions(),
                    ]
                ),
                $tab->createElement(
                    'textarea',
                    'hover-plain-text',
                    [
                        'label' => 'field.hover-plain-text',
                        'class' => 'documentwork',
                    ]
                ),
            ]
        );
    }

    protected static function _getOptionsForAudioTool(DefaultForm $tab)
    {
        $tab->addAccordion(
            'settings',
            [
                $tab->createElement(
                    'checkbox',
                    'loop',
                    [
                        'label' => 'field.loop',
                    ]
                ),
                $tab->createElement(
                    'checkbox',
                    'autoplay',
                    [
                        'label' => 'field.autoplay',
                    ]
                ),
                $tab->createElement(
                    'checkbox',
                    'displayVolume',
                    [
                        'label' => 'field.display.volume',
                        'value' => false,
                    ]
                ),
                $tab->createElement(
                    'checkbox',
                    'displayMinimal',
                    [
                        'label' => 'field.display.minimal',
                        'value' => false,
                    ]
                ),
                $tab->createElement(
                    'checkbox',
                    'hidePlayer',
                    [
                        'label' => 'field.hide.player',
                        'value' => false,
                    ]
                ),
                $tab->createElement(
                    'checkbox',
                    'stopOtherPlayers',
                    [
                        'label' => 'field.stop.other.players',
                        'value' => false,
                    ]
                ),
                $tab->createElement(
                    'select',
                    'hover-popup',
                    [
                        'label'        => 'field.hover-popup',
                        'multioptions' => Tool::getHoverPopupOptions(),
                    ]
                ),
                $tab->createElement(
                    'textarea',
                    'hover-plain-text',
                    [
                        'label' => 'field.hover-plain-text',
                        'class' => 'documentwork',
                    ]
                ),
            ]
        );
    }

    protected static function _getOptionsForPdfTool(DefaultForm $tab)
    {
        $tab->addAccordion(
            'position',
            [
                $tab->createElement(
                    'select',
                    'size',
                    [
                        'label'        => 'field.size',
                        'multioptions' => PictureTool::getSizes(),
                    ]
                ),
                $tab->createElement(
                    'select',
                    'position',
                    [
                        'label'        => 'field.position',
                        'multioptions' => PictureTool::getPositions(),
                    ]
                ),
            ]
        );
        $tab->addAccordion(
            'settings',
            [
                $tab->createElement(
                    'checkbox',
                    'display-in-browser',
                    [
                        'label'       => 'field.display-in-browser',
                        'description' => 'description.display-in-browser',
                    ]
                ),
                $tab->createElement(
                    'select',
                    'hover-popup',
                    [
                        'label'        => 'field.hover-popup',
                        'multioptions' => Tool::getHoverPopupOptions(),
                    ]
                ),
                $tab->createElement(
                    'textarea',
                    'hover-plain-text',
                    [
                        'label' => 'field.hover-plain-text',
                        'class' => 'documentwork',
                    ]
                ),
            ]
        );
    }

    protected static function _getOptionsForTextTool(DefaultForm $tab)
    {
        $tab->addAccordion(
            'settings',
            [
                $tab->createElement(
                    'select',
                    'scrollbar-visibility',
                    [
                        'label'        => 'field.scrollbar-visibility',
                        'multioptions' => TextTool::getScrollbarVisibilities(),
                    ]
                ),
            ]
        );
    }

    protected static function _getOptionsForSimpletextTool(DefaultForm $tab)
    {
        $tab->addAccordion(
            'settings',
            [
                $tab->createElement(
                    'select',
                    'scrollbar-visibility',
                    [
                        'label'        => 'field.scrollbar-visibility',
                        'multioptions' => TextTool::getScrollbarVisibilities(),
                    ]
                ),
            ]
        );
    }

    protected static function _getOptionsForCommentTool(DefaultForm $tab)
    {
    }

    protected static function _getOptionsForEmbedTool(DefaultForm $tab)
    {
        $tab->addAccordion(
            'settings',
            [
                $tab->createElement(
                    'checkbox',
                    'loop',
                    [
                        'label' => 'field.loop',
                    ]
                ),
                $tab->createElement(
                    'checkbox',
                    'autoplay',
                    [
                        'label' => 'field.autoplay',
                    ]
                ),
                $tab->createElement(
                    'checkbox',
                    'stopOtherPlayers',
                    [
                        'label' => 'field.stop.other.players',
                        'value' => false,
                    ]
                ),
            ]
        );
    }

    protected static function _getOptionsForPlayTool(DefaultForm $tab, ToolInterface $tool)
    {
        $multiOptions = [];
        if ($tool->getWeave()) {
            $items = ToolRepository::findByGenreAndWeave(
                [
                    ToolFactory::TOOL_VIDEO,
                    ToolFactory::TOOL_AUDIO,
                ],
                $tool->getWeave()
            );
            foreach ($items as $item) {
                if ($item->hasMedia()) {
                    $title = $item->getMedia()->getName() . ' (' . $item->getTitle() . ')';
                } else {
                    $title = '<i>no media</i> (' . $item->getTitle() . ')';
                }
                $multiOptions[$item->getId()] = $title;
            }
            $tab->addAccordion(
                'settings',
                [
                    $tab->createElement(
                        'multiselect',
                        'players',
                        [
                            'label'            => 'field.players',
                            'multioptions'     => $multiOptions,
                            'data-multiselect' => true,
                        ]
                    ),
                    $tab->createElement(
                        'checkbox',
                        'pauseOtherPlayers',
                        [
                            'label' => 'field.pauseOtherPlayers',
                        ]
                    ),
                ]
            );
        }
    }

    protected static function _getOptionsForShapeTool(DefaultForm $tab)
    {
        $tab->addAccordion(
            'style',
            [
                $tab->createElement(
                    'color',
                    'fill-color',
                    [
                        'label' => 'field.fill.color',
                    ]
                ),
                $tab->createElement(
                    'color',
                    'stroke-color',
                    [
                        'label' => 'field.color',
                    ]
                ),
                $tab->createElement(
                    'number',
                    'stroke-width',
                    [
                        'label' => 'field.strength',
                    ]
                ),
                $tab->createElement(
                    'select',
                    'stroke-dash-array',
                    [
                        'label'        => 'field.style',
                        'multioptions' => ShapeTool::getStrokeDashArrays(),
                    ]
                ),
            ]
        );
        $tab->addAccordionContainerOptions(
            [
                'create' => new Zend_Json_Expr('function(event) {
                $("#accordionstyle-container").appendTo($("#frm-media-shapeoptions"));
                $("#frm-media-shapes").find("li.active").trigger("click");
            }'),
            ]
        );
    }

    protected static function _getToolsSupportingResources(array $resources)
    {
        $mediaTypes = [];
        foreach ($resources as $resource) {
            $mediaTypes[] = $resource->getFile()->getMediaType();
        }
        $mediaTypes = array_unique($mediaTypes);

        $tools = [];
        foreach (ToolFactory::getTools() as $tool) {
            $hits = 0;
            foreach ($mediaTypes as $mediaType) {
                $hits += false !== array_search($mediaType, ToolFactory::buildByName($tool)->getSupportedMediaTypes());
            }
            if ($hits == count($mediaTypes)) {
                $tools[] = $tool;
            }
        }
        return $tools;
    }

    protected static function getObjectViewerResourceList(Tool $tool)
    {
        if (!$tool->hasMedia()) {
            return [];
        }

        $resourceList = [];
        foreach ($tool->getMediaList() as $media) {
            $resourceList[$media->getId()] = $media->getMediaContainer()->getThumb()->getExternalUri();
        }
        return $resourceList;
    }

    protected static function getSlideshowResourceList(Tool $tool)
    {
        if (!$tool->hasMedia()) {
            return [];
        }
        $resourceList = [];
        foreach ($tool->getMediaList() as $media) {
            $thumb = $media->getMediaContainer()->getThumb();
            if(!isset($thumb)) {
                $info = [];
                if (PHP_SAPI == 'cli') {
                    $info[] = 'cli: ' . implode(' ', $_SERVER['argv']);
                } else {
                    $info[] = 'url: ' . config('app.url') . $_SERVER['REQUEST_URI'];
                    if (!empty($_SERVER['HTTP_REFERER'])) {
                        $info[] = 'referer: ' . $_SERVER['HTTP_REFERER'];
                    }
                }
                if (\Identity::exists()) {
                    $info[] = 'identity: ' . \Identity::getUserId();
                }
                trigger_error(implode(' , ', $info) . "\tMISSING THUMB FOR #". $media->getId(), E_USER_WARNING);
            }
            $resourceList[] = [
                'id'  => $media->getId(),
                'url' => $thumb ? $thumb->getExternalUri() : null,
            ];
        }
        return $resourceList;
    }
}

?>