<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Research;
use Rc\Domain\Model\Research\Exposition\SimpleMediaInterface;

class ProfileEntityFormGenerator extends BasicFormGenerator
{
    /**
     * @param Research $entity
     * @return DefaultForm
     */
    public static function getShare(Research $entity = null)
    {
        $form = parent::getShare($entity);
        $form->setTitle('dialog.' . $entity->getName() . '.share', $entity);

        return $form;
    }

    /**
     * @param Research $entity
     * @return DefaultForm
     */
    public static function getRemove($entity = null)
    {
        $form = parent::getRemove();
        $form->setTitle('dialog.' . $entity->getName() . '.remove');

        return $form;
    }

    /**
     * @param Research $entity
     * @return DefaultForm
     */
    public static function getVersioning(Research $entity)
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.versioning');

        $tab = $form->addTab('snapshots', [], [
            'contentUrl' => '/snapshot/list?research=' . $entity->getId(),
        ]);

        $tab = $form->addTab('help');
        $tab->addElement('html', 'info', [
            'value' => '<p>' . static::_getTranslate()->translate('info.research.versioning') . '</p>',
        ]);

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    /**
     * @param Research $entity
     * @return DefaultForm
     */
    public static function getConnectToPortal(Research $entity)
    {
        $form = new DefaultForm();;
        $form->setTitle('dialog.connect-to-portal');

        $form
            ->addElement('html', 'hint', [
                'value' => '<p>' . static::_getTranslate()->translate('info.research.portal') . '</p>',
            ])
            ->addElement('select', 'partofportalrequestid', [
                'label'        => 'field.assign.to.portal',
                'multioptions' => ['' => ''] + static::_getPortalsWhichAllowsConnecting($entity->getName()),
            ]);

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    /**
     * @param SimpleMediaInterface $simpleMedia
     * @return DefaultForm
     */
    public static function getFileRemove(SimpleMediaInterface $simpleMedia)
    {
        $form = parent::getRemove();
        $form->setTitle('dialog.file.remove', $simpleMedia);

        return $form;
    }

    /**
     * @return DefaultForm
     */
    public static function getChangeLayout()
    {
        $form = new DefaultForm();;
        $form->setTitle('dialog.change-layout');

        $form
            ->addElement('radio', 'layout', [
                'label'        => 'field.layout',
                'multioptions' => ['', '', '', ''],
            ]);

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }
}