<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Portal;
use Rc\Domain\Model\User;
use Rc\Domain\Repository\UserRepository;

class ProfileFormGenerator extends BasicFormGenerator
{

    public static function getEdit(User $user)
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.profile.edit', $user);

        $form->addElement('languageBar', 'language-bar')
            ->addElement('text', 'name', [
                'required' => true,
                'label'    => 'field.name',
                'readonly' => 'readonly',
            ])
            ->addElement('text', 'email', [
                'required'   => true,
                'label'      => 'field.email',
                'validators' => [
                    'EmailAddress',
                    [
                        'NoRecordExists',
                        false,
                        [
                            'adapter' => new UserRepository(),
                            'field'   => 'email',
                            'exclude' => [
                                'field' => 'id',
                                'value' => isset($user) ? $user->getId() : '',
                            ],
                        ],
                    ],
                ],
            ])
            ->addElement('number', 'yearofbirth', [
                'required' => false,
                'label'    => 'field.birthday',
            ])
            ->addElement('multiselect', 'countryresidence', [
                'label'            => 'field.country.residence',
                'multiOptions'     => static::_getCountryList(),
                'data-multiselect' => true,
            ])
            ->addElement('multiselect', 'countrycitizenship', [
                'label'            => 'field.country.citizenship',
                'multiOptions'     => static::_getCountryList(),
                'data-multiselect' => true,
            ])
            ->addElement('text', 'tagline', [
                'label' => 'field.tagline',
            ])
            ->addElement('text', 'keywords', [
                'label'       => 'field.research-interests',
                'data-tagit'  => '',
                'description' => '<span class="form-description-line">' . self::_getTranslate()->translate('field.tagline.description') . '</span>',
            ])
            ->addElement('text', 'affiliation', [
                'label' => 'field.affiliation',
            ])
            ->addMultiLanguageElement('textarea', 'description', [
                'label'       => 'field.description',
                'data-editor' => true,
                'validators'  => [
                    [
                        'StringLength',
                        false,
                        [
                            'max' => 2500,
                        ],
                    ],
                ],
            ]);

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    public static function getPortalMembershipEdit(User $user)
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.profile.editMembership', $user);
        $form->addElement('multiselect', 'portalids', [
            'label'            => 'field.memberOfPortals',
            'multiOptions'     => self::_getPortalList(true, Portal::FEATURE_MEMBERSHIP_SELECTION),
            'data-multiselect' => true,
        ]);

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    public static function getChangePicture(User $user)
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.picture.change', $user);

        $form->addElement('file', 'media', [
            'label'      => 'field.profile.picture',
            'validators' => [
                'IsImage',
            ],
        ]);

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    public static function getChangeCv(User $user)
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.cv.change', $user);

        $form->addElement('file', 'media', [
            'label'      => 'field.profile.cv',
            'required'   => true,
            'validators' => [
                ['MimeType', false, ['application/pdf', 'application/x-pdf']],
            ],
        ]);

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    public static function getRemoveCv(User $user)
    {
        $form = parent::getRemove();
        $form->setTitle('dialog.cv.remove');

        return $form;
    }

    public static function getAddGroup(User $user)
    {
        $form = new DefaultForm();;
        $form->setTitle('dialog.grouping.add');

        $form->addElement('text', 'name', [
            'label'      => 'field.name',
            'required'   => true,
            'validators' => [
                [
                    'Db_NoRecordExists',
                    false,
                    [
                        'table'   => 'rc_profilegroup',
                        'field'   => 'name',
                        'adapter' => static::_getDatabaseAdapter(),
                        'exclude' => Zend_Registry::get('Zend_Db_Adapter')->quoteInto('userid = ?', $user->getId()),
                    ],
                ],
            ],
        ]);

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    public static function getEditGroup()
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.grouping.edit');
        $form->addElement('text', 'name', [
            'required' => true,
            'label'    => 'field.name',
        ]);

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    public static function getRemoveGroup()
    {
        $form = parent::getRemove();
        $form->setTitle('dialog.grouping.remove');
        return $form;
    }

}