<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

class SettingsFormGenerator extends BasicFormGenerator
{

    public static function getEdit()
    {
        $form = new DefaultForm();
        $form->setAction('/settings?type=settings')
            ->addElement('select', 'language', [
                'label'        => 'field.language',
                'required'     => true,
                'multiOptions' => [
                    'en' => 'English',
                ],
            ])
            ->addElement('checkbox', 'announcements', [
                'label'       => 'field.announcements',
                'description' => 'description.announcements',
            ])
            ->addElement('checkbox', 'digest', [
                'label'       => 'field.digest',
                'description' => 'description.digest',
            ])
            ->addElement('checkbox', 'uploadNotification', [
                'label'       => 'field.upload.notification',
                'description' => 'description.upload.notification',
            ])
            ->addElement('checkbox', 'messageNotification', [
                'label'       => 'field.message.notification',
                'description' => 'description.message.notification',
            ]);

        if (Identity::getUser()->isLimitedUser() && !Identity::getUser()->isInvolved()) {
            $form->addElement('checkbox', 'hide', [
                'label'       => 'field.hideProfile',
                'description' => 'description.hideProfile',
            ]);
        }

        $form->addSubmitButton('settings', 'button.submit');

        return $form;
    }

    public static function getChangePassword()
    {
        $form = new DefaultForm();
        $form->setAttrib('autocomplete', 'off');
        $form->setAction('/settings?type=password')
            ->addElement('password', 'password', [
                'label'      => 'field.password',
                'required'   => true,
                'validators' => [
                    ['StringLength', false, [4, 15]],
                ],
            ])
            ->addElement('password', 'password_confirm', [
                'label'      => 'field.password.confirm',
                'required'   => true,
                'validators' => [
                    ['IdenticalField', false, ['password', '&quot;' . static::_getTranslate()->translate('field.password') . '&quot;']],
                ],
            ])
            ->addSubmitButton('password', 'button.submit');

        return $form;
    }

}

?>