<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Research;
use Rc\Domain\Model\Research\Snapshot;
use Rc\Domain\Model\Research\StatusAwareInterface;

class SnapshotFormGenerator extends BasicFormGenerator
{

    /**
     * @return DefaultForm
     */
    public static function getAdd()
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.snapshot.add');

        $form->addElement('text', 'name', [
            'label'    => 'field.name',
            'required' => true,
        ]);

        $form->addCancelButton()->addSubmitButton('create');

        return $form;
    }

    /**
     * @param Snapshot $snapshot
     * @return DefaultForm
     */
    public static function getRestore(Snapshot $snapshot)
    {
        $research = $snapshot->getResearch();

        $form = new DefaultForm();
        $form->setTitle('dialog.snapshot.restore');

        // duplicate
        self::createRestoreTab(
            $form,
            $research,
            'duplicate',
            $snapshot->getResearch()->getTitle() . ' [' . ($snapshot->getName() ? $snapshot->getName() . ' - ' : '') . date('Y-m-d H:i') . ']'
        );

        /**
        // replace
        self::createRestoreTab(
            $form,
            $research,
            'replace'
        );
        */

        if ($research instanceof StatusAwareInterface && $research->getStatus() != StatusAwareInterface::STATUS_PROGRESS) {
            $form->disableTab('replace');
        }

        return $form;
    }

    /**
     * @return DefaultForm
     */
    public static function getRemove()
    {
        $form = parent::getRemove();
        $form->setTitle('dialog.snapshot.remove');

        return $form;
    }

    /**
     * @param DefaultForm $form
     * @param Research    $research
     * @param string      $type
     * @param string|null $title
     * @return DefaultForm
     */
    protected static function createRestoreTab(DefaultForm $form, Research $research, $type, $title = null)
    {
        $tab = $form->addTab($type)
            ->addElement('html', 'info', [
                'value' => '<br/>' . self::translate('info.restore.' . $type),
            ]);

        if (isset($title)) {
            $tab->addElement('text', 'title', [
                'label'    => 'field.title',
                'value'    => $title,
                'required' => true,
            ]);
        }

        if ($research->hasCollaborateurs()) {
            $tab->addElement('radio', 'collaboration', [
                'label'        => 'field.collaboration',
                'multioptions' => [
                    'private'                                           => 'private',
                    'restore'                                           => 'restore',
                    ($type == 'duplicate' ? 'keep_active' : 'keep_all') => 'keep',
                ],
                'listsep'      => '',
                'labelStyle'   => 'min-width:auto;padding-right:25px',
                'value'        => $type == 'duplicate' ? 'private' : 'keep_all',
                'required'     => true,
            ]);
        } else {
            $tab->addElement('html', 'collaboration', [
                'label' => 'field.collaboration',
                'value' => 'private',
                'tag'   => 'span',
                'style' => 'font-style:italic',
            ]);
        }

        if ($research instanceof StatusAwareInterface) {
            $tab->addElement('html', 'status', [
                'label' => 'field.status',
                'value' => self::translate('research.status.progress'),
                'tag'   => 'span',
                'style' => 'font-style:italic',
            ]);
        }

        $tab->addElement('html', 'share', [
            'label' => 'field.access-level',
            'value' => self::translate('field.access.private'),
            'tag'   => 'span',
            'style' => 'font-style:italic',
        ]);

        $tab->addCancelButton()->addSubmitButton('submit', $type);

        return $tab;
    }

}