<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Model\Research\Exposition\Weave\Tool;

class ToolFormGenerator extends ItemFormGenerator
{
    /**
     * @param Tool $tool
     * @param Exposition $exposition
     * @return DefaultForm
     */
    public static function getEditTemplate(Tool $tool, Exposition $exposition)
    {
        $form = parent::getEdit($tool, $exposition);
        $form->setTitle(
            sprintf(
                self::_getTranslate()->translate('dialog.item.template.edit'),
                self::_getTranslate()->translate('tool.' . $tool->getName())
            ),
            $tool
        );

        $form->removeTab('media');
        $form->removeTab('history');

        $form->removeElement('remove');
        self::removeElement($form, 'style', ['left', 'top']);
        switch ($tool->getName()) {
            case ToolFactory::TOOL_OBJECT_VIEWER:
                self::removeElement($form, 'options', ['levels', 'startlevel']);
                break;
        }

        return $form;
    }

    /**
     * @param DefaultForm $form
     * @param string $tabName
     * @param array $elementNames
     */
    protected static function removeElement(DefaultForm $form, $tabName, array $elementNames)
    {
        foreach ($form->getTab($tabName) as $accordion) {
            foreach ($elementNames as $name) {
                $accordion->removeElement($name);
            }
        }
    }
}

?>