<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Model\Research\Exposition\Weave;
use Rc\Domain\Model\Research\Exposition\Weave\Tool;
use Rc\Domain\Model\Research\Exposition\Weave\Tool\PictureTool;

class WeaveFormGenerator extends BasicFormGenerator
{

    public static function getAdd(Exposition $exposition)
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.weave.add');

        $form->addTab('meta')
            ->addMultiLanguageElement('text', 'title', [
                'required' => true,
                'label'    => 'field.title',
            ])
            ->addMultiLanguageElement('textarea', 'description', [
                'label' => 'field.description',
            ])
            ->addElement('text', 'iframe', [
                'label'      => 'field.iframe',
                'info'       => 'If you specify an iframe url then the defined site will be displayed in fullscreen.<br/><b>No weave styling and no tools are displayed anymore!</b>',
                'validators' => [
                    ['Url'],
                ],
            ]);

        $tab = $form->addTab('style');
        $tab->addAccordion('margin', [
            $tab->createElement('number', 'margin-left', [
                'label' => 'field.left',
            ]),
            $tab->createElement('number', 'margin-top', [
                'label' => 'field.top',
            ]),
            $tab->createElement('number', 'margin-right', [
                'label' => 'field.right',
            ]),
            $tab->createElement('number', 'margin-bottom', [
                'label' => 'field.bottom',
            ]),
        ]);
        $tab->addAccordion('padding', [
            $tab->createElement('number', 'padding-left', [
                'label' => 'field.left',
            ]),
            $tab->createElement('number', 'padding-top', [
                'label' => 'field.top',
            ]),
            $tab->createElement('number', 'padding-right', [
                'label' => 'field.right',
            ]),
            $tab->createElement('number', 'padding-bottom', [
                'label' => 'field.bottom',
            ]),
        ]);
        $tab->addAccordion('border', [
            $tab->createElement('select', 'border-style', [
                'label'        => 'field.style',
                'multioptions' => Tool::getBorderStyles(),
            ]),
            $tab->createElement('number', 'border-width', [
                'label' => 'field.strength',
                'min'   => 0,
            ]),
            $tab->createElement('color', 'border-color', [
                'label' => 'field.color',
            ]),
            $tab->createElement('number', 'border-radius', [
                'label' => 'field.radius',
            ]),
        ]);
        $tab->addAccordion('background', [
            $tab->createElement('color', 'background-color', [
                'label' => 'field.color',
            ]),
            $tab->createElement('select', 'background-image-file-id', [
                'label'        => 'field.image',
                'multiOptions' => ['' => ''] + self::_getSupportedFiles(new PictureTool(), $exposition),
            ]),
            $tab->createInlineElement([
                $tab->createElement('select', 'background-image-style', [
                    'label'        => 'field.image.properties',
                    'style'        => 'width:90px',
                    'multioptions' => Tool::getBackgroundImageStyles(),
                ]),
                $tab->createElement('select', 'background-image-position', [
                    'style'        => 'width:112px',
                    'multioptions' => Tool::getBackgroundImagePositions(),
                ]),
                $tab->createElement('select', 'background-image-size', [
                    'style'        => 'width:90px',
                    'multioptions' => Tool::getBackgroundImageSizes(),
                ]),
            ]),
        ]);
        $tab->addAccordion('hyperlink', [
            $tab->createElement('color', 'hyperlink-color', [
                'label' => 'field.hyperlink.color',
            ]),
            $tab->createElement('color', 'hyperlink-visited-color', [
                'label' => 'field.hyperlink.visited.color',
            ]),
            $tab->createElement('color', 'hyperlink-hover-color', [
                'label' => 'field.hyperlink.hover.color',
            ]),
        ]);
        $tab->addAccordion('footnote', [
            $tab->createElement('color', 'footnote-color', [
                'label' => 'field.hyperlink.color',
            ]),
            $tab->createElement('color', 'footnote-visited-color', [
                'label' => 'field.hyperlink.visited.color',
            ]),
            $tab->createElement('color', 'footnote-hover-color', [
                'label' => 'field.hyperlink.hover.color',
            ]),
        ]);
        $tab->addAccordion('enhancedsettings', [
            $tab->createElement('html', 'rawcssinfo', [
                'value' => 'Here you can define custom css rules. <b>Please use this feature with care!</b><br/><br/>
Best practice is to prefix all css rules with "#container-weave " to make sure you don\'t affect site layout. For example: <br/><pre>
#container-weave .tool-text a {
    color: brown;
}
</pre>',
            ]),
            $tab->createElement('textarea', 'raw-css', [
                'style' => 'max-width: 500px; width: 500px',
            ]),
        ]);

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    public static function getEdit(Exposition $exposition, Weave $weave)
    {
        $form = self::getAdd($exposition);
        $form->setTitle('dialog.weave.edit', $weave);

        return $form;
    }

    public static function getRemove()
    {
        $form = parent::getRemove();
        $form->setTitle('dialog.weave.remove');

        return $form;
    }

}