<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Core\FormValuesTrait;
use Rc\Domain\Model\MediaInterface;
use Rc\Domain\Model\Research\Application;

class ApplicationDefaultForm extends DefaultForm
{
    /**
     * @var array
     */
    protected $files = [];

    /**
     * @param Application|FormValuesTrait $entity
     * @param array $forSubForms
     * @param array $attributes
     * @return $this
     */
    public function setDefaultsFrom($entity, $forSubForms = null, array $attributes = [])
    {
        parent::setDefaultsFrom($entity, $forSubForms, $attributes);

        $this->setElementDefault($entity->getMetaData(), 'application_form', 'custom_elements_');
        $this->setElementDefault($entity->getRequiredDocuments(), 'application_file', 'custom_files_');
        $this->setElementDefault($entity->getOptionalDocuments(), 'application_file', 'custom_optional-files_');

        if ($entity->isInReview()) {
            $this->setElementDefault($entity->getReview(Identity::getUser())->getMetaData(), 'review_form',
                'custom_elements_');
        }

        return $this;
    }

    /**
     * @param bool $merged
     * @return array
     */
    public function getValuesIncludingFiles($merged = true)
    {
        $values = $this->getValues($merged);
        if ($merged) {
            $values = $this->normalizeValues($values);
        }

        foreach ($this->files as $tab => $elements) {
            foreach ($elements as $key => $element) {
                $values[$tab][$key] = $element->getMaster()->getFilename();
            }
        }

        if (!$merged) {
            return $values;
        }

        return $this->normalizeValues($values);
    }

    /**
     * @param array $values
     * @return array
     */
    public function normalizeValues(array $values)
    {
        $values['custom'] = ['elements' => [], 'files' => [], 'optionalfiles' => []];
        foreach ($values as $key => $value) {
            if (!$this->isDynamicFormElement($key)) {
                continue;
            }
            list(, $customType, $name) = explode('_', $key, 3);
            $values['custom'][$customType][$name] = $value;
            unset($values[$key]);
        }
        return $values;
    }

    /**
     * @param array $values
     * @param string $tabName
     * @param string $elementPrefix
     */
    protected function setElementDefault(array $values, $tabName, $elementPrefix)
    {
        foreach ($values as $key => $value) {
            $tab = $this->getTab($tabName);
            if (!isset($tab)) {
                continue;
            }

            $element = $tab->getElement($elementPrefix . $key);
            if (!isset($element)) {
                continue;
            }

            // set default value
            $element->setValue($value);

            if (!$element instanceof Zend_Form_Element_File || !$value instanceof MediaInterface) {
                continue;
            };
            $element->setAttrib('data-resource', $value->getUuid());
            $element->setDescription(
                '<a href="' . $value->getExternalUri() . '" target="_blank">open</a> 
                <a href="javascript:changeMediaFile($(\'#' . $element->getId() . '\'), $(this));">change</a> 
                <a href="javascript:deleteMediaFile($(\'#' . $element->getId() . '\'), $(this));">delete</a>'
            );
            $this->files[$tabName][$elementPrefix . $key] = $value;
        }
    }

    /**
     * @param string $name
     * @return bool
     */
    protected function isDynamicFormElement($name)
    {
        return strpos($name, 'custom_') === 0;
    }

}