<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

/**
 * @method ApplicationFormDefaultForm setDefault($name, $value)
 * @method ApplicationFormDefaultForm addElement($element, $name = null, $options = null)
 * @method Zend_Form_Element[] getElements()
 */
class ApplicationFormDefaultForm extends DefaultForm
{
    public function isValid($data = null)
    {
        if (!isset($data)) {
            $data = $this->_getRequest()->getParams();
            if ($this->isType(self::TYPE_TAB)) {
                $key = substr($this->getName(), strlen(self::TYPE_TAB));
                if (isset($data[$key])) {
                    $data = $data[$key];
                }
            }
        }

        $accordions = [];
        $prototype = $this->getTab('prototype');
        if (isset($data['elements']['element']) && is_array($data['elements']['element'])) {
            $this->getTab('elements')->removeSubForm(self::PREFIX_ACCORDION_NAME . '-placeholder');
            foreach ($data['elements']['element'] as $key => $element) {
                $accordion = $this->appendFormElement($element['element'], $key);
                $accordion->setElementsBelongTo('elements[element][' . $key . ']');
                $accordions[] = $accordion;
            }
        }

        $this->removeTab('prototype');

        $valid = parent::isValid($data);

        $this->addSubForm($prototype, 'tab-prototype');
        foreach ($accordions as $key => $accordion) {
            $accordion->setElementsBelongTo('element[' . $key . ']');
        }
        return $valid;
    }

    /**
     * @param string $type
     * @param string $key
     * @return DefaultForm
     */
    public function appendFormElement($type, $key)
    {
        $accordion = clone $this->getTab('prototype')->getAccordion($type);
        $accordion->setName($accordion->getName() . '-' . $key);
        $accordion->setElementsBelongTo('element[' . $key . ']');
        $accordion->getElement('identifier')->setAttrib('disabled', 'disabled');

        $this->getTab('elements')->addSubForm($accordion, self::PREFIX_ACCORDION_NAME . '-' . $key);

        return $accordion;
    }

}