<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

class AuthorizationPlugin extends Zend_Controller_Plugin_Abstract
{

    private static $isAllowedWithoutAuthorization = array(
        'index' => '*',
        'session' => '*',
        'error' => '*',
        'statistic' => '*',
        'portal' => '*',
        'portals'     => '*',
        'feed'        => '*',
        'view'        => '*',
        'metrics'     => '*',
        'shared'      => '*',
        'research'    => array('search'),
        'work'        => array('search'),
        'project'     => array('search'),
        'degree'      => array('search'),
        'user'        => array('search'),
        'profile'     => array('index', 'show', 'show-work', 'show-person', 'show-exposition', 'show-degree', 'show-project', 'download-media', 'load-archive-urls'),
        'weave'       => array('thumb'),
        'zencoder'    => '*',
        'text-editor' => array('simple-media-resource', 'simple-media-thumb'),
    );
    private static $isRestrictedForNonAdmins = array(
        'admin' => '*',
    );
    private static $isRestrictedForNonSuperAdmins = array(
        'admin' => array(
            'database' => '*',
	        'misc' => '*',
        ),
        'import' => '*',
        'import-polifonia' => '*',
    );

    public function preDispatch(Zend_Controller_Request_Abstract $request)
    {
        // Portal, Editor & Co.
        if (!$this->_isAuthorized()) {
            if (!$this->_isAllowedWithoutAuthorization($request)) {
                $this->_forwardToLoginAction($request);
            }

            // Admin-Interface
        } else if (!$this->_isAdmin() && $this->_isRestrictedForNonAdmins($request)) {
            $request->setControllerName('portal');
            $request->setActionName(null);
            $request->setModuleName(null);
            $request->clearParams();

            // Super-Admin-Interface
        } else if (!$this->_isSuperAdmin() && $this->_isRestrictedForNonSuperAdmins($request)) {
            $request->setControllerName('admin');
            $request->setActionName(null);
            $request->setModuleName(null);
            $request->clearParams();
        }
    }

    private function _isRestrictedForNonAdmins(Zend_Controller_Request_Abstract $request)
    {
        if (!isset(self::$isRestrictedForNonAdmins[$request->getControllerName()])) {
            return false;
        }

        $entry = self::$isRestrictedForNonAdmins[$request->getControllerName()];
        $action = preg_split('~\-~', $request->getActionName());

        if (!isset($entry[$action[0]])) {
            return false;
        }

        $entry = $entry[$action[0]];

        if (count($action) == 1) {
            $action[1] = $request->getActionName();
        }

        if (is_array($entry)) {
            $isRestricted = in_array($action[1], $entry);
        } else if ($entry == '*') {
            $isRestricted = true;
        } else {
            $isRestricted = false;
        }

        return $isRestricted;
    }

    private function _isRestrictedForNonSuperAdmins(Zend_Controller_Request_Abstract $request)
    {
        if (!isset(self::$isRestrictedForNonSuperAdmins[$request->getControllerName()])) {
            return false;
        }

        $entry = self::$isRestrictedForNonSuperAdmins[$request->getControllerName()];
        $action = preg_split('~\-~', $request->getActionName());

        if (!isset($entry[$action[0]])) {
            return false;
        }

        $entry = $entry[$action[0]];

        if (count($action) == 1) {
            $action[1] = $request->getActionName();
        }

        if (is_array($entry)) {
            $isRestricted = in_array($action[1], $entry);
        } else if ($entry == '*') {
            $isRestricted = true;
        } else {
            $isRestricted = false;
        }

        return $isRestricted;
    }

    private function _isAllowedWithoutAuthorization(Zend_Controller_Request_Abstract $request)
    {
        if (isset(self::$isAllowedWithoutAuthorization[$request->getControllerName()])) {
            $entry = self::$isAllowedWithoutAuthorization[$request->getControllerName()];

            if (is_array($entry)) {
                $isAllowed = in_array($request->getActionName(), $entry);
            } else if ($entry == '*') {
                $isAllowed = true;
            } else {
                $isAllowed = false;
            }
        } else {
            $isAllowed = false;
        }
        return $isAllowed;
    }

    private function _forwardToLoginAction(Zend_Controller_Request_Abstract $request)
    {
        if ($request->isXmlHttpRequest() || $request->getParam('iframe-submit')) {
            $this->_sendUnauthorizedHeader();
            echo $this->_getLoginForm();
            exit();
        } else {
            $request->setControllerName('portal');
            $request->setActionName('login');
            $request->setModuleName(null);
        }
    }

    private function _sendUnauthorizedHeader()
    {
        header($_SERVER["SERVER_PROTOCOL"] . " 401 Unauthorized");
    }

    private function _getLoginForm()
    {
        return LoginFormGenerator::get()->render();
    }

    private function _isSuperAdmin()
    {
        return (Identity::exists() && Identity::getUser()->isSuperAdmin());
    }

    private function _isAdmin()
    {
        return (Identity::exists() && Identity::getUser()->isAdmin());
    }

    private function _isAuthorized()
    {
        return Identity::exists();
    }

}

?>