<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

/**
 * @method static Log errPageNotFound(... $args)
 * @method static Log errApplicationError(... $args)
 * @method static Log infoUpload(... $args)
 * @method static Log debugMedia(... $args)
 * @method static Log debugImport(... $args)
 * @method static Log debugNewsletter(... $args)
 */
class Log
{
    const EMERG = 0;  // Emergency: system is unusable
    const ALERT = 1;  // Alert: action must be taken immediately
    const CRIT = 2;  // Critical: critical conditions
    const ERR = 3;  // Error: error conditions
    const WARN = 4;  // Warning: warning conditions
    const NOTICE = 5;  // Notice: normal but significant condition
    const INFO = 6;  // Informational: informational messages
    const DEBUG = 7;  // Debug: debug messages

    /**
     * @var array
     */
    private static $instances = [];

    /**
     * @param string $method
     * @param array $args
     */
    public static function __callStatic($method, array $args)
    {
        list($logType, $fileName) = preg_split('/([a-z]*)([A-Z].*)/', $method, null, PREG_SPLIT_DELIM_CAPTURE | PREG_SPLIT_NO_EMPTY);
        self::_getLogger(strtolower($fileName))->log(implode("\t", $args), constant('Zend_Log::' . strtoupper($logType)));
    }

    /**
     * @param string $name
     * @return Zend_Log
     */
    private static function _getLogger($name)
    {
        if (!isset(self::$instances[$name])) {
            self::$instances[$name] = new Zend_Log(new Zend_Log_Writer_Stream(log_path('application.log')));
        }
        return self::$instances[$name];
    }

}

?>