<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

class UserGroups
{
    const TYPE_KEY = 'key';
    const TYPE_ENTITY_ID = 'id';

    /**
     * @var Zend_Session_Namespace
     */
    protected static $session;

    protected $userGroups;

    protected $tempGroups = [];

    protected $finalGroups;

    protected $domain;

    protected $config;

    protected $uid;

    public function __construct($config, $domain = 'rc')
    {
        $this->config = $config;
        $this->domain = $domain;

        self::$session = self::$session ?? new Zend_Session_Namespace('rc.usergroups');
        self::$session->read_groups = self::$session->read_groups ?? [];
        self::$session->edit_groups = self::$session->edit_groups ?? [];
        self::$session->groups = self::$session->groups ?? [];
    }

    public function login($uid)
    {
        $this->uid = $uid;
        $this->finalGroups = null;
    }

    public function isLoggedIn()
    {
        return $this->uid !== false;
    }

    public function getUID()
    {
        return $this->domain . '/' . $this->uid;
    }

    public function getGroups($type = null)
    {
        if (!$this->isLoggedIn()) {
            throw new \Exception("not logged in");
        }

        $groups = $this->_getGroups();
        if (!isset($type)) {
            return $groups;
        }

        if ($type == self::TYPE_KEY) {
            return array_keys($groups);
        }

        if ($type == self::TYPE_ENTITY_ID) {
            return array_values(array_filter(array_map(function ($value) {
                return (int)preg_replace('~[^\d]~i', '', $value);
            }, array_keys($groups))));
        }

        throw new InvalidArgumentException('The given type ' . $type . ' is not supported!');
    }

    public function getTemporaryGroups($type = null)
    {
        $groups = $this->tempGroups;
        if (!isset($type)) {
            return $groups;
        }

        if ($type == self::TYPE_KEY) {
            return array_keys($groups);
        }

        if ($type == self::TYPE_ENTITY_ID) {
            return array_values(array_filter(array_map(function ($value) {
                return (int)preg_replace('~[^\d]~i', '', $value);
            }, array_keys($groups))));
        }

        throw new InvalidArgumentException('The given type ' . $type . ' is not supported!');
    }

    /**
     * @param int|array $id
     */
    public function addTemporaryReadGroupFor($id)
    {
        foreach ((array)$id as $group) {
            $this->_addTemporaryGroup($group . '_reader');
        }
    }

    /**
     * @param string $id
     * @return bool
     */
    public function hasTemporaryReadGroupFor($id)
    {
        $group = $this->domain . '/' . $id . '_reader';
        return !empty($this->tempGroups[$group]);
    }

    /**
     * @param int|array $id
     */
    public function addSessionReadGroupFor($id)
    {
        foreach ((array)$id as $group) {
            self::$session->read_groups[] = $group;
        }

        $this->addTemporaryReadGroupFor($id);
    }

    /**
     * @param int|array $id
     */
    public function addTemporaryEditGroupFor($id)
    {
        foreach ((array)$id as $group) {
            $this->addTemporaryReadGroupFor($group);
            $this->_addTemporaryGroup($group . '_writer');
        }
    }

    /**
     * @param string $id
     * @return bool
     */
    public function hasTemporaryEditGroupFor($id)
    {
        $group = $this->domain . '/' . $id . '_writer';
        return !empty($this->tempGroups[$group]);
    }

    public function addGroupsFromSession()
    {
        foreach (self::$session->read_groups ?? [] as $group) {
            $this->addTemporaryReadGroupFor($group);
        }
        foreach (self::$session->edit_groups ?? [] as $group) {
            $this->addTemporaryEditGroupFor($group);
        }
        foreach (self::$session->groups ?? [] as $group) {
            $this->_addTemporaryGroup($group);
        }
    }

    /**
     * @param string $group
     */
    protected function _addTemporaryGroup($group)
    {
        $group = $this->domain . '/' . $group;
        $this->tempGroups[$group] = true;
        $this->finalGroups = null;
    }

    /**
     * @return array
     */
    protected function _getGroups()
    {
        if (isset($this->finalGroups)) {
            return $this->finalGroups;
        }

        if (Identity::exists()) {
            if (!isset($this->userGroups)) {
                $this->userGroups = [];
                foreach (Identity::getUser()->getGroups() as $group) {
                    $this->userGroups[$this->domain . '/' . $group] = true;
                }
            }
            $retVal = $this->userGroups + $this->tempGroups;
        } else {
            $retVal = $this->tempGroups;
        }

        $this->finalGroups = [];
        foreach (array_keys($retVal) as $group) {
            $this->finalGroups[str_replace('rc/', '', strpos($group, '_') ? preg_replace('~^[^\d]+~', '', $group) : $group)] = true;
        }
        return $this->finalGroups;
    }

}

?>