<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

class My_Validate_GreaterThan extends Zend_Validate_GreaterThan
{

    const NOT_EQUAL_OR_GREATER = 'notEqualOrGreaterThan';

    /**
     * @var array
     */
    protected $_messageTemplates = [
        self::NOT_GREATER          => "'%value%' is not greater than '%min%'",
        self::NOT_EQUAL_OR_GREATER => "'%value%' is not equal or greater than '%min%'",
    ];

    /**
     * @var bool
     */
    protected $_inclusive = false;

    /**
     * Sets validator options
     *
     * @param  mixed|Zend_Config $config
     * @throws Zend_Validate_Exception
     */
    public function __construct($config)
    {
        parent::__construct($config);

        if ($config instanceof Zend_Config) {
            $config = $config->toArray();
        }

        if (is_array($config) && array_key_exists('inclusive', $config)) {
            $this->setInclusive($config['inclusive']);
        }
    }

    /**
     * Returns the inclusive option
     *
     * @return mixed
     */
    public function getInclusive()
    {
        return $this->_inclusive;
    }

    /**
     * Sets the inclusive option
     *
     * @param  mixed $inclusive
     * @return Zend_Validate_GreaterThan Provides a fluent interface
     */
    public function setInclusive($inclusive)
    {
        $this->_inclusive = $inclusive;
        return $this;
    }

    /**
     * @param  mixed $value
     * @return boolean
     */
    public function isValid($value)
    {
        if (!$this->_inclusive) {
            return parent::isValid($value);
        }

        $this->_setValue($value);

        if ($this->_min > $value) {
            $this->_error(self::NOT_EQUAL_OR_GREATER);
            return false;
        }
        return true;
    }

}
