<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

class ShapeFactory
{

    static protected $shapeList = [
        'rect'           => '<rect width="{$width}" height="{$height}" fill="{$fill}" stroke="{$strokeColor}" stroke-width="{$strokeWidth}" stroke-dashArray="{$strokeDashArray}"></rect>',
        'circle'         => '<ellipse cx="{$centerX}" cy="{$centerY}" rx="{$radiusX}" ry="{$radiusY}" fill="{$fill}" stroke="{$strokeColor}" stroke-width="{$strokeWidth}" stroke-dashArray="{$strokeDashArray}"></ellipse>',
        'line'           => '<line x1="{$left}" y1="{$top}" x2="{$right}" y2="{$bottom}" stroke="{$strokeColor}" stroke-width="{$strokeWidth}" stroke-dashArray="{$strokeDashArray}"></line>',
        'verticalLine'   => '<line x1="{$left}" y1="{$top}" x2="{$right}" y2="{$bottom}" stroke="{$strokeColor}" stroke-width="{$strokeWidth}" stroke-dashArray="{$strokeDashArray}"></line>',
        'arrowLeft'      => '<line x1="{$left}" y1="{$topMinusOffset}" x2="{$head}" y2="{$bottom}" stroke="{$strokeColor}" stroke-width="{$strokeWidth}" transform="rotate(45,{$left},{$top})"></line>
                      <line x1="{$left}" y1="{$top}" x2="{$right}" y2="{$bottom}" stroke="{$strokeColor}" stroke-width="{$strokeWidth}" stroke-dashArray="{$strokeDashArray}"></line>
                      <line x1="{$left}" y1="{$topPlusOffset}" x2="{$head}" y2="{$bottom}" stroke="{$strokeColor}" stroke-width="{$strokeWidth}" transform="rotate(315,{$left},{$top})"></line>',
        'arrowUp'        => '<line x1="{$leftPlusOffset}" y1="{$top}" x2="{$right}" y2="{$head}" stroke="{$strokeColor}" stroke-width="{$strokeWidth}" transform="rotate(45,{$left},{$top})"></line>
                      <line x1="{$left}" y1="{$top}" x2="{$right}" y2="{$bottom}" stroke="{$strokeColor}" stroke-width="{$strokeWidth}" stroke-dashArray="{$strokeDashArray}"></line>
                      <line x1="{$leftMinusOffset}" y1="{$top}" x2="{$right}" y2="{$head}" stroke="{$strokeColor}" stroke-width="{$strokeWidth}" transform="rotate(315,{$left},{$top})"></line>',
        'arrowLeftRight' => '<line x1="{$left}" y1="{$top}" x2="{$head}" y2="{$bottom}" stroke="{$strokeColor}" stroke-width="{$strokeWidth}" stroke-dashArray="{$strokeDashArray}" transform="rotate(45,{$left},{$top})"></line>
                      <g transform="rotate(180,25,25)"><line x1="{$left}" y1="{$top}" x2="{$head}" y2="{$bottom}" stroke="{$strokeColor}" stroke-width="{$strokeWidth}" stroke-dashArray="{$strokeDashArray}" transform="rotate(45,{$left},{$top})"></line></g>
                      <line x1="{$left}" y1="{$top}" x2="{$right}" y2="{$bottom}" stroke="{$strokeColor}" stroke-width="{$strokeWidth}" stroke-dashArray="{$strokeDashArray}"></line>
                      <line x1="{$left}" y1="{$top}" x2="{$head}" y2="{$bottom}" stroke="{$strokeColor}" stroke-width="{$strokeWidth}" stroke-dashArray="{$strokeDashArray}" transform="rotate(315,{$left},{$top})"></line>
                      <line x1="{$left}" y1="{$top}" x2="{$head}" y2="{$bottom}" stroke="{$strokeColor}" stroke-width="{$strokeWidth}" stroke-dashArray="{$strokeDashArray}" transform="rotate(315,{$left},{$top})"></line>',

    ];

    static public function getAvailableShapes()
    {
        return [
            'rect',
            'circle',
            'line',
            'verticalLine',
            'arrowLeft',
            'arrowUp',
            'arrowRight',
            'arrowDown',
            //'arrowLeftRight',
        ];
    }

    static public function createShapeByType($type, $width, $height, $options = [])
    {
        if (!$type) {
            return null;
        }

        $method = 'create' . ucfirst($type);
        return self::wrapShapeWithSvgElement(self::$method($width, $height, $options), $width, $height);
    }

    static protected function createRect($width, $height, $options = [])
    {
        $options += [
            'fill'   => 'black',
            'stroke' => [
                'color'     => 'black',
                'width'     => 3,
                'dashArray' => null,
            ],
        ];

        $searchAndReplace = [
            '$width'           => $width,
            '$height'          => $height,
            '$fill'            => self::getColor($options['fill']),
            '$strokeColor'     => self::getColor($options['stroke']['color']),
            '$strokeWidth'     => $options['stroke']['width'],
            '$strokeDashArray' => $options['stroke']['dashArray'],
        ];

        return self::replaceVars($searchAndReplace, self::$shapeList['rect']);
    }

    static protected function createCircle($width, $height, $options = [])
    {
        $options = $options + [
                'fill'   => 'black',
                'stroke' => [
                    'color'     => 'black',
                    'width'     => 3,
                    'dashArray' => null,
                ],
            ];

        $searchAndReplace = [
            '$centerX'         => $width / 2,
            '$centerY'         => $height / 2,
            '$radiusX'         => $width / 2 - $options['stroke']['width'],
            '$radiusY'         => $height / 2 - $options['stroke']['width'],
            '$fill'            => self::getColor($options['fill']),
            '$strokeColor'     => self::getColor($options['stroke']['color']),
            '$strokeWidth'     => $options['stroke']['width'],
            '$strokeDashArray' => $options['stroke']['dashArray'],
        ];

        return self::replaceVars($searchAndReplace, self::$shapeList['circle']);
    }

    static protected function createLine($width, $height, $options = [])
    {
        $options += [
            'stroke' => [
                'color'     => 'black',
                'width'     => 3,
                'dashArray' => null,
            ],
        ];

        $searchAndReplace = [
            '$left'            => 0,
            '$top'             => round($height / 2),
            '$right'           => $width,
            '$bottom'          => round($height / 2),
            '$strokeColor'     => self::getColor($options['stroke']['color']),
            '$strokeWidth'     => $options['stroke']['width'],
            '$strokeDashArray' => $options['stroke']['dashArray'],
        ];

        return self::replaceVars($searchAndReplace, self::$shapeList['line']);
    }

    static protected function createVerticalLine($width, $height, $options = [])
    {
        $options += [
            'stroke' => [
                'color'     => 'black',
                'width'     => 3,
                'dashArray' => null,
            ],
        ];

        $searchAndReplace = [
            '$left'            => round($width / 2),
            '$top'             => 0,
            '$right'           => round($width / 2),
            '$bottom'          => $height,
            '$strokeColor'     => self::getColor($options['stroke']['color']),
            '$strokeWidth'     => $options['stroke']['width'],
            '$strokeDashArray' => $options['stroke']['dashArray'],
        ];

        return self::replaceVars($searchAndReplace, self::$shapeList['line']);
    }

    static protected function createArrowLeft($width, $height, $options = [])
    {
        $options += [
            'stroke' => [
                'color'     => 'black',
                'width'     => 3,
                'dashArray' => null,
            ],
        ];

        $searchAndReplace = [
            '$left'            => 0,
            '$top'             => $height / 2,
            '$topPlusOffset'   => $height / 2 + 1,
            '$topMinusOffset'  => $height / 2 - 1,
            '$right'           => $width,
            '$bottom'          => $height / 2,
            '$head'            => $height / 2 * sqrt(2),
            '$strokeColor'     => self::getColor($options['stroke']['color']),
            '$strokeWidth'     => $options['stroke']['width'],
            '$strokeDashArray' => $options['stroke']['dashArray'],
            '$transform'       => 'scale(1,' . ($height / 100) . ')',
        ];

        return self::replaceVars($searchAndReplace, self::$shapeList['arrowLeft']);
    }

    static protected function createArrowUp($width, $height, $options = [])
    {
        $options += [
            'stroke' => [
                'color'     => 'black',
                'width'     => 3,
                'dashArray' => null,
            ],
        ];

        $searchAndReplace = [
            '$left'            => $width / 2,
            '$leftPlusOffset'  => $width / 2 + 1,
            '$leftMinusOffset' => $width / 2 - 1,
            '$top'             => 0,
            '$right'           => $width / 2,
            '$bottom'          => $height,
            '$head'            => $width / 2 * sqrt(2),
            '$strokeColor'     => self::getColor($options['stroke']['color']),
            '$strokeWidth'     => $options['stroke']['width'],
            '$strokeDashArray' => $options['stroke']['dashArray'],
            '$transform'       => 'scale(' . ($width / 100) . ',1)',
        ];

        return self::replaceVars($searchAndReplace, self::$shapeList['arrowUp']);
    }

    static protected function createArrowRight($width, $height, $options = [])
    {
        return '<g transform="rotate(180,' . ($width / 2) . ',' . ($height / 2) . ')">' . self::createArrowLeft($width, $height, $options) . '</g>';
    }

    static protected function createArrowDown($width, $height, $options = [])
    {
        return '<g transform="rotate(180,' . ($width / 2) . ',' . ($height / 2) . ')">' . self::createArrowUp($width, $height, $options) . '</g>';
    }

    static protected function createArrowLeftRight($width, $height, $options = [])
    {
        $options += [
            'stroke' => [
                'color'     => 'black',
                'width'     => 3,
                'dashArray' => null,
            ],
        ];

        $searchAndReplace = [
            '$left'            => 0,
            '$top'             => $height / 2,
            '$topPlusOffset'   => $height / 2 + 1,
            '$topMinusOffset'  => $height / 2 - 1,
            '$right'           => $width,
            '$bottom'          => $height / 2,
            '$head'            => $height / 2 * sqrt(2),
            '$strokeColor'     => self::getColor($options['stroke']['color']),
            '$strokeWidth'     => $options['stroke']['width'],
            '$strokeDashArray' => $options['stroke']['dashArray'],
            '$transform'       => 'scale(1,' . ($height / 100) . ')',
        ];

        return self::replaceVars($searchAndReplace, self::$shapeList['arrowLeftRight']);
    }

    static protected function wrapShapeWithSvgElement($content, $width, $height)
    {
        return '<svg xmlns="http://www.w3.org/2000/svg" width="' . $width . 'px" height="' . $height . 'px">' . $content . '</svg>';
    }

    static protected function getColor($color)
    {
        if (empty($color)) {
            return 'transparent';
        } else {
            return $color;
        }
    }

    /**
     * @param array $searchAndReplace
     * @param string $subject
     * @return string
     */
    static protected function replaceVars(array $searchAndReplace, $subject)
    {
        $searchAndReplaceAttr = [];

        foreach ($searchAndReplace as $key => $value) {
            $searchAndReplaceAttr['{' . $key . '}'] = $value;
        }

        return str_replace(
            array_keys($searchAndReplaceAttr),
            array_values($searchAndReplaceAttr),
            $subject
        );
    }
}

?>