<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Repository\Research\ExpositionRepository;

class Comment extends MySqlEntity
{

    protected $id;

    protected $userid;

    protected $researchid;

    protected $content;

    protected $created;

    protected $deleted;

    protected $deletedBy;

    private $user;

    private $research;

    public function __construct($data = null)
    {
        static::_setDefaultValues();
        parent::__construct($data);
    }

    public function getUserId()
    {
        return $this->userid;
    }

    public function setUserId($userid)
    {
        $this->userid = $userid;
        return $this;
    }

    public function getUser($reload = false)
    {
        if (is_null($this->user) || $reload) {
            $this->user = \Rc\Domain\Repository\UserRepository::findById($this->getUserId());
        }
        return $this->user;
    }

    public function getResearch($reload = false)
    {
        if (is_null($this->research) || $reload) {
            $this->research = ExpositionRepository::getById($this->getResearchId());
        }
        return $this->research;
    }

    public function getResearchId()
    {
        return $this->researchid;
    }

    public function setResearchId($researchid)
    {
        $this->researchid = $researchid;
        return $this;
    }

    public function getContent()
    {
        return $this->removeStyles($this->content);
    }

    public function setContent($content)
    {
        $this->content = $content;
        return $this;
    }

    public function getCreated()
    {
        return $this->_parseDateTimeForGetter($this->created);
    }

    public function setCreated($date)
    {
        $this->created = $this->_parseDateTimeForSetter($date);
        return $this;
    }

    public function getDeleted()
    {
        return $this->_parseDateTimeForGetter($this->deleted);
    }

    public function setDeleted($date)
    {
        $this->deleted = $this->_parseDateTimeForSetter($date);
        return $this;
    }

    public function isDeleted()
    {
        return !is_null($this->_parseDateTimeForGetter($this->deleted));
    }

    public function setDeletedById($deletedById)
    {
        $this->deletedBy = $deletedById;
        return $this;
    }

    public function getDeletedBy()
    {
        if ($this->isDeleted()) {
            $retVal = \Rc\Domain\Repository\UserRepository::findById($this->deletedBy);
        } else {
            $retVal = null;
        }

        return $retVal;
    }

    public function setDeletedBy(\Rc\Domain\Model\User $deletedBy)
    {
        $this->deletedBy = $deletedBy->getId();
        return $this;
    }

    protected function _setDefaultValues()
    {
        $this->setCreated(time());

        return $this;
    }

    protected function removeStyles($content)
    {
        $content = preg_replace('~(style=".*)font-family: [\w,\' ]+(.*")~iU', '$1$2', $content);
        $content = preg_replace('~(style=".*)font-size: [\w,\' ]+(.*")~iU', '$1$2', $content);
        return $content;
    }
}

?>