<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Research;
use Rc\Domain\Model\User;
use Rc\Domain\Repository\PortalRepository;
use Rc\Domain\Repository\ResearchRepository;
use Rc\Domain\Repository\UserRepository;

class EntityChange extends MySqlEntity
{

    const TYPE_SHARE = 'share';
    const TYPE_EDIT = 'edit';
    const TYPE_COMMENT = 'comment';
    const TYPE_PUBLISH = 'publish';

    /**
     * @var int
     */
    protected $userid;

    /**
     * @var int
     */
    protected $entityid;

    /**
     * @var int
     */
    protected $ownerid;

    /**
     * @var int
     */
    protected $publishedinid;

    /**
     * @var int
     */
    protected $connectedtoid;

    /**
     * @var string
     */
    protected $type;

    /**
     * @var int
     */
    protected $accessLevel;

    /**
     * @var int
     */
    protected $created;

    /**
     * @var User
     */
    private $user;

    /**
     * @var User
     */
    private $owner;

    /**
     * @var Research
     */
    private $entity;

    /**
     * @var \Rc\Domain\Model\Portal
     */
    private $publishedin;

    /**
     * @var \Rc\Domain\Model\Portal
     */
    private $connectedto;

    /**
     * @param array $data
     */
    public function __construct($data = null)
    {
        $this->setCreated(time());
        parent::__construct($data);
    }

    /**
     * @return int
     */
    public function getUserId()
    {
        return $this->userid;
    }

    /**
     * @param int $userId
     * @return void
     */
    public function setUserId($userId)
    {
        $this->userid = $userId;
    }

    /**
     * @return int
     */
    public function getEntityId()
    {
        return $this->entityid;
    }

    /**
     * @param int $entityId
     * @return void
     */
    public function setEntityId($entityId)
    {
        $this->entityid = $entityId;
    }

    /**
     * @return int
     */
    public function getOwnerId()
    {
        return $this->ownerid;
    }

    /**
     * @param int $ownerId
     * @return void
     */
    public function setOwnerId($ownerId)
    {
        $this->ownerid = $ownerId;
    }

    /**
     * @return int
     */
    public function getPublishedInId()
    {
        return $this->publishedinid;
    }

    /**
     * @param int $publishedInId
     * @return void
     */
    public function setPublishedInId($publishedInId)
    {
        $this->publishedinid = $publishedInId;
    }

    /**
     * @return int
     */
    public function getConnectedToId()
    {
        return $this->connectedtoid;
    }

    /**
     * @param int $connectedToId
     * @return void
     */
    public function setConnectedToId($connectedToId)
    {
        $this->connectedtoid = $connectedToId;
    }

    /**
     * @return int
     */
    public function getAccessLevel()
    {
        return $this->accessLevel;
    }

    /**
     * @param int $accessLevel
     * @return void
     */
    public function setAccessLevel($accessLevel)
    {
        $this->accessLevel = $accessLevel;
    }

    /**
     * @return string
     */
    public function getType()
    {
        return $this->type;
    }

    /**
     * @param string $type
     * @return void
     */
    public function setType($type)
    {
        $this->type = $type;
    }

    /**
     * @return int
     */
    public function getCreated()
    {
        return $this->_parseDateTimeForGetter($this->created);
    }

    /**
     * @param int $date
     * @return void
     */
    public function setCreated($date)
    {
        $this->created = $this->_parseDateTimeForSetter($date);
    }

    /**
     * @return User
     * @throws Exception
     */
    public function getUser()
    {
        if (!isset($this->user)) {
            $this->user = UserRepository::findById($this->userid);
        }
        return $this->user;
    }

    /**
     * @param User $user
     * @return void
     */
    public function setUser($user)
    {
        $this->user = $user;
        $this->userid = $user->getId();
    }

    /**
     * @return User
     * @throws Exception
     */
    public function getOwner()
    {
        if (!isset($this->owner)) {
            $this->owner = UserRepository::findById($this->ownerid);
        }
        return $this->owner;
    }

    /**
     * @param User $owner
     * @return void
     */
    public function setOwner(User $owner)
    {
        $this->owner = $owner;
        $this->ownerid = $owner->getId();
    }

    /**
     * @return Research
     */
    public function getEntity()
    {
        if (!isset($this->entity)) {
            try {
                $this->entity = ResearchRepository::findById($this->entityid);
            } catch (Exception $e) {
            }
        }
        return $this->entity;
    }

    /**
     * @param Research $entity
     * @return void
     */
    public function setEntity(Research $entity)
    {
        $this->entity = $entity;
        $this->entityid = $entity->getId();
    }

    /**
     * @return \Rc\Domain\Model\Portal
     * @throws Exception
     */
    public function getConnectedTo()
    {
        if (!isset($this->connectedto)) {
            $this->connectedto = PortalRepository::findById($this->connectedtoid);
        }
        return $this->connectedto;
    }

    /**
     * @param \Rc\Domain\Model\Portal $connectedTo
     * @return void
     */
    public function setConnectedTo($connectedTo)
    {
        $this->connectedto = $connectedTo;
        $this->connectedtoid = $connectedTo->getId();
    }

    /**
     * @return \Rc\Domain\Model\Portal
     * @throws Exception
     */
    public function getPublishedIn()
    {
        if (!isset($this->publishedin)) {
            $this->publishedin = PortalRepository::findById($this->publishedinid);
        }
        return $this->publishedin;
    }

    /**
     * @param \Rc\Domain\Model\Portal $publishedIn
     * @return void
     */
    public function setPublishedIn($publishedIn)
    {
        $this->publishedin = $publishedIn;
        $this->publishedinid = $publishedIn->getId();
    }

}