<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Utility\TokenGenerator;

class Newsletter extends MySqlEntity
{

    /**
     * @var int
     */
    protected $id;

    /**
     * @var int
     */
    protected $portalid;

    /**
     * @var int
     */
    protected $userid;

    /**
     * @var int
     */
    protected $testmode;

    /**
     * @var string
     */
    protected $scope;

    /**
     * @var string
     */
    protected $target;

    /**
     * @var string
     */
    protected $subject;

    /**
     * @var string
     */
    protected $content;

    /**
     * @var string
     */
    protected $hashedContent;

    /**
     * @var int
     */
    protected $created;

    /**
     * @var int
     */
    protected $delivery;

    /**
     * @var int
     */
    protected $started;

    /**
     * @var int
     */
    protected $lastUpdated;

    /**
     * @var int
     */
    protected $ended;

    /**
     * @var string
     */
    protected $targetEmails;

    /**
     * @var string
     */
    protected $succeededEmails;

    /**
     * @var string
     */
    protected $skippedEmails;

    /**
     * @var string
     */
    protected $failedEmails;

    /**
     * @var bool
     */
    protected $approved;

    /**
     * @var \Rc\Domain\Model\User
     */
    private $user;

    /**
     * @var \Rc\Domain\Model\Portal
     */
    private $portal;

    /**
     * @return int
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @param int $id
     * @return void
     */
    public function setId($id)
    {
        $this->id = $id;
    }

    /**
     * @return int
     */
    public function getPortalid()
    {
        return $this->portalid;
    }

    /**
     * @param int $portalid
     * @return void
     */
    public function setPortalid($portalid)
    {
        $this->portalid = $portalid;
    }

    /**
     * @return int
     */
    public function getUserid()
    {
        return $this->userid;
    }

    /**
     * @param int $userid
     * @return void
     */
    public function setUserid($userid)
    {
        $this->userid = $userid;
    }

    /**
     * @return \Rc\Domain\Model\User
     */
    public function getUser()
    {
        if (!isset($this->user)) {
            $this->user = \Rc\Domain\Repository\UserRepository::findById($this->userid);
        }

        return $this->user;
    }

    /**
     * @return \Rc\Domain\Model\Portal
     */
    public function getPortal()
    {
        if (!isset($this->portal)) {
            $this->portal = \Rc\Domain\Repository\PortalRepository::findById($this->portalid);
        }

        return $this->portal;
    }

    /**
     * @param \Rc\Domain\Model\Portal $portal
     * @return void
     */
    public function setPortal(\Rc\Domain\Model\Portal $portal)
    {
        $this->portal = $portal;
        $this->portalid = $portal->getId();
    }

    /**
     * @return int
     */
    public function getTestmode()
    {
        return $this->testmode;
    }

    /**
     * @param int $testmode
     * @return void
     */
    public function setTestmode($testmode)
    {
        $this->testmode = $testmode;
    }

    /**
     * @return string
     */
    public function getScope()
    {
        return $this->scope;
    }

    /**
     * @param string $scope
     * @return void
     */
    public function setScope($scope)
    {
        $this->scope = $scope;
    }

    /**
     * @return string
     */
    public function getTarget()
    {
        return $this->target;
    }

    /**
     * @param string $target
     * @return void
     */
    public function setTarget($target)
    {
        $this->target = $target;
    }

    /**
     * @return string
     */
    public function getSubject()
    {
        return $this->subject;
    }

    /**
     * @param string $subject
     * @return void
     */
    public function setSubject($subject)
    {
        $this->subject = $subject;
    }

    /**
     * @param string $content
     */
    public function setMailContent($content)
    {
        $this->setContent(base64_encode($content));
        $this->setHashedContent(md5($content));
    }

    /**
     * @return string
     */
    public function getMailContent()
    {
        return base64_decode($this->getContent());
    }

    /**
     * @return string
     * @internal
     */
    public function getContent()
    {
        return $this->content;
    }

    /**
     * @param string $content
     * @return void
     * @internal
     */
    public function setContent($content)
    {
        $this->content = $content;
    }

    /**
     * @return string
     */
    public function getHashedContent()
    {
        return $this->hashedContent;
    }

    /**
     * @param string $hashedContent
     * @return void
     */
    public function setHashedContent($hashedContent)
    {
        $this->hashedContent = $hashedContent;
    }

    /**
     * @return int
     */
    public function getCreated()
    {
        return $this->_parseDateTimeForGetter($this->created);
    }

    /**
     * @param int $created
     * @return void
     */
    public function setCreated($created)
    {
        $this->created = $this->_parseDateTimeForSetter($created);
    }

    /**
     * @return int
     */
    public function getDelivery()
    {
        return $this->_parseDateTimeForGetter($this->delivery);
    }

    /**
     * @param int $delivery
     * @return void
     */
    public function setDelivery($delivery)
    {
        $this->delivery = $this->_parseDateTimeForSetter($delivery);
    }

    /**
     * @return int
     */
    public function getStarted()
    {
        return $this->_parseDateTimeForGetter($this->started);
    }

    /**
     * @param int $started
     * @return void
     */
    public function setStarted($started)
    {
        $this->started = $this->_parseDateTimeForSetter($started);
    }

    /**
     * @return int
     */
    public function getLastUpdated()
    {
        return $this->_parseDateTimeForGetter($this->lastUpdated);
    }

    /**
     * @param int $lastUpdated
     * @return void
     */
    public function setLastUpdated($lastUpdated)
    {
        $this->lastUpdated = $this->_parseDateTimeForSetter($lastUpdated);
    }

    /**
     * @return int
     */
    public function getEnded()
    {
        return $this->_parseDateTimeForGetter($this->ended);
    }

    /**
     * @param int $ended
     * @return void
     */
    public function setEnded($ended)
    {
        $this->ended = $this->_parseDateTimeForSetter($ended);
    }

    /**
     * @return string
     */
    public function getTargetEmails()
    {
        return $this->targetEmails;
    }

    /**
     * @param string $targetEmails
     * @return void
     */
    public function setTargetEmails($targetEmails)
    {
        $this->targetEmails = $targetEmails;
    }

    /**
     * @return string
     */
    public function getTargetEmailCount()
    {
        if (empty($this->targetEmails)) {
            return 0;
        }

        return substr_count($this->targetEmails, ',') + 1;
    }

    /**
     * @param string $email
     */
    public function addTargetEmail($email)
    {
        $this->targetEmails = trim($this->targetEmails . ',' . $email, ',');
    }

    /**
     * @return string
     */
    public function getSucceededEmails()
    {
        return $this->succeededEmails;
    }

    /**
     * @param string $succeededEmails
     * @return void
     */
    public function setSucceededEmails($succeededEmails)
    {
        $this->succeededEmails = $succeededEmails;
    }

    /**
     * @return string
     */
    public function getSucceededEmailCount()
    {
        if (empty($this->succeededEmails)) {
            return 0;
        }

        return substr_count($this->succeededEmails, ',') + 1;
    }

    /**
     * @param string $email
     */
    public function addSucceededEmail($email)
    {
        $this->succeededEmails = trim($this->succeededEmails . ',' . $email, ',');
    }

    /**
     * @return string
     */
    public function getSkippedEmails()
    {
        return $this->skippedEmails;
    }

    /**
     * @param string $skippedEmails
     * @return void
     */
    public function setSkippedEmails($skippedEmails)
    {
        $this->skippedEmails = $skippedEmails;
    }

    /**
     * @return string
     */
    public function getSkippedEmailCount()
    {
        if (empty($this->skippedEmails)) {
            return 0;
        }

        return substr_count($this->skippedEmails, ',') + 1;
    }

    /**
     * @param string $email
     */
    public function addSkippedEmail($email)
    {
        $this->skippedEmails = trim($this->skippedEmails . ',' . $email, ',');
    }

    /**
     * @return string
     */
    public function getFailedEmails()
    {
        return $this->failedEmails;
    }

    /**
     * @param string $failedEmails
     * @return void
     */
    public function setFailedEmails($failedEmails)
    {
        $this->failedEmails = $failedEmails;
    }

    /**
     * @return string
     */
    public function getFailedEmailCount()
    {
        if (empty($this->failedEmails)) {
            return 0;
        }

        return substr_count($this->failedEmails, ',') + 1;
    }

    /**
     * @param string $email
     */
    public function addFailedEmail($email)
    {
        $this->failedEmails = trim($this->failedEmails . ',' . $email, ',');
    }

    /**
     * @return bool
     */
    public function getApproved()
    {
        return (bool)$this->approved;
    }

    /**
     * @param bool $approved
     * @return void
     */
    public function setApproved($approved)
    {
        $this->approved = $approved;
    }

    /**
     * @param string $recipient
     * @return PHPMailer
     */
    public function generateMailObject($recipient)
    {
        $mail = new PHPMailer(true);
        $mail->Subject = $this->getSubject();
        $mail->IsHTML(true);
        $mail->SetFrom(config('app.email'), config('app.name'));
        $mail->AddAddress($recipient);

        $body = $this->getMailContent();
        if ($this->getTarget() !== 'default') {
            $unsubscribeUrl = config('app.url') . '/portal/unsubscribe?email=' . $recipient
                . '&token=' . TokenGenerator::generate($recipient);
            $unsubscribe = '<p class="unsubscribe" style="text-align: center; font-size: 13px">to unsubscribe this newsletter please use this <a href="' . $unsubscribeUrl . '">link</a>.</p>';

            if (strpos($body, '</body>') !== false) {
                $body = str_replace('</body>', $unsubscribe . '</body>', $body);
            } else {
                $body .= $unsubscribe;
            }
        }
        $mail->msgHTML($body);

        return $mail;
    }
}
