<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Model\User;

class CommentRepository extends MySqlRepository
{

    protected static $orderBy = [
        'field' => 'created',
        'order' => 'ASC',
    ];

    public static function findNotDeleted($page, $limit = null, $orderBy = null)
    {
        return static::filterComments(
            static::_findBy('deleted', [null], $page, $limit, $orderBy)
        );
    }

    public static function findByResearch(Exposition $research, $page, $limit = null, $orderBy = null)
    {
        return static::findByResearchId($research->getId(), $page, $limit, $orderBy);
    }

    public static function findByResearchId($researchid, $page, $limit = null, $orderBy = null)
    {
        return static::filterComments(
            static::_findBy('researchid', $researchid, $page, $limit, $orderBy)
        );
    }

    public static function findByPerson(User $user, $page, $limit = null, $orderBy = null)
    {
        return static::findByPersonId($user->getId(), $page, $limit, $orderBy);
    }

    public static function findByPersonId($personId, $page, $limit = null, $orderBy = null)
    {
        return static::filterComments(
            static::_findBy([
                'userid',
                'deleted',
            ], [
                $personId,
                null,
            ], $page, $limit, $orderBy)
        );
    }

    public static function removeByResearch(Exposition $research)
    {
        return static::removeByResearchId($research->getId());
    }

    public static function removeByResearchId($researchid)
    {
        $result = static::_query('
            DELETE FROM
                ' . static::_getJoinedTables() . '
            WHERE
                researchid = "' . static::_escape($researchid) . '"
        ');

        if (empty($result)) {
            static::_printError();
            return false;
        }

        return (boolean)$result;
    }

    /**
     * @param Comment[] $comments
     * @return Comment[]
     * @throws Exception
     */
    protected static function filterComments($comments)
    {
        if (Identity::exists() && Identity::isAdmin()) {
            return $comments;
        }

        foreach ($comments as $key => $comment) {
            try {
                $research = $comment->getResearch();
            } catch (Exception $exception) {
                unset($comments[$key]);
                continue;
            }

            if (!$research->isStatus(Exposition::STATUS_REVIEW)) {
                continue;
            }

            if (Identity::exists() && $research->isEditor(Identity::getUser())) {
                continue;
            }

            if (!$research->isReviewer($comment->getUser()) || Identity::getUserId() == $comment->getUser()->getId()) {
                continue;
            }

            unset($comments[$key]);
        }

        return array_values($comments);
    }
}

?>