<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */
/**
 * jQuery Accordion View Helper
 *
 * @uses       Zend_Json
 * @package    ZendX_JQuery
 * @subpackage View
 * @copyright  Copyright (c) 2005-2014 Zend Technologies USA Inc. (http://www.zend.com)
 * @license    http://framework.zend.com/license/new-bsd     New BSD License
 */
class My_View_Helper_AccordionContainer extends ZendX_JQuery_View_Helper_AccordionContainer
{

    /**
     * Render Accordion with the currently registered elements.
     *
     * If no arguments are given, the accordion object is returned so that
     * chaining the {@link addPane()} function allows to register new elements
     * for an accordion.
     *
     * @link   http://docs.jquery.com/UI/Accordion
     * @param  string $id
     * @param  array $params
     * @param  array $attribs
     * @return string|ZendX_JQuery_View_Helper_AccordionContainer
     */
    public function accordionContainer($id = null, array $params = array(), array $attribs = array())
    {
        if (0 === func_num_args()) {
            return $this;
        }

        if (!isset($attribs['id'])) {
            $attribs['id'] = $id;
        }

        $html = "";
        if (isset($this->_panes[$id])) {
            foreach ($this->_panes[$id] AS $element) {
                $html .= sprintf($this->getElementHtmlTemplate($params), $element['name'], $element['content']) . PHP_EOL;
            }

            $this->addOnLoad($attribs, $params);

            $html = $this->getAccordionTemplate($attribs, $html);
        }
        return $html;
    }

    /**
     * @param array $params
     * @return string
     */
    protected function getElementHtmlTemplate(array $params = [])
    {

        if (!isset($this->_elementHtmlTemplate)) {
            $this->_elementHtmlTemplate = parent::getElementHtmlTemplate();
            if (!empty($params['sortable'])) {
                $this->_elementHtmlTemplate = '<div>' . $this->_elementHtmlTemplate . '</div>';
            }
            if (!empty($params['deletable'])) {
                $this->_elementHtmlTemplate = vsprintf($this->_elementHtmlTemplate, [
                    '%s<i class="ui-icon ui-icon-trash" onclick="$(this).closest(\'div\').remove()"></i>',
                    '%s',
                ]);
            }
        }

        return $this->_elementHtmlTemplate;
    }

    /**
     * @param array $attribs
     * @param array $params
     */
    protected function addOnLoad(array $attribs, array $params)
    {
        $command = '%s("[id=%s]:last").accordion(%s)';
        if (!empty($params['sortable'])) {
            $command .= '.sortable({
                    axis: "y", 
                    handle: "h3",
                    stop: function() {
                        var counter = 0;
                        $(".ui-accordion-header", %1$s("#%2$s")).each(function(){
                            $(this).html($(this).html().replace(/\d+\./i,++counter + "."));
                        });

                        var counter = 0;
                        $(".ui-accordion-content", %1$s("#%2$s")).each(function(){
                            $("[name]",this).each(function(){
                                $(this).attr("name",$(this).attr("name").replace(/\[\d+\]/i,"["+counter+"]"));
                            });
                            counter++;
                        });
                    }});';
        } else {
            $command .= ';';
        }

        if (!empty($params['sortable'])) {
            $params['header'] = '> div > h3';
        }

        $js = sprintf($command,
            ZendX_JQuery_View_Helper_JQuery::getJQueryHandler(),
            $attribs['id'],
            !empty($params) ? ZendX_JQuery::encodeJson($params) : '{}'
        );
        $this->jquery->addOnLoad($js);
    }
}
