<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Research\Exposition\Weave\Tool\EmbedTool;

class My_View_Helper_EmbedToolDecorator extends My_View_Helper_ToolDecorator
{

    /**
     * @var EmbedTool
     */
    protected $tool;

    /**
     * @param EmbedTool $tool
     * @param string $renderFor
     * @return mixed
     */
    public function embedToolDecorator($tool, $renderFor = self::RENDER_FOR_EDITOR)
    {
        $this->_init($tool, $renderFor);
        return $this->_renderTool();
    }

    /**
     * @param array $options
     * @return string
     */
    public function _getToolContentForEditor($options = [])
    {
        if (empty($this->tool->getPreviewUrl())) {
            return parent::_getToolContentForEditor();
        } else {
            return '<img src="' . $this->tool->getPreviewUrl() . '" width="' . $this->tool->getWidth() . '" height="' . $this->tool->getHeight() . '"/>';
        }
    }

    /**
     * @param array $options
     * @return string
     */
    public function _getToolContentForView($options = [])
    {
        if ($this->tool->isYoutube()) {
            return $this->renderYoutubePlayer();
        } elseif ($this->tool->isSketchfab()) {
            return $this->renderSketchfabPlayer();
        } elseif ($this->tool->isMadek()) {
            return $this->renderMadekPlayer();
        } elseif ($this->tool->isVimeo()) {
            return $this->renderVimeoPlayer();
        } elseif ($this->tool->isSoundcloud()) {
            return $this->renderSoundcloudPlayer();
        } else {
            return '<iframe id="player-' . $this->tool->getId() . '" width="' . $this->tool->getWidth() . '" height="' . $this->tool->getHeight() . '" src="' . $this->tool->getUrlApi() . '?width=' . $this->tool->getWidth() . '&height=' . $this->tool->getHeight() . '" scrolling="0" frameborder="0"></iframe>';
        }
    }

    protected function _showToolInView()
    {
        return true;
    }

    /**
     * @return string
     */
    protected function renderMadekPlayer()
    {
        $urlApi = $this->tool->getUrlApi();
        $urlApi = preg_replace('~(?<=height=)(\d+)~i', $this->tool->getHeight(), $urlApi);
        $urlApi = preg_replace('~(?<=width=)(\d+)~i', $this->tool->getWidth(), $urlApi);
        return '
        <iframe id="player-' . $this->tool->getId() . '" width="' . $this->tool->getWidth() . '" height="' . $this->tool->getHeight() . '" src="' . $urlApi . '"  scrolling="0" frameborder="0"></iframe>';
    }

    /**
     * @return string
     */
    protected function renderSketchfabPlayer()
    {
        return '
        <iframe id="player-' . $this->tool->getId() . '" width="' . $this->tool->getWidth() . '" height="' . $this->tool->getHeight() . '" src="' . $this->tool->getUrlApi() . '" frameborder="0"></iframe>
        <script type="text/javascript" src="https://static.sketchfab.com/api/sketchfab-viewer-1.0.1.js"></script>
        <script type="text/javascript">
        var client = new Sketchfab("1.0.0", document.getElementById("player-' . $this->tool->getId() . '"));
        client.init("' . preg_replace('~.*/models/([^/]*).*~i', '$1', $this->tool->getUrlApi()) . '", {
            autostart: ' . ($this->tool->isAutoPlay() ? 'true' : 'false') . ',
            success: function onSuccess(api){
                api.start();
                api.addEventListener("viewerready", function() {});
            }
        });
        </script>';
    }

    /**
     * @return string
     */
    protected function renderSoundcloudPlayer()
    {
        return '
        <iframe id="player-' . $this->tool->getId() . '" width="' . $this->tool->getWidth() . '" height="' . $this->tool->getHeight() . '" src="' . $this->tool->getUrlApi() . '" scrolling="0" frameborder="0"></iframe>
        <script type="text/javascript" src="https://w.soundcloud.com/player/api.js"></script>
        <script type="text/javascript">
          (function(){
            var playerId = "' . $this->tool->getId() . '";
            var widget = SC.Widget(document.getElementById("player-' . $this->tool->getId() . '"));
            widget.bind(SC.Widget.Events.READY, function() {
                if (' . ($this->tool->isAutoPlay() ? 'true' : 'false') . ') {
                    widget.play();
                }
            }).bind(SC.Widget.Events.PLAY, function() {
                    mediaPlayers[playerId] = true;
                    
                    if (' . ($this->tool->isStopOtherPlayers() ? 'true' : 'false') . ') {
                        pausePlayersExceptOfThis(playerId);
                    }
            }).bind(SC.Widget.Events.PAUSE, function() {
                    mediaPlayers[playerId] = false;
            }).bind(SC.Widget.Events.FINISH, function() {
                    mediaPlayers[playerId] = false;
                if (' . ($this->tool->isLoop() ? 'true' : 'false') . ') {
                    widget.pause();
                    widget.play();
                }
            });
            
            mediaPlayerStopCallbacks[playerId] = function() {
              widget.pause();
            }
          }());
        </script>';
    }

    /**
     * @return string
     */
    protected function renderVimeoPlayer()
    {
        return '
        <iframe id="player-' . $this->tool->getId() . '" width="' . $this->tool->getWidth() . '" height="' . $this->tool->getHeight() . '" src="' . $this->tool->getUrlApi() . '?autoplay=' . ($this->tool->isAutoPlay() ? 'true' : 'false') . '&loop=' . ($this->tool->isLoop() ? 'true' : 'false') . '" scrolling="0" frameborder="0"></iframe>
        <script src="https://player.vimeo.com/api/player.js"></script>
        <script type="text/javascript">
            var playerId = "' . $this->tool->getId() . '";
            var player = new Vimeo.Player("player-" + playerId);
            player.on("play", function() {
                mediaPlayers[playerId] = true;
                
                if (' . ($this->tool->isStopOtherPlayers() ? 'true' : 'false') . ') {
                    pausePlayersExceptOfThis(playerId);
                }
            });
            player.on("pause ended", function() {
                mediaPlayers[playerId] = false;
            });
            
            mediaPlayerStopCallbacks[playerId] = function() {
              player.pause();
            }
        </script>';
    }

    /**
     * @return string
     */
    protected function renderYoutubePlayer()
    {
        return '
        <iframe id="player-' . $this->tool->getId() . '" data-autoplay="' . $this->tool->isAutoPlay() . '" data-loop="' . $this->tool->isLoop() . '" data-stopOtherPlayers="' . $this->tool->isStopOtherPlayers() . '" data-provider="youtube" width="' . $this->tool->getWidth() . '" height="' . $this->tool->getHeight() . '"
  src="' . $this->tool->getUrlApi() . '&enablejsapi=1&origin=' . config('app.url') . '" scrolling="0" frameborder="0"></iframe>
        <script type="text/javascript" src="https://www.youtube.com/player_api"></script>';
    }
}
