<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Media\Dimension;
use Rc\Domain\Model\Media\MediaType;
use Rc\Domain\Model\Media\Resource\AudioResource;
use Rc\Domain\Model\Media\Resource\VideoResource;
use Rc\Domain\Model\Research\MediaContainerInterface;

class My_View_Helper_Enclosure extends Zend_View_Helper_Abstract
{

    /**
     * @var My_View_Helper_SizeMedia
     */
    protected $sizeMedia;

    /**
     * @var Dimension
     */
    protected $dimension;

    public function __construct()
    {
        $this->dimension = new Dimension(640, 480);
    }

    /**
     * @return My_View_Helper_SizeMedia
     */
    public function getSizeMedia()
    {
        if (!isset($this->sizeMedia)) {
            $this->sizeMedia = $this->view->getHelper('sizeMedia');
        }
        return $this->sizeMedia;
    }

    /**
     * @param MediaContainerInterface $mediaContainer
     * @return null|string
     */
    public function enclosure(MediaContainerInterface $mediaContainer)
    {
        if (!$mediaContainer->hasMedia()) {
            return null;
        }

        switch ($mediaContainer->getContentType()) {
            case MediaType::IMAGE:
            case MediaType::PDF:
                $derivative = $mediaContainer->getThumb();
                if (!isset($derivative)) {
                    return null;
                }
                return '<enclosure url="' . htmlentities($this->getSizeMedia()->url($derivative, $this->dimension)) . '" type="' . $derivative->getMimeType() . '"></enclosure>';

            case MediaType::AUDIO:
                $derivative = $mediaContainer->getMedia()->getDerivative(AudioResource::DERIVATIVE_TYPE_MP3);
                if (!isset($derivative)) {
                    return null;
                }
                return '<enclosure url="' . htmlentities($this->getSizeMedia()->url($derivative, $this->dimension)) . '" type="' . $derivative->getMimeType() . '"></enclosure>';

            case MediaType::VIDEO:
                $derivative = $mediaContainer->getMedia()->getDerivative(VideoResource::DERIVATIVE_TYPE_MP4);
                if (!isset($derivative)) {
                    return null;
                }
                return '<enclosure url="' . htmlentities($this->getSizeMedia()->url($derivative, $this->dimension)) . '" type="' . $derivative->getMimeType() . '"></enclosure>';

            default:
                return null;
        }
    }
}
