<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\ModelInterface;
use Rc\Domain\Model\Portal;
use Rc\Domain\Model\Portal\Issue;
use Rc\Domain\Model\Research;
use Rc\Domain\Model\User;

class My_View_Helper_Link extends Zend_View_Helper_Abstract
{

    /**
     * @return $this
     */
    public function link()
    {
        return $this;
    }

    /**
     * @param ModelInterface $entity
     * @param array $params
     * @return string
     */
    public function entityLink(ModelInterface $entity, $attribs = [], $params = [])
    {
        if ($entity instanceof User) {
            return $this->userLink($entity, $attribs, $params);
        }

        if ($entity instanceof Research) {
            return $this->createLink($entity, $entity->getTitle(), $attribs, $params);
        }

        if (!method_exists($this, $entity->getName() . 'Link')) {
            throw new InvalidArgumentException('Object of type "' . get_class($entity) . ') is not supported.');
        }

        return $this->{$entity->getName() . 'Link'}($entity, $attribs, $params);
    }

    /**
     * @param User $user
     * @param array $attribs
     * @param array $params
     * @return string
     */
    public function userLink(User $user, array $attribs = [], array $params = [])
    {
        if (isset($attribs['value'])) {
            $value = $attribs['value'];
            unset($attribs['value']);
        } else {
            $value = $user->getName();
        }

        if ($user->getName() == 'System') {
            return $this->view->htmlElement('span', $value, $attribs, $params);
        } else {
            return $this->view->htmlElement('a', $value, $attribs + [
                    'href'   => $this->view->url()->relationUrl($user, $params),
                    'target' => '_blank',
                ]);
        }
    }

    /**
     * @deprecated
     * @param User $user
     * @param array $attribs
     * @param array $params
     * @return string
     */
    public function personLink(User $user, array $attribs = [], array $params = [])
    {
        return $this->userLink($user, $attribs, $params);
    }

    /**
     * @param Issue $issue
     * @param array $attribs
     * @param array $params
     * @return string
     */
    public function issueLink(Issue $issue = null, array $attribs = [], array $params = [])
    {
        if (!isset($issue)) {
            return null;
        }

        if (isset($attribs['value'])) {
            $value = $attribs['value'];
            unset($attribs['value']);
        } else {
            $value = $issue->getNumber() . '. ' . $issue->getTitle();
        }

        return $this->view->htmlElement('a', $value, $attribs + [
                'href'   => $this->view->url()->issueUrl($issue),
                'target' => '_blank',
            ], $params);
    }

    /**
     * @param Portal $portal
     * @param array $attribs
     * @param array $params
     * @return string
     */
    public function portalLink(Portal $portal, array $attribs = [], array $params = [])
    {
        if (isset($attribs['value'])) {
            $value = $attribs['value'];
            unset($attribs['value']);
        } else {
            $value = $portal->getName();
        }

        if ($portal->getName() == 'System') {
            return $this->view->htmlElement('span', $value, $attribs, $params);
        } else {
            return $this->view->htmlElement('a', $value, $attribs + [
                    'href'   => $this->view->url()->portalUrl($portal),
                    'target' => '_blank',
                ], $params);
        }
    }

    /**
     * @param Research $research
     * @param array $attribs
     * @param array $params
     * @return string
     */
    public function relationLink(Research $research, array $attribs = [], array $params = [])
    {
        $title = $research->getTitle() . ' (' . $this->view->translate('entity.' . $research->getName()) . ')';

        return $this->createLink($research, $title, $attribs, $params);
    }

    /**
     * @param ModelInterface $entity
     * @param string $title
     * @param array $attribs
     * @param array $params
     * @return mixed
     */
    protected function createLink($entity, $title, array $attribs= [], array $params= [])
    {
        return $this->view->htmlElement('a', $title, $attribs + [
                'href'   => str_replace('&amp;', '&', $this->view->url()->relationUrl($entity, $params)),
                'target' => '_blank',
            ]);
    }

}

?>