<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

class My_View_Helper_MaskEmail extends Zend_View_Helper_Abstract
{

    static public function mask($email, $subject)
    {
        $charCode = [];
        $length = strlen($email);
        for ($i = 0; $i < $length; $i++) {
            $charCode[] = ord($email[$i]);
        }

        $maskedEmail = 'String.fromCharCode(' . implode(',', $charCode) . ')';
        if (!empty($subject)) {
            $subject = '?subject=' . self::urlencode($subject);
        }

        return "javascript:location.href='mailto:' + " . $maskedEmail . " + '" . $subject . "'";
    }

    public function maskEmail($email, $subject = null, $innerHtml = null, $attribs = [], $params = [])
    {
        if (!isset($subject)) {
            $subject = $email;
        }
        if (!isset($innerHtml)) {
            $innerHtml = $subject;
        }

        return $this->view->htmlElement(
            'a',
            $innerHtml,
            $attribs + [
                'href' => self::mask($email, $subject),
            ],
            $params
        );
    }

    public function button($email, $subject, $innerHtml, $attribs = [], $params = [])
    {
        return $this->view->htmlElement(
            'button',
            $innerHtml,
            $attribs + [
                'onclick' => self::mask($email, $subject),
            ],
            $params
        );
    }

    static protected function urlencode($in)
    {
        $out = '';
        for ($i = 0; $i < strlen($in); $i++) {
            $hex = dechex(ord($in[$i]));
            if ($hex == '') {
                $out = $out . urlencode($in[$i]);
            } else {
                $out = $out . '%' . ((strlen($hex) == 1) ? ('0' . strtoupper($hex)) : (strtoupper($hex)));
            }
        }
        $out = str_replace('+', '%20', $out);
        $out = str_replace('_', '%5F', $out);
        $out = str_replace('.', '%2E', $out);
        $out = str_replace('-', '%2D', $out);
        return $out;
    }

}

?>