<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Research\Exposition\Weave\Tool\PdfTool;

/**
 * @property PdfTool $tool
 */
class My_View_Helper_PdfToolDecorator extends My_View_Helper_ToolDecorator
{

    /**
     * @var PdfTool
     */
    protected $tool;

    /**
     * @param PdfTool $tool
     * @param string $renderFor
     * @return mixed
     */
    public function pdfToolDecorator($tool, $renderFor = self::RENDER_FOR_EDITOR)
    {
        $this->_init($tool, $renderFor);
        return $this->_renderTool();
    }

    protected function _getToolContentForEditor($options = [])
    {
        return $this->view->getHelper('sizeMedia')->sizeResource($this->getThumb(), $this->tool->getWidth(), $this->tool->getHeight(), [
            'size'     => $this->tool->getSize(),
            'position' => $this->tool->getPosition(),
        ]);
    }

    protected function _getToolContentForView($options = [])
    {
        $mediaContainer = $this->tool->getMediaContainer();
        if (!$mediaContainer->hasMedia()) {
            return $this->getMediaHelper()->getErrorMessage($mediaContainer);
        }

        $content = $this->view->getHelper('sizeMedia')->sizeResource($this->getThumb(), $this->tool->getWidth(), $this->tool->getHeight(), [
            'size'     => $this->tool->getSize(),
            'position' => $this->tool->getPosition(),
            'url'      => $this->getMedia()->getExternalUri(),
        ]);
        $this->addHoverPopupIfNecessary();

        if ($this->tool->isDisplayInBrowser()) {
            return '<object data="' . $this->getMedia()->getExternalUri() . '" type="application/pdf" width="' . $this->tool->getWidth() . '" height="' . $this->tool->getHeight() . '">' . $content . '</object>';
        } else {
            return $content;
        }
    }
}

?>