<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Research\Exposition\Weave\Tool\PictureTool;

/**
 * @property PictureTool $tool
 */
class My_View_Helper_PictureToolDecorator extends My_View_Helper_ToolDecorator
{

    /**
     * @var PictureTool
     */
    protected $tool;

    /**
     * @param PictureTool $tool
     * @param string $renderFor
     * @return string
     */
    public function pictureToolDecorator(PictureTool $tool, $renderFor = self::RENDER_FOR_EDITOR)
    {
        $this->_init($tool, $renderFor);
        return $this->_renderTool();
    }

    /**
     * @param array $options
     * @return string|null
     */
    public function _getToolContentForView($options = [])
    {
        $mediaEntity = $this->getMedia();
        if (isset($mediaEntity)) {
            $thumb = $this->getMedia()->getBestFittingSplashScreen(PictureTool::HIGH_QUALITY_WIDTH, PictureTool::HIGH_QUALITY_HEIGHT);
        }

        if ($this->tool->isOpenPictureInPopup() && !empty($thumb)) {
            $this->attributes['href'] = $thumb->getExternalUri();
        }
        if ($this->tool->getOpenLink() != '') {
            $this->attributes['data-follow-link'] = $this->tool->getOpenLink();
        }
        $this->addHoverPopupIfNecessary();

        return parent::_getToolContentForView([
            'aspectRatio' => $this->tool->isFixAspectRatio() && !empty($thumb) ? $thumb->getAspectRatio() : null,
        ]);
    }

    /**
     * @param array $options
     * @return string|null
     */
    protected function _getToolContentForEditor($options = [])
    {
        $mediaEntity = $this->getMedia();
        if (isset($mediaEntity)) {
            $thumb = $this->getMedia()->getBestFittingSplashScreen(PictureTool::HIGH_QUALITY_WIDTH, PictureTool::HIGH_QUALITY_HEIGHT);
        }

        return $this->getMediaHelper()->sizeMedia(
            $this->tool->getMediaContainer(),
            $this->tool->getWidth(),
            $this->tool->getHeight(),
            [
                'size'        => $this->tool->getSize(),
                'position'    => $this->tool->getPosition(),
                'aspectRatio' => $this->tool->isFixAspectRatio() && !empty($thumb) ? $thumb->getAspectRatio() : null,
                'raw'         => true,
            ] + ($this->tool->hasMedia() ? $this->tool->getMedia()->getOptions() : [])
        );
    }

}

?>