<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Research\Exposition\Weave\Tool\PlayTool;

/**
 * @property PlayTool $tool
 */
class My_View_Helper_PlayToolDecorator extends My_View_Helper_ToolDecorator
{

    public function playToolDecorator(PlayTool $tool, $renderFor = self::RENDER_FOR_EDITOR)
    {
        $this->_init($tool, $renderFor);
        return $this->_renderTool();
    }

    protected function _getToolContentForEditor($options = [])
    {
        $firstResource = $this->tool->getMediaForPlaySlide();

        $content = '<div class="content" style="margin-top: -2px">';
        if ($firstResource) {
            switch ($firstResource->getMediaType()) {
                case 'text':
                    $content .= $firstResource->getTextContent();
                    break;
                case 'image':
                    $content .= $this->getMediaHelper()->sizeMedia($firstResource->getMediaContainer(), $this->tool->getWidth(), $this->tool->getHeight(), [
                        'size'     => $this->tool->getSize(),
                        'position' => $this->tool->getPosition(),
                    ]);
                    break;
            }
        }
        $content .= '</div>';

        return $content;
    }

    protected function _getToolContentForView($options = [])
    {
        $playResource = $this->tool->getMediaForPlaySlide();
        $pauseResource = $this->tool->getMediaForPauseSlide();
        $content = null;

        if ($playResource) {
            $content .= '<div class="content" style="margin-top: -2px">';
            switch ($playResource->getMediaType()) {
                case 'text':
                    $content .= $playResource->getTextContent();
                    break;
                case 'image':
                    $content .= $this->getMediaHelper()->sizeMedia($playResource->getMediaContainer(), $this->tool->getWidth(), $this->tool->getHeight(), [
                        'size'     => $this->tool->getSize(),
                        'position' => $this->tool->getPosition(),
                    ]);
                    break;
            }
            $content .= '</div>';
        }

        if ($pauseResource) {
            $content .= '<div class="content" style="margin-top: -2px">';
            switch ($pauseResource->getMediaType()) {
                case 'text':
                    $content .= $pauseResource->getTextContent();
                    break;
                case 'image':
                    $content .= $this->getMediaHelper()->sizeMedia($pauseResource->getMediaContainer(), $this->tool->getWidth(), $this->tool->getHeight(), [
                        'size'     => $this->tool->getSize(),
                        'position' => $this->tool->getPosition(),
                    ]);
                    break;
            }
            $content .= '</div>';
        }

        $this->attributes['data']['players'] = json_encode($this->tool->getPlayers());

        return $content;
    }

    protected function _getToolOptions()
    {
        return [
            'pauseOtherPlayers' => $this->tool->getPauseOtherPlayers(),
        ];
    }
}

?>