<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Media\Resource\AudioResource;
use Rc\Domain\Model\Research\Exposition\SimpleMediaInterface;
use Rc\Domain\Model\Research\Exposition\Weave\Tool\SlideshowTool;

/**
 * @property SlideshowTool $tool
 */
class My_View_Helper_SlideshowToolDecorator extends My_View_Helper_ToolDecorator
{

    public function slideshowToolDecorator(SlideshowTool $tool, $renderFor = self::RENDER_FOR_EDITOR)
    {
        $this->_init($tool, $renderFor);
        return $this->_renderTool();
    }

    protected function _getToolContentForView($options = [])
    {
        $slideshow = [];
        foreach ($this->tool->getMediaList() as $key => $media) {
            $content = $this->getMediaHelper()->sizeMedia($media->getMediaContainer(), $this->tool->getWidth(), $this->tool->getHeight(), [
                'size'     => $this->tool->getSize(),
                'position' => $this->tool->getPosition(),
            ]);

            $attribs = [
                'style'         => [
                    'overflow' => 'hidden',
                    'width'    => $this->tool->getWidth() . 'px',
                    'height'   => $this->tool->getHeight() . 'px',
                ],
                'data-audio'    => $this->_getAudioDerivateUrl($media),
                'data-id'       => $media->getId(),
                'data-lightbox' => 'slideshow-' . $this->tool->getId(),
            ];

            if (!in_array($this->tool->getHoverPopup(), ['', 'plaintext'])) {
                $attribs['data-copyright'] = $this->_replaceAuthorNamesForDoubleBlindReview($media->getCopyrightHolder());
            }

            if ($this->tool->isOpenPictureInPopup() && $this->tool->getPlayOption() == SlideshowTool::PLAY_OPTION_NONE) {
                $attribs['href'] = $media->getMediaContainer()->getThumb()->getExternalUri();
            }
            $slideshow[] = $this->view->htmlElement('li', $content, $attribs, ['render' => false]);
        }

        $this->addHoverPopupIfNecessary();

        return '<ul>' . implode(null, $slideshow) . '</ul>';
    }

    protected function addHoverPopupIfNecessary()
    {
        if ($this->tool->getHoverPopup() == '' || !$this->tool->getFirstResource()) {
            return;
        }

        if ($this->tool->getHoverPopup() != 'plaintext') {
            $this->attributes['data']['copyright'] = 'auto';
        }

        if ($this->tool->getHoverPopup() != 'copyright') {
            $this->attributes['data']['plaintext'] = $this->tool->getHoverPlainText();
        }
    }

    protected function _getToolOptions()
    {
        return [
            'loop'          => $this->tool->isLoop(),
            'crossfade'     => $this->tool->getCrossfade(),
            'timeout'       => $this->tool->getTimeoutDuration(),
            'fade'          => $this->tool->getFadeDuration(),
            'defaultspeed'  => $this->tool->getDefaultSlideSpeed(),
            'speedperslide' => $this->tool->getSlidesSpeed(false),
            'autoplay'      => $this->tool->getPlayOption(),
            'shownavbar'    => (int)$this->tool->isShowNavbar(),
        ];
    }

    /**
     * @param SimpleMediaInterface $media
     * @param string $type
     * @return string|null
     */
    protected function _getAudioDerivateUrl(SimpleMediaInterface $media, $type = AudioResource::DERIVATIVE_TYPE_MP3)
    {
        if (!$this->tool->hasAudio($media)) {
            return null;
        }

        $entity = $this->tool->getAudio($media)->getMedia();
        if (!isset($entity)) {
            return null;
        }

        $type = $entity->getDerivative($type);
        if (!isset($type)) {
            return null;
        }

        return $type->getExternalUri();
    }
}

?>
