<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Research\Exposition\Weave\Tool\TextTool;

/**
 * @property TextTool $tool
 */
class My_View_Helper_TextToolDecorator extends My_View_Helper_ToolDecorator
{

    /**
     * @param TextTool $tool
     * @param string $renderFor
     * @return mixed
     */
    public function TextToolDecorator($tool, $renderFor = self::RENDER_FOR_EDITOR)
    {
        $this->_init($tool, $renderFor);
        $this->attributes['class'] .= ' ' . $this->tool->getScrollbarVisibility();
        if (strpos($this->tool->getTextContent(), 'class="del cts-1"') !== false || strpos($this->tool->getTextContent(), 'class="ins cts-1"') !== false) {
            $this->attributes['class'] .= ' track-changes';
        }
        return $this->_renderTool();
    }

    protected function _getToolContentForView($options = [])
    {
        $retVal = $this->view->text($this->tool->getTextContent());
        $retVal = $this->_removeUserDefinedFootnoteClass($retVal);
        $retVal = $this->_parsePopovers($retVal);

        $research = $this->tool->getWeave()->getExposition();
        if ($research->isPublished() === false || $research->getPublishedDate(false)->getTimestamp() >= 1364462257 || in_array($research->getId(), [21732])) {
            $retVal = $this->_normaliseOutput($retVal);
        }

        return $this->_replaceAuthorNamesForDoubleBlindReview($retVal);
    }

    protected function _getToolContentForEditor($options = [])
    {
        if ($this->tool->getTextAuthor()) {
            $this->attributes['data']['text-author'] = $this->tool->getTextAuthor()->getName();
            $this->attributes['data']['text-modified'] = $this->tool->getTextModified();
        }
        $retVal = $this->view->text($this->tool->getTextContent());
        $retVal = $this->_parsePopovers($retVal);
        $retVal = $this->_removeUserDefinedFootnoteClass($retVal);

        $research = $this->tool->getWeave()->getExposition();
        if ($research->isPublished() === false || $research->getPublishedDate(false)->getTimestamp() >= 1364462257 || in_array($research->getId(), [21732])) {
            $retVal = $this->_normaliseOutput($retVal);
        }

        return $retVal;
    }

    protected function _showToolInView()
    {
        return true;
    }

    protected function _normaliseOutput($content)
    {
        $content = mb_convert_encoding($content, 'HTML-ENTITIES', "UTF-8");
        $content = str_replace('<p></p>', '<p><br/></p>', $content);


        $doc = new DOMDocument();
        @$doc->loadHTML($content);
        $doc->removeChild($doc->firstChild);
        $doc->replaceChild($doc->firstChild->firstChild->firstChild, $doc->firstChild);

        if (false && strpos($_SERVER['HTTP_USER_AGENT'], 'Mac OS') !== false && strpos($_SERVER['HTTP_USER_AGENT'], 'Firefox/') !== false) {
            $lineHeight = '1em';
        } else {
            $lineHeight = '1.25em';
        }

        $this->_childrenWalkRecursive($doc->documentElement->childNodes, $lineHeight);

        return $doc->saveHTML();
    }

    private function _childrenWalkRecursive(DOMNodeList $childNodeList, $lineHeight)
    {
        /** @var DOMNode $childNode */
        foreach ($childNodeList as $childNode) {
            if ($childNode->hasAttributes() === false && $childNode->hasChildNodes() === false) {
                continue;
            }

            foreach ($childNode->attributes as $attribute) {
                if ($attribute->name != 'style') {
                    continue;
                }

                // update line-height for all child nodes
                if (strpos($attribute->value, 'line-height:') !== false) {
                    $lineHeight = preg_replace('~.*line-height: ?([^;]+);?.*~', '\\1', $attribute->value);
                }

                // if no font-size is set, we have nothing to do
                if (strpos($attribute->value, 'font-size:') === false) {
                    continue;
                }

                // if line-height is not specified and font-size is, we have to set line-height
                if (strpos($attribute->value, 'line-height:') === false) {
                    $attribute->value .= 'line-height: ' . $lineHeight . ';';
                }
            }

            if ($childNode->hasChildNodes() === true) {
                $this->_childrenWalkRecursive($childNode->childNodes, $lineHeight);
            }
        }
    }

    private function _removeUserDefinedFootnoteClass($retVal)
    {
        return preg_replace('~class="([^"]*)footnote([^"]*)"~i', 'class="$1$2"', $retVal);
    }
}

?>