<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Research\Exposition\Weave\Tool;
use Rc\Domain\Model\Research\Exposition\Weave\ToolInterface;
use Rc\Domain\Repository\Research\Exposition\PopoverRepository;

class My_View_Helper_ToolDecorator extends Zend_View_Helper_Abstract
{

    const RENDER_FOR_EDITOR = 'editor';
    const RENDER_FOR_VIEW = 'view';

    /**
     * @var int
     */
    protected static $footNoteCounter = 0;

    /**
     * @var Tool
     */
    protected $tool;

    /**
     * @var string
     */
    protected $content;

    /**
     * @var string
     */
    protected $renderFor;

    /**
     * @var array
     */
    protected $attributes = [];

    /**
     * @var array
     */
    protected $options = [];

    public static function parsePopovers($content, $renderFor)
    {
        if (empty($content)) {
            return '';
        }

        $content = mb_convert_encoding($content, 'HTML-ENTITIES', "UTF-8");
        $content = str_replace('<p></p>', '<p><br/></p>', $content);

        $doc = new DOMDocument();
        @$doc->loadHTML($content);

        self::childrenWalkRecursive($doc->documentElement->childNodes, $renderFor);
        return $doc->saveHTML();
    }

    /**
     * @param Tool $tool
     * @return string
     */
    protected static function getShadowString($tool)
    {
        return
            (int)$tool->getShadowMarginLeft() . 'px ' .
            (int)$tool->getShadowMarginTop() . 'px ' .
            (int)$tool->getShadowBlur() . 'px ' .
            self::hex2rgba($tool->getShadowColor(), ($tool->getOpacity() !== null ? $tool->getOpacity() : 100) / 100);
    }

    protected static function hex2rgba($hex, $opacity)
    {
        $htmlColors = [
            'maroon'               => '#800000',
            'darkred'              => '#8B0000',
            'brown'                => '#A52A2A',
            'firebrick'            => '#B22222',
            'crimson'              => '#DC143C',
            'red'                  => '#FF0000',
            'tomato'               => '#FF6347',
            'coral'                => '#FF7F50',
            'indianred'            => '#CD5C5C',
            'lightcoral'           => '#F08080',
            'darksalmon'           => '#E9967A',
            'salmon'               => '#FA8072',
            'lightsalmon'          => '#FFA07A',
            'orangered'            => '#FF4500',
            'darkorange'           => '#FF8C00',
            'orange'               => '#FFA500',
            'gold'                 => '#FFD700',
            'darkgoldenrod'        => '#B8860B',
            'goldenrod'            => '#DAA520',
            'palegoldenrod'        => '#EEE8AA',
            'darkkhaki'            => '#BDB76B',
            'khaki'                => '#F0E68C',
            'olive'                => '#808000',
            'yellow'               => '#FFFF00',
            'yellowgreen'          => '#9ACD32',
            'darkolivegreen'       => '#556B2F',
            'olivedrab'            => '#6B8E23',
            'lawngreen'            => '#7CFC00',
            'chartreuse'           => '#7FFF00',
            'greenyellow'          => '#ADFF2F',
            'darkgreen'            => '#006400',
            'green'                => '#008000',
            'forestgreen'          => '#228B22',
            'lime'                 => '#00FF00',
            'limegreen'            => '#32CD32',
            'lightgreen'           => '#90EE90',
            'palegreen'            => '#98FB98',
            'darkseagreen'         => '#8FBC8F',
            'mediumspringgreen'    => '#00FA9A',
            'springgreen'          => '#00FF7F',
            'seagreen'             => '#2E8B57',
            'mediumaquamarine'     => '#66CDAA',
            'mediumseagreen'       => '#3CB371',
            'lightseagreen'        => '#20B2AA',
            'darkslategray'        => '#2F4F4F',
            'teal'                 => '#008080',
            'darkcyan'             => '#008B8B',
            'aqua'                 => '#00FFFF',
            'cyan'                 => '#00FFFF',
            'lightcyan'            => '#E0FFFF',
            'darkturquoise'        => '#00CED1',
            'turquoise'            => '#40E0D0',
            'mediumturquoise'      => '#48D1CC',
            'paleturquoise'        => '#AFEEEE',
            'aquamarine'           => '#7FFFD4',
            'powderblue'           => '#B0E0E6',
            'cadetblue'            => '#5F9EA0',
            'steelblue'            => '#4682B4',
            'cornflowerblue'       => '#6495ED',
            'deepskyblue'          => '#00BFFF',
            'dodgerblue'           => '#1E90FF',
            'lightblue'            => '#ADD8E6',
            'skyblue'              => '#87CEEB',
            'lightskyblue'         => '#87CEFA',
            'midnightblue'         => '#191970',
            'navy'                 => '#000080',
            'darkblue'             => '#00008B',
            'mediumblue'           => '#0000CD',
            'blue'                 => '#0000FF',
            'royalblue'            => '#4169E1',
            'blueviolet'           => '#8A2BE2',
            'indigo'               => '#4B0082',
            'darkslateblue'        => '#483D8B',
            'slateblue'            => '#6A5ACD',
            'mediumslateblue'      => '#7B68EE',
            'mediumpurple'         => '#9370DB',
            'darkmagenta'          => '#8B008B',
            'darkviolet'           => '#9400D3',
            'darkorchid'           => '#9932CC',
            'mediumorchid'         => '#BA55D3',
            'purple'               => '#800080',
            'thistle'              => '#D8BFD8',
            'plum'                 => '#DDA0DD',
            'violet'               => '#EE82EE',
            'magenta/fuchsia'      => '#FF00FF',
            'orchid'               => '#DA70D6',
            'mediumvioletred'      => '#C71585',
            'palevioletred'        => '#DB7093',
            'deeppink'             => '#FF1493',
            'hotpink'              => '#FF69B4',
            'lightpink'            => '#FFB6C1',
            'pink'                 => '#FFC0CB',
            'antiquewhite'         => '#FAEBD7',
            'beige'                => '#F5F5DC',
            'bisque'               => '#FFE4C4',
            'blanchedalmond'       => '#FFEBCD',
            'wheat'                => '#F5DEB3',
            'cornsilk'             => '#FFF8DC',
            'lemonchiffon'         => '#FFFACD',
            'lightgoldenrodyellow' => '#FAFAD2',
            'lightyellow'          => '#FFFFE0',
            'saddlebrown'          => '#8B4513',
            'sienna'               => '#A0522D',
            'chocolate'            => '#D2691E',
            'peru'                 => '#CD853F',
            'sandybrown'           => '#F4A460',
            'burlywood'            => '#DEB887',
            'tan'                  => '#D2B48C',
            'rosybrown'            => '#BC8F8F',
            'moccasin'             => '#FFE4B5',
            'navajowhite'          => '#FFDEAD',
            'peachpuff'            => '#FFDAB9',
            'mistyrose'            => '#FFE4E1',
            'lavenderblush'        => '#FFF0F5',
            'linen'                => '#FAF0E6',
            'oldlace'              => '#FDF5E6',
            'papayawhip'           => '#FFEFD5',
            'seashell'             => '#FFF5EE',
            'mintcream'            => '#F5FFFA',
            'slategray'            => '#708090',
            'lightslategray'       => '#778899',
            'lightsteelblue'       => '#B0C4DE',
            'lavender'             => '#E6E6FA',
            'floralwhite'          => '#FFFAF0',
            'aliceblue'            => '#F0F8FF',
            'ghostwhite'           => '#F8F8FF',
            'honeydew'             => '#F0FFF0',
            'ivory'                => '#FFFFF0',
            'azure'                => '#F0FFFF',
            'snow'                 => '#FFFAFA',
            'black'                => '#000000',
            'dimgray/dimgrey'      => '#696969',
            'gray/grey'            => '#808080',
            'darkgray/darkgrey'    => '#A9A9A9',
            'silver'               => '#C0C0C0',
            'lightgray/lightgrey'  => '#D3D3D3',
            'gainsboro'            => '#DCDCDC',
            'whitesmoke'           => '#F5F5F5',
            'white'                => '#FFFFFF',
        ];
        if (empty($hex)) {
            return null;
        }

        if (isset($htmlColors[$hex])) {
            $hex = $htmlColors[$hex];
        }

        $hex = str_replace("#", "", $hex);

        if (strlen($hex) == 3) {
            $r = hexdec(substr($hex, 0, 1) . substr($hex, 0, 1));
            $g = hexdec(substr($hex, 1, 1) . substr($hex, 1, 1));
            $b = hexdec(substr($hex, 2, 1) . substr($hex, 2, 1));
        } else {
            $r = hexdec(substr($hex, 0, 2));
            $g = hexdec(substr($hex, 2, 2));
            $b = hexdec(substr($hex, 4, 2));
        }

        return 'rgba(' . $r . ',' . $g . ',' . $b . ',' . $opacity . ')';
    }

    protected static function childrenWalkRecursive(DOMNodeList $childNodeList, $renderFor)
    {
        /** @var DOMElement $childNode */
        foreach ($childNodeList as $childNode) {
            if ($childNode->hasAttributes() === false && $childNode->hasChildNodes() === false) {
                continue;
            }

            $popover = null;
            foreach ($childNode->attributes as $attribute) {
                if ($attribute->name == 'data-popover') {
                    try {
                        $popover = PopoverRepository::getById($attribute->value);
                    } catch (\Exception $exception) {
                        $childNode->removeAttribute('data-popover');
                        continue;
                    }
                    $options = [
                        'viewon'     => $popover->getViewOn(),
                        'width'      => $popover->getWidth() + $popover->getPaddingLeft() + $popover->getPaddingRight() + 2 * $popover->getBorderWidth(),
                        'height'     => $popover->getHeight() + $popover->getPaddingTop() + $popover->getPaddingBottom() + 2 * $popover->getBorderWidth(),
                        'background' => $popover->getBackground(),
                        'position'   => $popover->getPosition(),
                    ];
                    if ($renderFor == self::RENDER_FOR_EDITOR) {
                        $options['title'] = (string)$popover->getTitle();
                        $options['autocontent'] = $popover->isAutoContent();
                    } else if ($popover->isAutoContent()) {
                        if ($childNode->firstChild) {
                            $childNode->removeChild($childNode->firstChild);
                        }
                        $childNode->appendChild(new DOMText(++self::$footNoteCounter));
                    }
                    if ($renderFor == self::RENDER_FOR_VIEW) {
                        $options['offset'] = $popover->getMargin();
                        $options['shadow'] = self::getShadowString($popover);
                    }
                    $childNode->setAttribute('data-popover-auto', (int)$popover->isAutoContent());
                    $childNode->setAttribute('data-popover-options', json_encode($options, JSON_NUMERIC_CHECK));
                    $childNode->setAttribute('href', '#' . $popover->getId());
                }
            }

            if ($childNode->hasChildNodes() === true) {
                self::childrenWalkRecursive($childNode->childNodes, $renderFor);
            }
        }
    }

    public function toolDecorator(ToolInterface $tool, $renderFor = self::RENDER_FOR_EDITOR)
    {
        $decorator = ucfirst($tool->getName()) . 'ToolDecorator';
        return $this->view->$decorator($tool, $renderFor);
    }

    protected function _renderTool()
    {
        return $this->view->htmlElement(
            'div',
            $this->_getToolContainer(),
            $this->_getToolAttribs(),
            ['render' => false]
        );
    }

    protected function _getToolContainer()
    {
        return $this->view->htmlElement(
            'div',
            $this->_getToolContent() . $this->_getIconContainer(),
            [
                'class' => 'tool-content',
                'style' => [
                    'top'     => (int)$this->tool->getPaddingTop() . 'px',
                    'right'   => (int)$this->tool->getPaddingRight() . 'px',
                    'bottom'  => (int)$this->tool->getPaddingBottom() . 'px',
                    'left'    => (int)$this->tool->getPaddingLeft() . 'px',
                    'opacity' => $this->tool->getOpacity() / 100,
                ],
            ],
            [
                'render' => false,
            ]
        );
    }

    protected function _getToolContent()
    {
        return $this->content;
    }

    protected function _getIconContainer()
    {
        return '<div class="icon"></div>';
    }

    protected function _getToolAttribs()
    {
        return $this->attributes;
    }

    protected function _init(ToolInterface $tool, $renderFor)
    {
        $this->tool = $tool;
        $this->renderFor = $renderFor;

        $popover = $this->tool->getPopover();
        $this->attributes = [
            'class' => 'tool tool-' . $this->tool->getName(),
            'data'  => [
                'id'              => $this->tool->getId(),
                'source'          => $this->tool->getSourceId(),
                'locked'          => (int)$this->tool->isLocked(),
                'editable'        => (int)$this->tool->isEditableBy(Identity::getUserOrNull()),
                'tool'            => $this->tool->getName(),
                'title'           => $this->renderFor == self::RENDER_FOR_EDITOR ? (string)$this->tool->getTitle() : null,
                'date'            => $this->renderFor == self::RENDER_FOR_EDITOR ? $this->tool->getCreated() : null,
                'options'         => json_encode(static::_getToolOptions()),
                'rotate'          => 0,
                'popover'         => $popover ? $popover->getId() : null,
                'popover-options' => json_encode($popover ? [
                    'viewon'     => $popover->getViewOn(),
                    'width'      => $popover->getWidth() + $popover->getPaddingLeft() + $popover->getPaddingRight() + 2 * $popover->getBorderWidth(),
                    'height'     => $popover->getHeight() + $popover->getPaddingTop() + $popover->getPaddingBottom() + 2 * $popover->getBorderWidth(),
                    'offset'     => $popover->getMargin(),
                    'shadow'     => $this->getShadowString($popover),
                    'background' => $popover->getBackground(),
                    'position'   => $popover->getPosition(),
                ] : []),
            ],
            'style' => [
                'left'       => $this->tool->getLeft() . 'px',
                'top'        => $this->tool->getTop() . 'px',
                'width'      => $this->tool->getWidth() . 'px',
                'height'     => $this->tool->getHeight() . 'px',
                'padding'    => [
                    'top'    => (int)$this->tool->getPaddingTop() . 'px',
                    'right'  => (int)$this->tool->getPaddingRight() . 'px',
                    'bottom' => (int)$this->tool->getPaddingBottom() . 'px',
                    'left'   => (int)$this->tool->getPaddingLeft() . 'px',
                ],
                'border'     => [
                    'width'  => (int)$this->tool->getBorderWidth() . 'px',
                    'style'  => $this->tool->getBorderStyle(),
                    'color'  => $this->hex2rgba($this->tool->getBorderColor(), $this->tool->getOpacity() / 100),
                    'radius' => (int)$this->tool->getBorderRadius() . 'px',
                ],
                'box-shadow' => $this->getShadowString($this->tool),
                'z-index'    => $this->tool->getZIndex(),
                'background' => [
                    'image'    => $this->view->wrap(
                        $this->tool->getBackgroundImageUrl($this->tool->getWidth(), $this->tool->getHeight()),
                        'url(',
                        ')',
                        false
                    ),
                    'position' => $this->tool->getBackgroundImagePosition(),
                    'repeat'   => $this->tool->getBackgroundImageStyle(),
                    'size'     => $this->tool->getBackgroundImageSize(),
                    'color'    => $this->hex2rgba($this->tool->getBackgroundColor(), $this->tool->getOpacity() / 100),
                ],
            ],
        ];

        if ($tool->getName() !== Tool::MEDIA_TYPE_AUDIO && $tool->getName() !== Tool::MEDIA_TYPE_VIDEO && $tool->getName() !== ToolFactory::TOOL_COMMENT
        ) {
            $this->attributes['data']['rotate'] = $this->tool->getRotate();
            $this->attributes['style'] += [
                'transform'         => 'rotate(' . $this->tool->getRotate() . 'deg)',
                '-moz-transform'    => 'rotate(' . $this->tool->getRotate() . 'deg)',
                '-webkit-transform' => 'rotate(' . $this->tool->getRotate() . 'deg)',
                '-o-transform'      => 'rotate(' . $this->tool->getRotate() . 'deg)',
                '-ms-transform'     => 'rotate(' . $this->tool->getRotate() . 'deg)',
            ];
        }

        if ($tool->getName() === ToolFactory::TOOL_COMMENT) {
            $this->attributes['data']['creator-id'] = $tool->getAuthor()->getId();
            $this->attributes['data']['creator-name'] = $tool->getAuthor()->getName();
        }

        if ($this->renderFor == self::RENDER_FOR_EDITOR) {
            if (static::_showToolInEditor()) {
                $this->content = static::_getToolContentForEditor();
                $this->content .= '<div class="menu"></div>';
            }
        } else {
            $this->content = static::_showToolInView() ? static::_getToolContentForView() : null;
        }
    }

    /**
     * @return array
     */
    protected function _getToolOptions()
    {
        return $this->options;
    }

    protected function _showToolInEditor()
    {
        return true;
    }

    protected function _getToolContentForEditor($options = [])
    {
        return $this->getMediaHelper()->sizeMedia(
            $this->tool->getMediaContainer(),
            $this->tool->getWidth(),
            $this->tool->getHeight(),
            [
                'size'     => $this->tool->getSize(),
                'position' => $this->tool->getPosition(),
            ] + $options + ($this->tool->hasMedia() ? $this->tool->getMedia()->getOptions() : [])
        );
    }

    /**
     * @return \Rc\Domain\Model\MediaInterface|null
     */
    protected function getMedia()
    {
        try {
            return $this->tool->getMediaContainer()->getMedia();
        } catch (\Exception $exception) {
            return null;
        }
    }

    /**
     * @return \Rc\Domain\Model\Media\Resource\ImageResource|null
     */
    protected function getThumb()
    {
        try {
            return $this->tool->getMediaContainer()->getThumb();
        } catch (\Exception $exception) {
            return null;
        }
    }

    protected function _showToolInView()
    {
        return $this->tool->hasMedia();
    }

    protected function _getToolContentForView($options = [])
    {
        return $this->getMediaHelper()->sizeMedia(
            $this->tool->getMediaContainer(),
            $this->tool->getWidth(),
            $this->tool->getHeight(),
            [
                'size'     => $this->tool->getSize(),
                'position' => $this->tool->getPosition(),
            ] + $options + ($this->tool->hasMedia() ? $this->tool->getMedia()->getOptions() : [])
        );
    }

    protected function addHoverPopupIfNecessary()
    {
        if ($this->tool->getHoverPopup() == '' || !$this->tool->getFirstResource()) {
            return;
        }

        if ($this->tool->getHoverPopup() != 'plaintext') {
            $this->attributes['data']['copyright'] = $this->_replaceAuthorNamesForDoubleBlindReview(
                $this->tool->getFirstResource()->getCopyrightHolder()
            );
        }

        if ($this->tool->getHoverPopup() != 'copyright') {
            $this->attributes['data']['plaintext'] = $this->tool->getHoverPlainText();
        }
    }

    protected function _replaceAuthorNamesForDoubleBlindReview($text)
    {
        $research = $this->tool->getExposition();
        if (!$research->isDoubleBlindReviewer()) {
            return $text;
        }

        $authors = array_merge([$research->getAuthor()], $research->getCoAuthors(false));
        foreach ($authors as $author) {
            $text = str_replace($author->getName(), 'author name hidden', $text);
        }

        return $text;
    }

    protected function _parsePopovers($content)
    {
        return self::parsePopovers($content, $this->renderFor);
    }

    /**
     * @return My_View_Helper_SizeMedia
     */
    protected function getMediaHelper()
    {
        return $this->view->getHelper('sizeMedia');
    }
}

?>
