<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Portal;
use Rc\Domain\Model\Portal\Issue;
use Rc\Domain\Model\Research\Application;
use Rc\Domain\Model\Research\Degree;
use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Model\Research\Exposition\SimpleMedia;
use Rc\Domain\Model\Research\Project;
use Rc\Domain\Model\Research\Work;
use Rc\Domain\Model\User;

class My_View_Helper_Url extends Zend_View_Helper_Abstract
{

    public function url()
    {
        return $this;
    }

    public function entityUrl(\Rc\Domain\Model\ModelInterface $entity, $params = [])
    {
        if ($entity instanceof User) {
            return $this->profileUrl($entity, $params);
        } elseif ($entity instanceof Portal) {
            return $this->portalUrl($entity, $params);
        } elseif (!method_exists($this, $entity->getName() . 'url')) {
            throw new InvalidArgumentException('Object of type "' . get_class($entity) . ' is not supported.');
        }
        return $this->{$entity->getName() . 'Url'}($entity, $params);
    }

    public function issueUrl(Issue $issue)
    {
        return $this->encodeUrl(config('app.url') . '/portals/issue?issue=' . $issue->getId());
    }

    public function researchUrl(Exposition $research, $params = [])
    {
        return $this->expositionUrl($research, $params);
    }

    public function applicationUrl(Application $application, $params = [])
    {
        return $this->createUrl($application->getId(), 'application', $params);
    }

    public function workUrl(Work $work, $params = [])
    {
        return $this->createUrl($work->getId(), 'work', $params);
    }

    public function expositionUrl(Exposition $exposition, $params = [])
    {
        return $this->createUrl($exposition->getId(), 'exposition', $params);
    }

    public function projectUrl(Project $project, $params = [])
    {
        return $this->createUrl($project->getId(), 'project', $params);
    }

    public function degreeUrl(Degree $degree, $params = [])
    {
        return $this->createUrl($degree->getId(), 'degree', $params);
    }

    /**
     * @param User $user
     * @param array $params
     * @return mixed
     */
    public function profileUrl(User $user, $params = [])
    {
        return $this->createUrl($user->getId(), 'person', $params);
    }

    /**
     * @param Portal $portal
     * @param array $params
     * @return string
     */
    public function portalUrl(Portal $portal, $params = [])
    {
        if ($portal->getAllowLinkExternal() && $portal->getUrl()) {
            return $portal->getUrl();
        } elseif (!empty($params['absolute'])) {
            return config('app.url') . '/portals?portal=' . $portal->getId();
        } else {
            return '/portals?portal=' . $portal->getId();
        }
    }

    public function fileUrl(SimpleMedia $file, $width = SimpleMedia::DEFAULT_THUMB_WIDTH, $height = SimpleMedia::DEFAULT_THUMB_HEIGHT)
    {
        return $this->encodeUrl($file->getMediaContainer()->getThumb()->getExternalUri());
    }

    public function relationUrl($entity, $params = [])
    {
        if ($entity instanceof Exposition) {
            return $this->expositionUrl($entity, $params);
        }
        if ($entity instanceof Work) {
            return $this->workUrl($entity, $params);
        }
        if ($entity instanceof Project) {
            return $this->projectUrl($entity, $params);
        }
        if ($entity instanceof Degree) {
            return $this->degreeUrl($entity, $params);
        }
        if ($entity instanceof User) {
            return $this->profileUrl($entity, $params);
        }
        if ($entity instanceof SimpleMedia) {
            return $this->fileUrl($entity, $params);
        }
        if ($entity instanceof Application) {
            return $this->applicationUrl($entity, $params);
        }

        throw new Exception('Unsupported object type ' . get_class($entity) . '.');
    }

    protected function createUrl($id, $type, $params = [])
    {
        $params = http_build_query($params);
        if (!empty($params)) {
            $params = '&' . $params;
        }
        return $this->encodeUrl(config('app.url') . '/profile/show-' . $type . '?' . $type . '=' . $id . $params);
    }

    protected function encodeUrl($url)
    {
        return str_replace('&', '&amp;', $url);
    }

}
