<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Media\Dimension;
use Rc\Domain\Model\Media\Resource\VideoResource;
use Rc\Domain\Model\Research\Exposition\Weave\Tool;
use Rc\Domain\Model\Research\Exposition\Weave\Tool\VideoTool;
use Rc\Domain\Model\Research\Exposition\Weave\ToolInterface;

/**
 * @property VideoTool $tool
 */
class My_View_Helper_VideoToolDecorator extends My_View_Helper_ToolDecorator
{

    /**
     * @var VideoTool
     */
    protected $tool;

    /**
     * @var Mobile_Detect
     */
    protected $mobileDetect;

    /**
     * @param VideoTool $tool
     * @param string $renderFor
     * @return mixed
     */
    public function videoToolDecorator($tool, $renderFor = self::RENDER_FOR_EDITOR)
    {
        $this->_init($tool, $renderFor);
        return $this->_renderTool();
    }

    public function _getToolContentForView($options = [])
    {
        $mediaContainer = $this->tool->getMediaContainer();
        if (!$this->getMediaHelper()->canBeRendered($mediaContainer)) {
            return $this->getMediaHelper()->getErrorMessage($mediaContainer);
        }

        $this->addHoverPopupIfNecessary();

        $file = null;
        if ($this->getDerivativeUrl(VideoResource::DERIVATIVE_TYPE_MP4)) {
            $file = $this->getDerivativeUrl(VideoResource::DERIVATIVE_TYPE_MP4);
        }

        if ($this->getThumb()) {
            $size = $this->getMediaHelper()->getBestFittingSizeFor(
                $this->getThumb()->getWidth(),
                $this->getThumb()->getHeight(),
                $this->tool->getWidth(),
                $this->tool->getHeight(),
                Tool::SIZE_CONTAIN
            );
        } else {
            $size = $this->tool->getDimension();
        }

        if ($this->mobileDetect->isMobile() || $this->mobileDetect->isTablet()) {
            $retVal = '<video controls width="' . $this->tool->getWidth() . '" height="' . $this->tool->getHeight() . '">';
            if (isset($file)) {
                $retVal .= '<source src="' . $file . '" type="video/mp4"/>';
            }
            $retVal .= '</video>';
            return $retVal;
        } else {
            return '<div id="video-' . $this->tool->getId() . '" width="' . $this->tool->getWidth() . '" height="' . $this->tool->getHeight() . '"
                    data-image="' . $this->getImageUrl($size) . '" data-image-width="' . $size->getWidth() . '" data-image-height="' . $size->getHeight() . '"
                    data-file="' . $file . '" data-loop="' . $this->tool->isLoop() . '" data-autoplay="' . $this->tool->isAutoPlay() . '"
                    data-hide-player="' . $this->tool->isHidePlayer() . '" data-stop-other-players="' . $this->tool->isStopOtherPlayers() . '"
                    data-display-volume="' . $this->tool->isDisplayVolume() . '" data-display-minimal="' . $this->tool->isDisplayMinimal() . '"></div>';
        }
    }

    protected function _showToolInView()
    {
        return true;
    }

    protected function _init(ToolInterface $tool, $renderFor)
    {
        $this->mobileDetect = new Mobile_Detect();
        parent::_init($tool, $renderFor);
    }

    protected function _getToolContentForEditor($options = [])
    {
        $mediaContainer = $this->tool->getMediaContainer();
        if (!$this->getMediaHelper()->canBeRendered($mediaContainer)) {
            return $this->getMediaHelper()->getErrorMessage($mediaContainer);
        }

        /** @var VideoResource $video */
        $video = $mediaContainer->getMedia()->getMaster();

        return $this->getMediaHelper()->sizeThumb($this->getThumb(), $this->tool->getWidth(), $this->tool->getHeight(), [
            'size'        => Tool::SIZE_CONTAIN,
            'position'    => Tool::POSITION_CENTER_CENTER,
            'aspectRatio' => $this->tool->isFixAspectRatio() && isset($video) ? $video->getAspectRatio() : null,
        ]);
    }

    /**
     * @param string $type
     * @return string|null
     */
    protected function getDerivativeUrl($type)
    {
        $entity = $this->getMedia();
        if (!isset($entity)) {
            return null;
        }
        $type = $entity->getDerivative($type);
        if (!isset($type)) {
            return null;
        }
        return $type->getExternalUri();
    }

    /**
     * @param Dimension $size
     * @return null|string
     */
        protected function getImageUrl(Dimension $size)
    {
        $splashScreen = $this->getThumb();
        if (!isset($splashScreen)) {
            return null;
        }
        return $this->view->getHelper('sizeMedia')->url($splashScreen, $size);
    }
}

?>