<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Model\Research\Exposition\Weave;

class My_View_Helper_ViewUrl extends Zend_View_Helper_Abstract
{

    /**
     * @param Exposition $research
     * @param Weave $weave
     * @param null $coordX
     * @param null $coordY
     * @param array $params
     * @return string
     */
    public function viewUrl(Exposition $research, Weave $weave = null, $coordX = null, $coordY = null, array $params = [])
    {
        return config('app.url') . $this->relativeViewUrl($research, $weave, $coordX, $coordY, $params);
    }

    /**
     * @param Exposition $research
     * @param Weave $weave
     * @param null $coordX
     * @param null $coordY
     * @param array $params
     * @return string
     */
    public function relativeViewUrl(Exposition $research, Weave $weave = null, $coordX = null, $coordY = null, array $params = [])
    {
        if (!isset($weave)) {
            $weave = $research->getDefaultWeave();
        }
        if (!isset($weave)) {
            $weaves = $research->getWeaves();
            $weave = reset($weaves);
        }

        if (!$weave) {
            return null;
        }
        $defaultAspect = $weave->getDefaultAspect();
        if (!isset($coordX)) {
            $coordX = $defaultAspect['x'];
        }
        if (!isset($coordY)) {
            $coordY = $defaultAspect['y'];
        }

        $retVal = '/view/' . $research->getId() . '/' . $weave->getId();
        if (!empty($coordX) || !empty($coordY)) {
            $retVal .= '/' . (int)$coordX;
        }
        if (!empty($coordY)) {
            $retVal .= '/' . $coordY;
        }

        $params = http_build_query($params);
        if (!empty($params)) {
            $params = '?' . $params;
        }

        return $retVal . $params;
    }

}

?>