function Editor() {
    if (Editor.prototype.init) {
        throw new Error('Only one instance of Editor is allowed (Singleton)!');
    } else {
        Editor.prototype.init = true;
    }

    var selectedComment,
        initialized = false,
        hintOnTabCloseStack = 0,
        $this = this;

    Editor.construct = function () {
        Editor.CONTAINER_MAIN = $('#container-main');

        Editor.RULER_HORIZONTAL = $('#content-ruler-n');
        Editor.RULER_VERTICAL = $('#content-ruler-w');

        Editor.ContentList = new ContentList(true);
        Editor.ToolList = new ToolList(true);
        Editor.PopoverList = new PopoverList(true);
        Editor.LayerList = new LayerList(true);
        Editor.Overview = new Overview(true);
        Editor.WorkList = new WorkList(true);
        Editor.WeaveList = new WeaveList(true);
        Editor.SimpleMediaList = new SimpleMediaList(true);
        Editor.MadekMediaList = new MadekMediaList(true);
        Editor.UploadList = new UploadList(true);
        Editor.TextList = new TextList(true);
        Editor.CommentList = new CommentList(true);

        Editor.ContentList.CONTAINER_SCROLL.triggerHandler('scroll');
    };

    Editor.getUserId = function () {
        return Editor.CONTAINER_MAIN.data('userid');
    };

    Editor.getUserName = function () {
        return Editor.CONTAINER_MAIN.data('username');
    };

    Editor.getCollaborators = function () {
        return collaborators;
    };

    Editor.loadItem = function (id, callback) {
        Editor.ContentList.loadChild(id, callback);
        //Editor.LayerList.loadChild(id);
    };

    Editor.selectItem = function (id) {
        Editor.ContentList.selectChild(id);
        Editor.LayerList.selectChild(id);
    };

    Editor.toggleItemSelection = function (id) {
        Editor.ContentList.toggleChildSelection(id);
        Editor.LayerList.toggleChildSelection(id);
    };

    Editor.selectAllItems = function () {
        $.each(Editor.ContentList.notSelectedChildrenIds(), function () {
            Editor.ContentList.toggleChildSelection(this);
            Editor.LayerList.toggleChildSelection(this);
        });
    };

    Editor.clearItemSelection = function () {
        $.each(Editor.ContentList.selectedChildrenIds(), function () {
            Editor.ContentList.toggleChildSelection(this);
        });
        $.each(Editor.LayerList.selectedChildrenIds(), function () {
            Editor.LayerList.toggleChildSelection(this);
        });
    };

    Editor.redoAction = function () {

    };

    Editor.undoAction = function () {
        // $.post('/item/undo?research=' + Editor.getResearchId() + '&weave=' + Editor.getWeaveId(), {}, function (data) {
        //     Editor.Overview.load();
        // });
    };

    Editor.addItemsToClipboard = function (action) {
        var clipboard = $.parseJSON($.cookie('clipboard'));
        if (clipboard && clipboard.items.length && clipboard.action == 'cut') {
            var cutItemIds = [];
            $.each(clipboard.items, function () {
                cutItemIds.push(this.id);
            });
            Editor.showItems(cutItemIds);
        }

        var items = [];
        var itemIds = [];
        var offset = {
            left: Editor.ContentList.CONTAINER_SCROLL.scrollLeft(),
            top: Editor.ContentList.CONTAINER_SCROLL.scrollTop()
        };
        Editor.ContentList.selectedChildren().each(function () {
            items.push({
                id: $(this).data('id'),
                l: parseInt($(this).css('left')) - offset.left,
                t: parseInt($(this).css('top')) - offset.top,
                w: parseInt($(this).css('width')),
                h: parseInt($(this).css('height')),
                r: parseInt($(this).css('rotate'))
            });
            itemIds.push($(this).data('id'));
        });
        if (action == 'cut') {
            Editor.hideItems(itemIds);
        }
        $.cookie('clipboard', JSON.stringify({action: action, items: items}));
    };

    Editor.pasteItemsFromClipboard = function () {
        var clipboard = $.parseJSON($.cookie('clipboard'));
        if (!clipboard || !clipboard.items.length) {
            return;
        }

        if (clipboard.action == 'cut') {
            $.cookie('clipboard', '');
        }

        var placeholder = {};
        var scrollOffset = {
            left: Editor.ContentList.CONTAINER_SCROLL.scrollLeft(),
            top: Editor.ContentList.CONTAINER_SCROLL.scrollTop()
        };

        var pasteOffset = {left: 0, top: 0};
        $.each(clipboard.items, function () {
            pasteOffset.left = Math.min(pasteOffset.left, this.l + scrollOffset.left);
            pasteOffset.top = Math.min(pasteOffset.top, this.t + scrollOffset.top);
        });

        var data = {
            item: {},
            left: {},
            top: {},
            width: {},
            height: {},
            rotate: {}
        };

        $.each(clipboard.items, function () {
            data.item[this.id] = this.id;
            data.left[this.id] = this.l + scrollOffset.left - pasteOffset.left;
            data.top[this.id] = this.t + scrollOffset.top - pasteOffset.top;
            data.width[this.id] = this.w;
            data.height[this.id] = this.h;
            data.rotate[this.id] = this.r;

            placeholder[this.id] = $('<div class="tool container-loading locked"></div>').css({
                width: this.w,
                height: this.h,
                top: this.t + scrollOffset.top,
                left: this.l + scrollOffset.left
            }).appendTo(Editor.ContentList.CONTAINER_CONTENT);
        });

        $.post('/item/' + clipboard.action + '?research=' + Editor.getResearchId() + '&weave=' + Editor.getWeaveId(), data, function (data) {
            $(data).each(function () {
                var itemId = $(this).data('id');
                placeholder[clipboard.action == 'copy' ? $(this).data('source') : itemId].attr('data-id', itemId);
                Editor.ContentList._eLoadChild.apply(Editor.ContentList, [this.outerHTML]);
                Editor.ContentList.loadChildAnimation(itemId, false);

                Editor.clearItemSelection();
                Editor.toggleItemSelection(itemId);
            });

            Editor.Overview.load();
        });
    };

    Editor.addRepository = function (form) {
        form = $(form);

        form.addClass('disabled');

        $.post(
            form.attr('action'),
            form.serializeArray(),
            function (data) {
                if (!data) {
                    location.reload();
                } else {
                    $('.error', form).text(data);
                    form.removeClass('disabled');
                }
            });
    };

    Editor.removeRepository = function (path) {
        $.post(
            path,
            {},
            function (data) {
                location.reload();
            });
    };

    Editor.hideItems = function (ids) {
        Editor.ContentList.hideChildren(ids);
        Editor.LayerList.hideChildren(ids);
    };

    Editor.editItem = function (id) {
        Editor.enableHintOnTabClose();
        Editor.ContentList.child(id).trigger('dblclick.edit');
        Editor.disableHintOnTabClose();
    };

    Editor.showItems = function (ids) {
        Editor.ContentList.showChildren(ids);
        Editor.LayerList.showChildren(ids);
    };

    Editor.removeItems = function (ids) {
        Editor.ContentList.removeChildren(ids);
        Editor.LayerList.removeChildren(ids);
        Editor.Overview.load();
    };

    Editor.removeSimpleMedia = function (ids) {
        Editor.SimpleMediaList.removeChildren(ids);
    };

    Editor.lockItems = function (ids, status) {
        Editor.LayerList.lockChildren(ids, {
            lock: status,
            save: true
        });
        Editor.ContentList.lockChildren(ids, {
            lock: status,
            save: true
        });
    };

    Editor.addWork = function (id) {
        Editor.WorkList.load();
    };

    Editor.loadWork = function (id) {
        Editor.WorkList.loadChild(id);
    };

    Editor.removeWorks = function (ids) {
        Editor.WorkList.removeChildren(ids);
    };

    Editor.updateRulerPosition = function () {
        Editor.RULER_VERTICAL.css('top', -Editor.ContentList.CONTAINER_SCROLL.scrollTop());
        Editor.RULER_HORIZONTAL.css('left', -Editor.ContentList.CONTAINER_SCROLL.scrollLeft());
    };

    Editor.restoreDeletedTools = function () {
        new DialogForm({
            url: '/item/restore-deleted?weave=' + Editor.getWeaveId()
        });
    };

    Editor.restoreDeletedFootnotes = function () {
        new DialogForm({
            url: '/popover/restore-deleted?weave=' + Editor.getWeaveId()
        });
    };

    Editor.getTabContainer = function (context) {
        return $(context).closest('.ui-tabs');
    };

    Editor.loadSelectedTab = function (context) {
        var tabContainer = Editor.getTabContainer(context),
            scrollPosition = $('.scrollable', tabContainer).scrollTop();

        tabContainer.bind('tabsload', function () {
            //jQueryNew('.scrollable', tabContainer).scrollTo(scrollPosition, 0);
        });
        tabContainer.tabs('load', Editor.getTabContainer(context).tabs('option', 'selected'));
    };

    Editor.disableTab = function (context, index, disable) {
        var disabled = Editor.getTabContainer(context).tabs('option', 'diabled');
        if (disable === false) {
            disabled = $.grep(disabled, function () {
                return this != index;
            });
        } else {
            disabled.push(index);
            disabled = $.unique(disabled);
        }
        Editor.getTabContainer(context).tabs('option', 'disabled', disabled);
    };

    Editor.initButtons = function () {
        $('button').each(function () {
            $(this).button({
                icons: {
                    primary: $(this).data('button-primary')
                },
                disabled: $(this).data('context') && $(this).data('context') != 'global',
                text: false
            });
        });
        //$('input[type=checkbox]').button();
        $('[data-buttonset]').buttonset();
    };

    Editor.setDefaultAspect = function (x, y) {
        x |= Editor.ContentList.CONTAINER_SCROLL.scrollLeft();
        y |= Editor.ContentList.CONTAINER_SCROLL.scrollTop();

        new DialogForm({
            url: '/weave/set-default-aspect?weave=' + Editor.getWeaveId() + '&x=' + x + '&y=' + y
        });
    };

    Editor.showCurrentAspect = function (x, y) {
        x |= Editor.ContentList.CONTAINER_SCROLL.scrollLeft();
        y |= Editor.ContentList.CONTAINER_SCROLL.scrollTop();

        new DialogForm({
            url: '/weave/show-current-aspect?weave=' + Editor.getWeaveId() + '&x=' + x + '&y=' + y
        });
    };

    Editor.toggleGrid = function () {
        Editor.ContentList.CONTAINER_CONTENT.toggleClass('grid');

        var label = $('#toolbox-layout-grid');
        var title = label.text();
        $('span', label).text(label.attr('data-title'));
        label.attr('data-title', title);
    };

    Editor.toggleTextChanges = function (toggle) {
        var editor = $('#content').toggleClass('hide-text-changes', !toggle);
        Editor.setSettings('textchanges', !editor.hasClass('hide-text-changes'));
    };

    Editor.toggleOverview = function (toggle) {
        var mainbar = $('#mainbar').toggleClass('expanded', !toggle);
        $('#container-editor').toggleClass('expanded', !toggle);
        $('#sidebar-content').toggleClass('collapsed', !toggle);

        Editor.setSettings('overview', !mainbar.hasClass('expanded'));
        $(window).trigger('resize');
    };

    Editor.getSettings = function () {
        return $.extend({}, {
            toolstab: 0,
            mediatab: -1,
            overview: false,
            textchanges: true,
            gridsize: 50,
            snaptogrid: false,
            showgrid: true,
            sidebar: null
        }, $.parseJSON($.cookie('editor') || '{}'));
    };

    Editor.setSettings = function (key, value) {
        var settings = Editor.getSettings();
        settings[key] = value;

        $.cookie('editor', JSON.stringify(settings));
    };

    Editor.init = function () {
        Editor.initButtons();
        Editor.initTabs();
        Editor.initContextMenu();
        Editor.initBrowserDetection();

        initialized = true;
    };

    Editor.isInitialized = function () {
        return initialized;
    };

    Editor.initContextMenu = function () {
        $(window)
            .bind('keydown', 'ctrl+a meta+a', function () {
                Editor.selectAllItems();
                return false;
            })
            .bind('keydown', 'ctrl+c meta+c ctrl+x meta+x', function (event) {
                Editor.addItemsToClipboard(event.keyCode == 67 ? 'copy' : 'cut');
                return false;
            })
            .bind('keydown', 'ctrl+v meta+v', function () {
                Editor.pasteItemsFromClipboard();
                return false;
            })
            .bind('keydown', 'ctrl+f meta+f', function () {
                Editor.LayerList.layersToFront(Editor.ContentList.selectedChildren());
                return false;
            })
            .bind('keydown', 'ctrl+b meta+b', function () {
                Editor.LayerList.layersToBack(Editor.ContentList.selectedChildren());
                return false;
            })
            .bind('keydown', 'ctrl+l meta+l', function () {
                Editor.lockItems(Editor.ContentList.selectedChildrenIds(), true);
                return false;
            })
            .bind('keydown', 'ctrl+u meta+u', function () {
                Editor.lockItems(Editor.ContentList.selectedChildrenIds(), false);
                return false;
            })
            .bind('keydown', 'del', function () {
                Dialog.removeItems(Editor.ContentList.selectedChildrenIds());
                return false;
            });

        $('#container-editor').contextPopup({
            title: 'context menu',
            events: {
                open: function (event, settings) {
                    var itemId = $(event.target).closest('.tool').data('id');
                    if (!Editor.ContentList.isSelected(itemId)) {
                        Editor.clearItemSelection();
                        Editor.toggleItemSelection(itemId);
                    }

                    settings.items = [];
                    var clipboard = $.parseJSON($.cookie('clipboard'));

                    if (clipboard && clipboard.items.length > 0) {
                        settings.items = [
                            {
                                label: 'paste <span class="gray">(ctrl+v)</span>',
                                icon: '<span class="ui-icon ui-icon-clipboard"></span>',
                                action: function () {
                                    Editor.pasteItemsFromClipboard();
                                }
                            }
                        ];
                    }

                    if (Editor.LayerList.selectedChildren().length > 0) {
                        if (Editor.LayerList.selectedChildren().length == 1) {
                            settings.items = $.merge(settings.items, [
                                {
                                    label: 'edit',
                                    icon: '<span class="ui-icon ui-icon-gear"></span>',
                                    action: function () {
                                        Dialog.editItem(Editor.LayerList.selectedChildren().data('id'), {
                                            context: $this
                                        });
                                    }
                                },
                                null
                            ]);

                            if (Editor.ContentList.selectedChildren().hasClass('tool-picture') && !Editor.isPopover()) {
                                var popoverId = Editor.ContentList.selectedChildren().attr('data-popover');
                                settings.items = $.merge(settings.items, [
                                    {
                                        label: popoverId ? 'goto footnote' : 'add footnote',
                                        icon: '<span class="ui-icon ui-icon-newwin"></span>',
                                        action: function () {
                                            if (popoverId) {
                                                // todo: select footnote tab
                                                Editor.PopoverList.selectChild(popoverId);
                                            } else {
                                                Dialog.addPopover(Editor.LayerList.selectedChildren().data('id'), {
                                                    context: $this
                                                });
                                            }
                                        }
                                    },
                                    null
                                ]);
                            }

                            if (Editor.ContentList.selectedChildren().hasClass('tool-simpletext')) {
                                settings.items = $.merge(settings.items, [
                                    {
                                        label: 'convert to html tool',
                                        icon: '<span class="ui-icon ui-icon-shuffle"></span>',
                                        action: function () {
                                            Dialog.convertItem(Editor.LayerList.selectedChildren().data('id'), {
                                                context: $this
                                            });
                                        }
                                    },
                                    null
                                ]);
                            }
                        }

                        settings.items = $.merge(settings.items, [
                            {
                                label: 'copy <span class="gray">(ctrl+c)</span>',
                                icon: '<span class="ui-icon ui-icon-copy"></span>',
                                action: function () {
                                    Editor.addItemsToClipboard('copy');
                                }
                            },
                            {
                                label: 'cut <span class="gray">(ctrl+x)</span>',
                                icon: '<span class="ui-icon ui-icon-scissors"></span>',
                                action: function () {
                                    Editor.addItemsToClipboard('cut');
                                }
                            },
                            null,
                            {
                                label: 'bring to front <span class="gray">(ctrl+f)</span>',
                                icon: '<span class="ui-icon ui-icon-arrowthickstop-1-n"></span>',
                                action: function () {
                                    Editor.LayerList.layersToFront(Editor.LayerList.selectedChildren());
                                }
                            },
                            {
                                label: 'bring to back <span class="gray">(ctrl+b)</span>',
                                icon: '<span class="ui-icon ui-icon-arrowthickstop-1-s"></span>',
                                action: function () {
                                    Editor.LayerList.layersToBack(Editor.LayerList.selectedChildren());
                                }
                            },
                            null,
                            {
                                label: 'lock <span class="gray">(ctrl+l)</span>',
                                icon: '<span class="ui-icon ui-icon-locked"></span>',
                                action: function () {
                                    Editor.lockItems(Editor.LayerList.selectedChildrenIds(), true);
                                }
                            },
                            {
                                label: 'unlock <span class="gray">(ctrl+u)</span>',
                                icon: '<span class="ui-icon ui-icon-unlocked"></span>',
                                action: function () {
                                    Editor.lockItems(Editor.LayerList.selectedChildrenIds(), false);
                                }
                            },
                            null,
                            {
                                label: 'delete <span class="gray">(del)</span>',
                                icon: '<span class="ui-icon ui-icon-trash"></span>',
                                action: function () {
                                    Dialog.removeItems(Editor.LayerList.selectedChildrenIds());
                                }
                            }
                        ])
                    }

                    return settings;
                }
            }
        })
    };

    Editor.initializeSidebar = function () {
        $('table[data-manager]:visible').each(function () {
            Editor[$(this).attr('data-manager')].draw();
        });

        if (Editor.Overview) {
            Editor.Overview.draw();
        }

        Editor.setSettings('sidebar', $('#navigation-sidebar li.active a').attr('href').substr(1));
    };

    Editor.initTabs = function () {
        Editor.PopoverList.load();

        if (Editor.getSettings().sidebar) {
            jQueryNew('a[href="#' + Editor.getSettings().sidebar + '"]', '#navigation-sidebar').trigger('click');
        }

        $('a', '#navigation-sidebar').click(function (event) {
            const parent = $(this).parent(),
                body = $('body'),
                sidebar = $('#sidebar-content'),
                content = $('.p-2', sidebar),
                collapse = $('li:last span', '#navigation-sidebar');

            if (parent.hasClass('active')) {
                sidebar.height('33px');
                body.addClass('fullscreen');
                collapse.addClass('disabled');
                parent.removeClass('active');
                content.hide();
                event.preventDefault();
                return false;

            }
            sidebar.css('height', 'auto');

            body.removeClass('fullscreen');
            collapse.removeClass('disabled');
            content.show();
        });

        $('#sidebar-commands-grid')
            .bind('click', function (event) {
                event.preventDefault();
                return false;
            })
            .bind('change', function () {
                var gridSize = $(this).val();

                Editor.showGrid(gridSize);

                Editor.setSettings('gridsize', gridSize);

                $(this).closest('li').trigger('click');
            })
            .val(Editor.getSettings().gridsize)
            .trigger('change');

        $('#sidebar-commands-text-changes')
            .bind('change', function () {
                Editor.toggleTextChanges($(this).is(':checked'));
            })
            .prop('checked', Editor.getSettings().textchanges)
            .trigger('change');

        $('#sidebar-commands-overview')
            .bind('change', function () {
                Editor.toggleOverview(true || $(this).is(':checked'));
            })
            .prop('checked', Editor.getSettings().overview)
            .trigger('change');

        $('#sidebar-commands-snaptogrid')
            .bind('change', function () {
                Editor.setSettings('snaptogrid', $(this).is(':checked'));
            })
            .prop('checked', Editor.getSettings().snaptogrid)
            .trigger('change');

        $('#sidebar-commands-showgrid')
            .bind('change', function () {
                if ($(this).is(':checked')) {
                    Editor.showGrid(Editor.getSettings().gridsize);
                } else {
                    Editor.showGrid(0);
                }

                Editor.setSettings('showgrid', $(this).is(':checked'));
            })
            .prop('checked', Editor.getSettings().showgrid)
            .trigger('change');

        $('#sidebar-commands-showtoolborders')
            .bind('change', function () {
                $('#container-editor').toggleClass('show-tool-borders', $(this).is(':checked'));

                Editor.setSettings('showtoolborders', $(this).is(':checked'));
            })
            .prop('checked', Editor.getSettings().showtoolborders || false)
            .trigger('change');
    };

    Editor.showGrid = function (size) {
        var content = $('#content'),
            images = content.css('backgroundImage').split(/, /);

        images[images.length - 2] = 'url(/rc/images/grid/grid_' + size + 'x' + size + '.png)';
        content.css('backgroundImage', images.join(', '));
    };

    Editor.getResearchId = function () {
        return Editor.getUrlParam('research');
    };

    Editor.getWeaveId = function (enforceWeave) {
        if (enforceWeave && Editor.isPopover()) {
            return Editor.getUrlParam('return_to');
        }
        return Editor.getUrlParam('weave');
    };

    Editor.getPopoverId = function () {
        return Editor.getUrlParam('weave');
    };

    Editor.getUrlParam = function (strParamName) {
        var returnVal = [];
        var qString = window.location.search.substr(1, window.location.search.length).split('&');
        if (qString === null) return null;

        for (var i = 0; i < qString.length; i++) {
            if (escape(unescape(qString[i].split("=")[0])) == strParamName) {
                returnVal.push(qString[i].split("=")[1]);
            }
        }

        if (returnVal.length === 0) return null;
        else if (returnVal.length == 1) return returnVal[0];
        else return returnVal;

    };

    Editor.updateCommentLayer = function (effect) {
        var select = $('#toolbox-layout-comments'),
            val = select.val();
        if (val == 'hide') {
            select.nextAll().css('visibility', 'hidden');
        } else {
            select.nextAll().css('visibility', 'visible');
        }
        Editor.ContentList.updateCommentVisibility(val, effect);
        Editor.LayerList.updateCommentVisibility(val, effect);
        Editor.CommentList.updateCommentVisibility(val);
        Editor.Overview.load();
    };

    Editor.previousComment = function () {
        if (!comments.length) {
            return;
        }
        if (!selectedComment) {
            navigateToPreviousWeave();
            return;
        } else if (selectedComment.length == 0) {
            selectedComment = Editor.ContentList.children().filter('[data-tool="comment"]:visible:last');
        } else {
            selectedComment = selectedComment.prevAll('[data-tool="comment"]:visible:first');
        }
        if (selectedComment.length == 0) {
            navigateToPreviousWeave();
            return;
        }
        Editor.selectItem(selectedComment.data('id'));

        function navigateToPreviousWeave() {
            var prevWeaveId = comments[comments.indexOf(Editor.getWeaveId()) - 1];
            if (!prevWeaveId) {
                prevWeaveId = comments[comments.length - 1];
            }
            if (prevWeaveId == Editor.getWeaveId()) {
                selectedComment = [];
                Editor.previousComment()
            } else {
                Editor.navigateToWeave(prevWeaveId);
            }
        }
    };

    Editor.setPopover = function (value) {
        Editor.popover = value;
    };

    Editor.isPopover = function () {
        return Editor.popover;
    };

    Editor.nextComment = function () {
        if (!comments.length) {
            return;
        }
        var filter = '[data-tool="comment"]:visible:first';
        if (!selectedComment || selectedComment.length == 0) {
            selectedComment = Editor.ContentList.children().filter(filter);
        } else {
            selectedComment = selectedComment.nextAll(filter);
        }
        if (selectedComment.length == 0) {
            var nextWeaveId = comments[comments.indexOf(Editor.getWeaveId()) + 1];
            if (!nextWeaveId) {
                nextWeaveId = comments[0];
            }
            if (nextWeaveId == Editor.getWeaveId()) {
                Editor.nextComment()
            } else {
                Editor.navigateToWeave(nextWeaveId);
            }
        } else {
            Editor.selectItem(selectedComment.data('id'));
        }
    };

    Editor.navigateToWeave = function (weaveId) {
        location.search = location.search.replace(Editor.getWeaveId(), weaveId);
    };

    Editor.toggleTextView = function () {
        Editor.TextList.toggle();
    };

    Editor.loadItems = function () {
        Editor.ContentList.load();
    };

    Editor.toggleCommentView = function () {
        Editor.CommentList.toggle();
    };

    Editor.initBrowserDetection = function () {
        var pgwBrowser = jQuery.pgwBrowser();
        $('#content').addClass('browser-' + pgwBrowser.browser.group.toLowerCase()).addClass('os-' + pgwBrowser.os.group.replace(' ', '').toLowerCase());
    };

    Editor.enableHintOnTabClose = function () {
        if (hintOnTabCloseStack++ > 0) {
            return;
        }

        $(window).bind('beforeunload', function (e) {
            var confirmationMessage = 'Your changes will be lost if you leave the editor now. Are you sure you want to exit?';

            (e || window.event).returnValue = confirmationMessage; //Gecko + IE
            return confirmationMessage;                            //Webkit, Safari, Chrome
        });
    };

    Editor.disableHintOnTabClose = function () {
        if (--hintOnTabCloseStack > 0) {
            return;
        }

        $(window).unbind('beforeunload');
    };

    Editor.processEmbedUrl = function (element) {
        var $element = $(element);
        var $dialog = $element.closest('.ui-dialog');
        var $elementApiUrl = $dialog.find('[name="media[urlApi]"]');
        var $elementPreviewUrl = $dialog.find('[name="media[previewUrl]"]');
        var $elementWidth = $dialog.find('[name="style[width]"]');
        var $elementHeight = $dialog.find('[name="style[height]"]');
        var $elementAutoplay = $dialog.find('[name="options[autoplay]"]');
        var $elementLoop = $dialog.find('[name="options[loop]"]');
        var $elementStopOtherPlayers = $dialog.find('[name="options[stopOtherPlayers]"]');
        var $elementError = $element.next('.error').length ? $element.next('.error') : $('<img src="/rc/images/exclamation.png" class="error" data-title="<ul><li>The url could not be parsed. Please check if you have entered a valid url.</ul>" onmouseover="showTitle(this)" onmouseout="hideTitle()">');

        openDialogs.editItem._showLoadingScreen();

        $elementApiUrl.val();
        $elementPreviewUrl.val();

        $.ajax({
            url: '/item/process-embed-url',
            data: {
                url: $element.val()
            },
            context: this,
            success: function (response) {
                var data = JSON.parse(response);

                $('#frm-media-infodatachanged').toggle(!!data.urlApi);

                if (!data.urlApi) {
                    $elementError.insertAfter($element);
                    showTitle($elementError);
                } else {
                    $elementError.remove();
                }

                $elementApiUrl.val(data.urlApi);
                $elementPreviewUrl.val(data.previewUrl);
                if (data.width) {
                    $elementWidth.val(data.width);
                }
                if (data.height) {
                    $elementHeight.val(data.height);
                }

                if (!data.autoplay) {
                    $elementAutoplay.attr('disabled', 'disabled').prop('checked', false);
                } else {
                    $elementAutoplay.removeAttr('disabled');
                }
                if (!data.loop) {
                    $elementLoop.attr('disabled', 'disabled').prop('checked', false);
                } else {
                    $elementLoop.removeAttr('disabled');
                }
                if (!data.stopOtherPlayers) {
                    $elementStopOtherPlayers.attr('disabled', 'disabled').prop('checked', false);
                } else {
                    $elementStopOtherPlayers.removeAttr('disabled');
                }

                openDialogs.editItem._hideLoadingScreen();
            }
        });
    };
}

new Editor();

Array.prototype.indexOf = function (obj, fromIndex) {
    if (fromIndex == null) {
        fromIndex = 0;
    } else if (fromIndex < 0) {
        fromIndex = Math.max(0, this.length + fromIndex);
    }
    for (var i = fromIndex, j = this.length; i < j; i++) {
        if (this[i] == obj)
            return i;
    }
    return -1;
};

function initObjectViewer(container) {
    if (container.css("cursor") == "move") {
        return;
    }

    var objectViewer = new ObjectViewer(container, {
        levels: container.attr("data-levels"),
        startimage: container.attr("data-start-image"),
        rotomatic: container.attr("data-auto-rotate") == "1" ? 80 : 0,
        images: container.attr("data-resources").split(",")
    });

    $(document)
        .bind("mousedown.objectviewer", function (event) {
            if ($(event.target).closest('.meta-media').length > 0) {
                return;
            }

            objectViewer.unload();
            delete(objectViewer);
            container.css("cursor", "pointer");
        })
        .bind("keyup.objectviewer", function (event) {
            if (event.keyCode !== 27) {
                return;
            }

            objectViewer.unload();
            delete(objectViewer);
            container.css("cursor", "pointer");
        });

    container.css("cursor", "move");
}

$(function () {
    jwplayer.key = "SeIeABotC08750h3V7r77b550BnnaXah0rvcdaFiL6U=";
});