function List() {
    if (List.prototype.init) {
        return this;
    }

    List.prototype.init = true;
    List.prototype.SCROLL_DURATION = 750;
    List.prototype.EFFECT_DURATION = 750;
    List.prototype.CLASS_EXPANDED = 'collapsable';
    List.prototype.CLASS_SELECTED = 'active';
    List.prototype.CLASS_LOCKED = 'locked';
    List.prototype.CLASS_LOADING = 'loading';

    List.constant = function (constant) {
        if (constant != constant.toUpperCase()) {
            throw new Error('names of constants have to be passed in upper case!');
        }
        return this.prototype[constant];
    };

    List.prototype.load = function (callback) {
        var $this = this;

        $.ajax({
            url: this.URL,
            context: this,
            success: function () {
                $this._apply('_eLoad', arguments);

                if ($.isFunction(callback)) {
                    callback();
                }
            }
        });
    };

    List.prototype.loadChild = function (id, callback) {
        var $this = this;
        this._apply('loadChildAnimation', [id, true]);

        $.ajax({
            url: this.URL_CHILD + id,
            context: this,
            success: function (data, status, settings) {
                $this._eLoadChild(data, status, settings);
                if (callback) {
                    callback($this.child(id));
                }
            }
        });
    };

    List.prototype.children = function (ids) {
        if (ids) {
            ids = this.listToString(ids);

            return this.children().filter(function () {
                return $.inArray($(this).attr('data-id'), ids) !== -1;
            });
        }

        return $(this.SELECTOR_CHILD, this.CONTAINER_CONTENT);
    };

    List.prototype.child = function (id) {
        return $('[data-id=' + id + ']', this.CONTAINER_CONTENT);
    };

    List.prototype.selectChild = function (id, options) {
        options = options || {};
        this.children().removeClass(this.CLASS_SELECTED);
        options.child = this.child(id).addClass(this.CLASS_SELECTED);
        this._apply('eAfterSelectChild', [options.child.data('id'), options]);
    };

    List.prototype.toggleChildSelection = function (id, options) {
        options = options || {};
        options.child = this.child(id);
        if (options.child.hasClass(this.CLASS_SELECTED)) {
            options.child.removeClass(this.CLASS_SELECTED);
        } else {
            options.child.addClass(this.CLASS_SELECTED);
        }
        this._apply('eAfterToggleChildSelection', [options.child.data('id'), options]);
    };

    List.prototype.isChildCollapsed = function (element) {
        return element.hasClass(this.CLASS_EXPANDED);
    };

    List.prototype.collapseChild = function (id, options) {
        options = options || {};
        this.children().removeClass(this.CLASS_EXPANDED);
        options.child = $(this.child(id)).addClass(this.CLASS_EXPANDED);
        this._apply('eAfterCollapseChild', [options.child.data('id'), options]);
    };

    List.prototype.isSelected = function (id) {
        return this.child(id).hasClass(this.CLASS_SELECTED);
    };

    List.prototype.selectedChild = function () {
        return this.selectedChildren().first();
    };

    List.prototype.selectedChildId = function () {
        return this.selectedChild().data('id');
    };

    List.prototype.notSelectedChilds = function () {
        var $this = this;

        return this.children().map(function () {
            if (!$this.isSelected($(this).data('id'))) {
                return this;
            }
        });
    };

    List.prototype.selectedChildrenIds = function () {
        return this.selectedChildren().map(function () {
            return $(this).data('id');
        })
    };

    List.prototype.notSelectedChildrenIds = function () {
        return this.notSelectedChilds().map(function () {
            return $(this).data('id');
        })
    };

    List.prototype.selectedChildren = function () {
        return $('.' + this.CLASS_SELECTED, this.CONTAINER_CONTENT);
    };

    List.prototype.collapsedChild = function () {
        return $('.' + this.CLASS_EXPANDED, this.CONTAINER_CONTENT).first();
    };

    List.prototype.collapsedChildId = function () {
        return this.collapsedChild().data('id');
    };

    List.prototype.lockChildren = function (ids, options) {
        var $this = this;

        options = options || {};
        options.children = options.children || $this.children(ids);
        if (options.lock === true) {
            options.children.addClass($this.CLASS_LOCKED);
            $this._apply('eLockChildren', [ids, options]);
        } else if (options.lock === false) {
            options.children.removeClass(this.CLASS_LOCKED);
            this._apply('eLockChildren', [ids, options]);
        } else { //toggle
            options.lock = options.children.filter(':not(' + $this.CLASS_LOCKED + ')').length > 0;
            $this.lockChildren(ids, options);
        }
    };

    List.prototype.removeChildren = function (ids) {
        var $this = this;

        this.children(ids).removeClass(this.CLASS_SELECTED).fadeOut(this.EFFECT_DURATION, 'easeInCubic', function () {
            $(this).remove();
            $this._apply('eRemoveChildren', arguments);
        });
    };

    List.prototype.hideChildren = function (ids) {
        this.children(ids).removeClass(this.CLASS_SELECTED).hide();
    };

    List.prototype.showChildren = function (ids) {
        this.children(ids).show();
    };

    List.prototype.updateCommentVisibility = function (value, effect) {
        var $this = this,
            effectDuration = effect ? this.EFFECT_DURATION : 0;

        this.children().each(function () {
            var el = $(this);
            if (el.attr('data-tool') != 'comment') {
                return;
            }

            if (value == '' ||
                value === 'open' && el.attr('data-resolution') == 'open' ||
                value === 'resolved' && el.attr('data-resolution') == 'resolved' ||
                value === el.attr('data-creator-id')) {
                el.fadeIn(effectDuration, 'easeInCubic');
            } else {
                el.removeClass($this.CLASS_SELECTED).fadeOut(effectDuration, 'easeInCubic');
            }
        });
    };

    List.prototype.listToString = function (ids) {
        return $.map(ids, function (value) {
            return value.toString();
        });
    };

    List.prototype = $.extend(List.prototype, {
        _eLoad: function () {
            this._apply('eBeforeLoad', arguments) !== false &&
            this._apply('eLoad', arguments) !== false &&
            this._apply('eAfterLoad', arguments);
        },
        eLoad: function (data) {
            if (this.CONTAINER_LOADING) this.CONTAINER_LOADING.hide();
            $(this.CONTAINER_CONTENT).empty().append(data);
        },
        _eLoadChild: function () {
            this._apply('eBeforeLoadChild', arguments) !== false &&
            this._apply('eLoadChild', arguments) !== false &&
            this._apply('eAfterLoadChild', arguments);
        },
        eLoadChild: function (data) {
            this.child($(data).data('id')).replaceWith(data);
        }
    });

    List.prototype._apply = function (method, args) {
        if ($.isFunction(this[method])) {
            this[method].apply(this, args);
        }
    };
}

new List();