function Overview(init) {
    var $this = this;

    this.CONTAINER_CONTENT = $('#sidebar-overview');
    this.CONTAINER_SIZE = $('#editor-overview-size');
    this.CURRENT_ASPECT = $('#editor-overview-current', this.CONTAINER_CONTENT);
    this.CONTENT = $('img', this.CONTAINER_CONTENT);
    this.CLASS_CHILD_DRAGGING = 'child-dragging';
    this.params = {};

    this.draw = function () {
        if (!this.CONTENT.is(':visible')) {
            return;
        }

        this.triggerLoad();

        this.currentAspect();
    };

    this.load = function () {
        var commentFiler = $('#toolbox-layout-comments').val();

        $this.CONTENT.attr('data-load', $this.CONTENT.data('url') + '&' + $.param({
            width: Editor.ContentList.CONTAINER_CONTENT.innerWidth(),
            height: Editor.ContentList.CONTAINER_CONTENT.innerHeight(),
            commentfilter: commentFiler == null ? '' : commentFiler,
            maxwidth: 252,
            maxheight: 156,
            t: new Date().getTime()
        }));

        if ($this.CONTENT.is(':visible')) {
            $this.triggerLoad();
        }
    };

    this.triggerLoad = function () {
        $this.CONTENT.not('[data-load=""]').attr('src', $this.CONTENT.attr('data-load')).attr('data-load', '');
    };

    this.init = function () {
        $this.CONTAINER_CONTENT.mousedown(function (event) {
            var factor = _getFactor();
            if ($(event.target).attr('id') != $this.CONTAINER_CONTENT.attr('id')) {
                event.layerX += $(event.target).position().left;
                event.layerY += $(event.target).position().top;
            }
            _scrollTo(Math.max(0, event.layerX - $this.CURRENT_ASPECT.innerWidth() / 2) / factor.width,
                Math.max(0, event.layerY - $this.CURRENT_ASPECT.innerHeight() / 2) / factor.height);
        }).click(function () {
            return false;
        }).disableSelection();

        $this.CURRENT_ASPECT.draggable({
            containment: 'parent',
            start: function () {
                $this.CONTAINER_CONTENT.addClass($this.CLASS_CHILD_DRAGGING);
            },
            drag: function () {
                var factor = _getFactor();
                _scrollTo($this.CURRENT_ASPECT.position().left / factor.width,
                    $this.CURRENT_ASPECT.position().top / factor.height);
            },
            stop: function () {
                $this.CONTAINER_CONTENT.removeClass($this.CLASS_CHILD_DRAGGING);
            }
        });

        $this.CONTENT.load(function () {
            $this.currentAspect();
        });
    };

    this.refresh = function () {
        var factor = _getFactor();

        this.currentAspect();

        _scrollTo($this.CURRENT_ASPECT.position().left / factor.width,
            $this.CURRENT_ASPECT.position().top / factor.height);
    };

    this.currentAspect = function (params) {
        $this.params = params || $this.params;
        if ($this.CURRENT_ASPECT.hasClass('ui-draggable-dragging')) return;
        var factor = _getFactor();
        $this.CURRENT_ASPECT.css({
            left: factor.width * $this.params.left,
            top: factor.height * $this.params.top,
            width: factor.width * $this.params.width,
            height: factor.height * $this.params.height
        });
    };

    var _getFactor = function () {
        return {
            width: $this.CONTENT.innerWidth() / $this.params.maxWidth,
            height: $this.CONTENT.innerHeight() / $this.params.maxHeight
        }
    };

    var _scrollTo = function (x, y) {
        Editor.ContentList.CONTAINER_SCROLL.scrollLeft(x).scrollTop(y);
    };

    if (init) this.init();
}

new Overview();