function PopoverList(init) {
    var $this = this,
        items = [];

    this.URL = '/editor/popovers?weave=' + Editor.getWeaveId(true) + '&research=' + Editor.getResearchId();
    this.CONTAINER = $('#_footnotes');
    this.CONTAINER_CONTENT = $('tbody', this.CONTAINER);
    this.CONTAINER_TABLE = $('.table', this.CONTAINER);
    this.CONTAINER_SCROLL = $('.dataTables_scrollBody', this.CONTAINER);
    this.CONTAINER_TOOLBAR = $('.t-menu', this.CONTAINER);
    this.CONTAINER_LOADING = $('.container-loading', this.CONTAINER);
    this.SELECTOR_CHILD = 'tr';

    this.setupDataTable = function () {
        jQueryNew(this.CONTAINER_TABLE).DataTable({
            columnDefs: [
                {
                    orderable: true
                }
            ],
            columns: [
                {type: 'string'}
            ],
            fixedColumns: true,
            bAutoWidth: false,
            preDrawCallback: function () {
                const parent = $this.CONTAINER_TABLE.parent();

                parent.css('height', $(window).height() - $this.CONTAINER_TABLE.offset().top - 15);
            },
            scrollY: $(window).height() - this.CONTAINER_TABLE.offset().top - 15,
            searching: true,
            paging: false,
            stateSave: false,
            info: false,
            classes: {
                sSortAsc: 'sort-asc sortable-column',
                sSortDesc: 'sort-desc sortable-column',
                sSortable: 'sortable-column',
                sFilterInput: 'border-left-none border-top-none border-bottom-none'
            },
            oLanguage: {
                sSearch: '' +
                '   <div class="border bd-default" style="margin-bottom:10px; overflow:hidden; width:357px">\n' +
                '        <ul class="t-menu horizontal open compact">\n' +
                '            <li data-context="global" style="width:237px">_INPUT_</li>\n' +
                '            <li data-context="single" ' + tooltip('Open workspace', 'left') + '><a href="#" onclick="Editor.PopoverList.openEditor(Editor.PopoverList.selectedChildId());return false"><span class="icon mif-wrench"></span></a></li>\n' +
                '            <li data-context="single" ' + tooltip('Edit settings', 'left') + '><a href="#" onclick="Dialog.editPopover(Editor.PopoverList.selectedChildId());return false"><span class="icon mif-pencil"></span></a></li>\n' +
                '            <li data-context="single" ' + tooltip('Delete selected footnote(s)', 'left') + '><a href="#" onclick="Dialog.removePopover(Editor.PopoverList.selectedChildId());return false"><span class="icon mif-bin"></span></a></li>\n' +
                '        </ul>\n' +
                '    </div>',
                sSearchPlaceholder: 'Search...'
            }
        }).on('draw', function () {
            $this._updateButtonState();
        });
    };

    this.items = function() {
        return items;
    };

    this.reload = function () {
        this.dataTable().clear().draw();
        this.load();
    };

    this.openEditor = function (id) {
        if (!id) {
            return;
        }

        location.href = '/editor?research=' + Editor.getResearchId() + '&weave=' + id + '&return_to=' + Editor.getWeaveId(Editor.isPopover()) + '#';
    };

    this.eAfterLoad = function () {
        items = [];

        this.children().each(function () {
            $this.initChild(this);

            items.push({
                id: $(this).attr('data-id'),
                label: $('td', this).text()
            });
        });

        if (Editor.isPopover()) {
            $this.selectChild(Editor.getPopoverId());
        }

        $this._updateButtonState();
    };

    this.eAfterSelectChild = function (id, options) {
        this._updateButtonState();

        const item = this.child(id);
        if (options.context === this || item.length === 0) {
            return;
        }

        jQueryNew($this.CONTAINER_SCROLL).stop().scrollTo(item.position().top, 0);
    };

    this.eAfterToggleChildSelection = function () {
        this._updateButtonState();
    };

    this.initChild = function (item) {
        item = $(item);

        const itemId = item.data('id');

        item
            .click(function (event) {
                const options = {context: $this},
                    selectedChildren = $this.selectedChildren();

                if (event.ctrlKey || event.metaKey || (selectedChildren.length === 1 && selectedChildren.is(item))) {
                    $this.toggleChildSelection(itemId, options);
                    Editor.ContentList.toggleChildSelection(item.data('tool-id'), options);
                } else {
                    $this.selectChild(itemId, options);
                    Editor.ContentList.selectChild(item.data('tool-id'), options);
                }
            })
            .dblclick(function () {
                $this.openEditor(itemId);
            })
            .disableSelection();

        if (itemId == Editor.getWeaveId()) {
            item.addClass('highlight');
        }
    };

    if (!PopoverList.prototype.init) {
        PopoverList.prototype = new TableList(init);
        PopoverList.prototype.constructor = PopoverList;
    }
}

new PopoverList();