function WorkList(init) {
    var $this = this;
    let children = [];

    this.URL = '/editor/works?research=' + Editor.getResearchId();
    this.URL_CHILD = '/editor/work-children?research=' + Editor.getResearchId() + '&work=';
    this.CONTAINER = $('#_works');
    this.CONTAINER_CONTENT = $('tbody', this.CONTAINER);
    this.CONTAINER_TABLE = $('.table', this.CONTAINER);
    this.CONTAINER_SCROLL = $('.dataTables_scrollBody', this.CONTAINER);
    this.CONTAINER_TOOLBAR = $('.t-menu', this.CONTAINER);
    this.CONTAINER_LOADING = $('.container-loading', this.CONTAINER);
    this.SELECTOR_CHILD = '.work';

    this.setupDataTable = function () {
        jQueryNew(this.CONTAINER_TABLE).DataTable({
            columns: [
                {width: 40, orderable: false},
                {type: 'string'}
            ],
            fixedColumns: true,
            bAutoWidth: false,
            preDrawCallback: function () {
                const parent = $this.CONTAINER_TABLE.parent();

                parent.css('height', $(window).height() - $this.CONTAINER_TABLE.offset().top - 15);
            },
            scrollY: $(window).height() - this.CONTAINER_TABLE.offset().top - 15,
            searching: true,
            paging: false,
            stateSave: false,
            info: false,
            classes: {
                sSortAsc: 'sort-asc sortable-column',
                sSortDesc: 'sort-desc sortable-column',
                sSortable: 'sortable-column',
                sFilterInput: 'border-left-none border-top-none border-bottom-none'
            },
            oLanguage: {
                sSearch: '' +
                '   <div class="border bd-default" style="margin-bottom:10px; overflow:hidden; width:357px; height: 36px;">\n' +
                '        <ul class="t-menu horizontal open compact">\n' +
                '            <li data-context="global" style="width:198px">_INPUT_</li>\n' +
                '            <li data-context="global"><a href="#" onclick="Dialog.addWork(Editor.getResearchId());return false" data-role="hint" data-hint-position="bottom" data-hint-background="bg-black" data-hint-color="fg-white" data-hint-mode="2" data-hint-text="Add work"><span class="icon mif-plus" style="font-size: 14px; margin-top: 4px;"></span></a></li>' +
                '            <li data-context="single"><a href="#" onclick="Dialog.showWork(Editor.WorkList.selectedChildId());return false" data-role="hint" data-hint-position="bottom" data-hint-background="bg-black" data-hint-color="fg-white" data-hint-mode="2" data-hint-text="Preview"><span class="icon mif-eye"></span></a></li>' +
                '            <li data-context="single"><a href="#" onclick="Dialog.editWork(Editor.WorkList.selectedChildId());return false" data-role="hint" data-hint-position="bottom" data-hint-background="bg-black" data-hint-color="fg-white" data-hint-mode="2" data-hint-text="Edit"><span class="icon mif-pencil"></span></a></li>' +
                '            <li data-context="multiple"><a href="#" onclick="Dialog.removeWorks(Editor.WorkList.selectedChildrenIds());return false" data-role="hint" data-hint-position="bottom" data-hint-background="bg-black" data-hint-color="fg-white" data-hint-mode="2" data-hint-text="Delete"><span class="icon mif-bin"></span></a></li>' +
                '        </ul>\n' +
                '    </div>',
                sSearchPlaceholder : 'Search...'
            }
        }).on('draw', function () {
            $this._updateButtonState();
        });
    };

    this.eAfterLoad = function () {
        this.children().each(function () {
            $this.initChild(this);
        });
    };

    this.eLoadChild = function (data) {
        var table = $('<table class="table table-border row-hover striped"><thead></thead><tbody></tbody></table>'),
            container = $('tbody', table);

        $(data.files).each(function () {
            let file = this,
                row = $('<tr ' +
                    'class="files" ' +
                    'data-id="' + this.id + '" ' +
                    'data-tool="' + this.tool + '" ' +
                    'data-size-width="' + this.mediaWidth + '" ' +
                    'data-size-height="' + this.mediaHeight + '" ' +
                    '>' +
                    '<td class="mif-' + this.toolIcon + ' ' + this.missingMediaClass + ' mif-2x" style="width:40px" data-id="' + this.id + '" data-tool="' + this.tool + '"></td>' +
                    '<td>' + this.title + '</td>' +
                    '<td style="width:85px">' + this.created + '</td>' +
                    '<td style="width:85px;text-align:right">' + this.details + '</td>');

            $('td:first', row).addClass('tool-draggable').draggable({
                helper: function () {
                    return $('<div id="tool-dragging"></div>').css({
                        width: row.attr('data-size-width'),
                        height: row.attr('data-size-height')
                    });
                },
                distance: 5,
                drag: Editor.ContentList.adjustContainerSize,
                cursorAt: {
                    left: 3,
                    top: 3
                },
                opacity: 0.7,
                appendTo: Editor.ContentList.CONTAINER_CONTENT
            });

            $('td:not(:first)', row).dblclick(function () {
                Dialog.editSimpleMedia(file.id);
            });

            container.append(row);
        });

        const row = this.dataTable().row(this.child(data.work));
        if (row) {
            row.data(['', data.workTitle]);
        }

        children[data.work] = table;
    };

    this.eAfterLoadChild = function (data) {
        this.loadChildAnimation(data.work, false);
    };

    this.collapseChild = function (id, options) {
        this.children().each(function () {
            expandChild(this, false);
        });

        List.prototype.collapseChild.call(this, id, options);

        expandChild($(this.child(id)), true);
    };

    this.loadChildAnimation = function (id, show) {
        let hitarea = this.child(id).children('.hitarea').toggleClass(this.CLASS_LOADING, show);

        if (show) {
            hitarea.removeClass('mif-plus mif-minus');
        } else {
            this.collapseChild(this.collapsedChildId());
        }
    };

    this.eAfterSelectChild = function (id, options) {
        this._updateButtonState();
    };

    this.eAfterToggleChildSelection = function () {
        this._updateButtonState();
    };

    this.eRemoveChildren = function () {
        this._updateButtonState();
    };

    this.initChild = function (item) {
        (item = $(item)).click(function (event) {
            const selectedChildren = $this.selectedChildren();

            if (event.ctrlKey || event.metaKey || (selectedChildren.length === 1 && selectedChildren.is(item))) {
                $this.toggleChildSelection(item.data('id'));
            } else {
                $this.selectChild(item.data('id'));
            }
        });

        $('td:last', item).dblclick(function () {
            Dialog.editWork(item.data('id'));
        });

        if (item.data('childcount') > 0) {
            const hitarea = $('.hitarea', item)
                .click(function (event) {
                    const itemId = item.data('id'),
                        open = ($this.collapsedChildId() !== itemId);

                    $this.children().each(function () {
                        const el = $(this);

                        if (!$this.isChildCollapsed(el)) {
                            return;
                        }

                        expandChild(el, false);

                        $('.hitarea', el).addClass('mif-plus').removeClass('mif-minus');
                    });

                    $this.collapseChild(null);

                    if (open) {
                        $this.selectChild(itemId);
                        $this.collapseChild(itemId);

                        const expandCallback = function () {
                            expandChild($this.collapsedChild(), true);

                            hitarea.removeClass('mif-plus').addClass('mif-minus');
                        };

                        if (!children[itemId]) {
                            $this.loadChild(itemId, expandCallback);
                        } else {
                            expandCallback();
                        }
                    }

                    return false;
                });
        }
    };

    function expandChild(el, show) {
        el = $(el);

        const row = jQueryNew($this.CONTAINER_TABLE).DataTable().row(el);
        if (!row) {
            return;
        }

        const child = row.child(children[el.data('id')], 'details');
        if (!child) {
            return;
        }

        const hitarea = $('.hitarea', el);
        if (show) {
            child.show();
            hitarea.addClass('mif-minus').removeClass('mif-plus');
        } else {
            child.hide();
            hitarea.addClass('mif-plus').removeClass('mif-minus');
        }
    }

    if (!WorkList.prototype.init) {
        WorkList.prototype = new TableList(init);
        WorkList.prototype.constructor = WorkList;
    }
}

new WorkList();