// 3DNP 1.2c
// Copyright (C) 2011 Thorsten Schlueter - www.thoro.de
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

function ObjectViewer(image, options) {
    var defaultOptions = {
        images: [],
        levels: 1, // number of Y axis levels
        startimage: 1, // defines the starting axis
        viewmode: 'object', // camera mode ('object'/'camera')
        friction: 0.5, // rotation friction (0 to 1)- default: 0.5
        rotomatic: 80, // automatic rotation speed - negative or positive value, smaller values = faster rotation, 0 disables
        rotoresume: 2, // the time in seconds 3DNP waits before resuming the autorotation, 0 disables
        keycodes: [119, 100, 115, 97] // keycodes for keyboard input (up,right,down,left) - default is [119,97,115,100] for 'W/D/S/A' keys, another example: [56,54,50,52] for '8/6/2/4' on num block
    };

    options = $.extend(defaultOptions, options || {});

    var total = options.images.length,
        perLevel = total / options.levels,
        time = 40,
        timer = null,
        frictionx = (1 - options.friction),
        frictiony = (1 - options.friction) / 1.5,
        multiplicator = (options.viewmode == 'object') ? 1 : -1,
        imageNumber = options.startimage,
        imageList = [],
        mouseCoords = {x: 0, y: 0};

    for (var i = 1; i <= total; i++) {
        imageList[i] = new Image();
        imageList[i].src = options.images[i - 1];
    }

    $(image).disableSelection().bind('dragstart.objectviewer', function () {
        return false
    });

    updateImage();
    if ((options.rotomatic != 0)) {
        autorotate();
    }

    $(document)
        .bind('mousemove.objectviewer', function (event) {
            mouseCoords.x = event.pageX;
            mouseCoords.y = event.pageY;
        })
        .bind('keypress.objectviewer', function keypressed(event) {
            var code = event.keyCode || event.which;
            if (options.keycodes.indexOf(code) == -1) {
                return;
            }

            clearTimeout(timer);

            var position = getPosition(),
                positionNew = getPosition();

            if (code == options.keycodes[0]) positionNew.y = position.y + multiplicator;
            if (code == options.keycodes[1]) positionNew.x = position.x - multiplicator;
            if (code == options.keycodes[2]) positionNew.y = position.y - multiplicator;
            if (code == options.keycodes[3]) positionNew.x = position.x + multiplicator;

            handleOutOfBoundValues(positionNew);
            updateImageNumber(positionNew);
            updateImage();
        })
        .bind('keyup.objectviewer mouseup.objectviewer', function () {
            clearTimeout(timer);

            if ((options.rotomatic != 0) && (options.rotoresume != 0)) {
                setTimeout(autorotate, Math.abs(options.rotoresume * 1000));
            }
        })
        .bind('keydown.objectviewer mousedown.objectviewer', function () {
            rotate();
        });

    function autorotate() {
        clearTimeout(timer);

        var position = getPosition(),
            positionNew = getPosition();

        if (options.rotomatic > 0) {
            positionNew.x = position.x + 1;
        } else {
            positionNew.x = position.x - 1;
        }

        handleOutOfBoundValues(positionNew);
        updateImageNumber(positionNew);
        updateImage();

        timer = setTimeout(autorotate, Math.abs(options.rotomatic));
    }


    function rotate(last) {
        clearTimeout(timer);

        var position = getPosition(),
            diff = {
                x: last ? last.restX * frictionx + last.x - mouseCoords.x : 0,
                y: last ? last.restY * frictiony + last.y - mouseCoords.y : 0
            },
            diffFull = {
                x: Math.round(diff.x / 8),
                y: Math.round(diff.y / 12)
            },
            positionNew = {
                x: position.x + diffFull.x * multiplicator,
                y: position.y + diffFull.y * multiplicator
            }

        handleOutOfBoundValues(positionNew);
        updateImageNumber(positionNew);
        updateImage();

        coordX = mouseCoords.x;
        coordY = mouseCoords.y;
        timer = setTimeout(function () {
            rotate({
                x: coordX,
                y: coordY,
                restX: diff.x - diffFull.x,
                restY: diff.y - diffFull.y
            });
        }, time);
    }

    function updateImage() {
        if (!imageList[imageNumber]) {
            return;
        }
        $(image).attr('src', imageList[imageNumber].src);
    }

    function updateImageNumber(position) {
        imageNumber = ((position.y - 1) * perLevel) + position.x;
    }

    function getPosition() {
        var positionY = Math.ceil(imageNumber / perLevel),
            positionX = imageNumber - ((positionY - 1) * perLevel);
        return {
            x: positionX,
            y: positionY
        };
    }

    function handleOutOfBoundValues(position) {
        if (position.x < 1)  position.x = perLevel;
        if (position.x > perLevel)  position.x = 1;
        if (position.y < 1) position.y = 1;
        if (position.y > options.levels) position.y = options.levels;
    }

    this.unload = function() {
        $(document).unbind('.objectviewer');
        $(image).unbind('.objectviewer');
        clearTimeout(timer);
    }
}