<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Model\Application\Form;

use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;
use Rc\Domain\Model\Core\FormValuesTrait;

/**
 * @ODM\EmbeddedDocument
 */
abstract class Element
{
    use FormValuesTrait;

    const FORM_TYPE = null;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $identifier;

    /**
     * @return string
     */
    public function getIdentifier()
    {
        return $this->identifier;
    }

    /**
     * @param string $identifier
     */
    public function setIdentifier($identifier)
    {
        $this->identifier = $identifier;
    }

    /**
     * @return string
     */
    abstract public function getName();

    /**
     * @return string
     */
    public function getFormType()
    {
        return static::FORM_TYPE;
    }

    /**
     * @return array
     */
    public function getFormOptions()
    {
        return [];
    }

    /**
     * @return array
     */
    public function toArray()
    {
        $properties = [];
        foreach ($this as $property => $value) {
            $properties[$property] = $value;
        }
        return $properties;
    }
}