<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Model\Application\Form;

use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;

/**
 * @ODM\EmbeddedDocument
 */
abstract class FormElement extends Element
{
    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $label;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $description;

    /**
     * @var bool
     * @ODM\Field(type="boolean")
     */
    protected $required;

    /**
     * @return array
     */
    public static function getElements()
    {
        return [
            'short-text'    => 'form-element.short-text',
            'text'          => 'form-element.text',
            'number'        => 'form-element.number',
            'range'         => 'form-element.range',
            'date'          => 'form-element.date',
            'money'         => 'form-element.money',
            'checkbox'      => 'form-element.checkbox',
            'checkbox-list' => 'form-element.checkboxlist',
            'select'        => 'form-element.select',
            'image'         => 'form-element.image',
            'pdf'           => 'form-element.pdf',
        ];
    }

    /**
     * @return string
     */
    public function getName()
    {
        $parts = explode('\\', get_called_class());
        return strtolower(substr(end($parts), 0, -11));
    }

    /**
     * @return array
     */
    public function getFormOptions()
    {
        return [
            'label'    => $this->label,
            'info'     => preg_replace('~https?://[^ ]*~i', '<a href=\'$0\' target=\'_blank\'>$0</a>', $this->description),
            'required' => $this->required,
        ];
    }

    /**
     * @return string
     */
    public function getLabel()
    {
        return $this->label;
    }

    /**
     * @param string $label
     */
    public function setLabel($label)
    {
        $this->label = $label;
    }

    /**
     * @return string
     */
    public function getDescription()
    {
        return $this->description;
    }

    /**
     * @param string $description
     */
    public function setDescription($description)
    {
        $this->description = $description;
    }

    /**
     * @return bool
     */
    public function isRequired()
    {
        return $this->required;
    }

    /**
     * @param bool $required
     */
    public function setRequired($required)
    {
        $this->required = $required;
    }

    /**
     * @return bool
     */
    public function requiresUpload()
    {
        return false;
    }
}