<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Model\Application\Form\FormElement;

use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;
use Rc\Domain\Model\Application\Form\FormElement;
use Rc\Domain\Model\Core\DateTimeTrait;

/**
 * @ODM\EmbeddedDocument
 */
class DateFormElement extends FormElement
{
    use DateTimeTrait;

    const FORM_TYPE = 'datePicker';

    /**
     * @var \DateTime
     * @ODM\Field(type="date")
     */
    protected $value;

    /**
     * @inheritDoc
     */
    public function getFormOptions()
    {
        $options = [
            'validators' => [
                'Date',
            ],
        ];
        return $options + parent::getFormOptions();
    }

    /**
     * @param string|bool $format
     * @return \DateTime|string
     */
    public function getValue($format = true)
    {
        return $this->formatDateTime($this->value, $format);
    }

    /**
     * @param \DateTime $date
     */
    public function setValue($date)
    {
        $this->value = $this->castToDateTime($date);
    }

}