<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Model\Application\Form;

use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;

/**
 * @ODM\EmbeddedDocument
 */
class FormElementValueContainer
{
    /**
     * @var FormElement
     * @ODM\EmbedOne
     */
    protected $element;

    /**
     * @var object
     * @ODM\EmbedOne
     */
    protected $object;

    /**
     * @var mixed
     * @ODM\Field(type="raw")
     */
    protected $simpleValue;

    /**
     * @param FormElement $element
     * @param mixed $value
     */
    public function __construct(FormElement $element, $value)
    {
        $this->element = $element;
        if (is_object($value)) {
            $this->object = $value;
        } else if (is_array($value)) {
            $this->simpleValue = $value;
        } else {
            $this->simpleValue = (string)$value;
        }
    }

    /**
     * @return FormElement
     */
    public function getElement()
    {
        return $this->element;
    }

    /**
     * @return mixed
     */
    public function getValue()
    {
        if (isset($this->object)) {
            return $this->object;
        }
        return $this->simpleValue;
    }

    /**
     * @return bool
     */
    public function hasValue()
    {
        return $this->getValue() != '';
    }
}