<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Model\Application;

use DateTime;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;
use Rc\Domain\Model\Application\Generic\MetaDataAwareTrait;
use Rc\Domain\Model\Core\FormValuesTrait;
use Rc\Domain\Model\Research\Application;
use Rc\Domain\Model\User;

/**
 * @ODM\EmbeddedDocument
 */
class Review
{
    use FormValuesTrait;
    use MetaDataAwareTrait;

    const STATUS_PROGRESS = 'progress';
    const STATUS_SUBMITTED = 'submitted';

    /**
     * @var Application
     * @ODM\ReferenceOne(targetDocument="Rc\Domain\Model\Research\Application", storeAs="dbRef")
     */
    protected $application;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $status = self::STATUS_PROGRESS;

    /**
     * @var DateTime
     * @ODM\Field(type="date")
     */
    protected $submitted;

    /**
     * @var User
     * @ODM\ReferenceOne(targetDocument="Rc\Domain\Model\User", storeAs="dbRef")
     */
    protected $author;

    /**
     * @param Application $application
     */
    public function __construct(Application $application)
    {
        $this->application = $application;
        $this->metaData = new ArrayCollection();
    }

    /**
     * @return string
     */
    public function getStatus()
    {
        return $this->status;
    }

    /**
     * @param string $status
     */
    public function setStatus($status)
    {
        $this->status = (string)$status;
    }

    /**
     * @param string $status
     * @return bool
     */
    public function hasStatus($status)
    {
        return $this->status == $status;
    }

    /**
     * @return DateTime
     */
    public function getSubmitted()
    {
        return $this->submitted;
    }

    /**
     * @param DateTime $submitted
     */
    public function setSubmitted($submitted)
    {
        $this->submitted = $submitted;
    }

    /**
     * @return User
     */
    public function getAuthor()
    {
        return $this->author;
    }

    /**
     * @param User $author
     */
    public function setAuthor(User $author)
    {
        $this->author = $author;
    }

    /**
     * @return ApplicationForm
     */
    protected function getMetadataForm()
    {
        return $this->application->getProgram()->getReviewForm();
    }
}
