<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Model\Core\Acl;

use Rc\Domain\Model\User;

interface AclAwareInterface
{

    const TYPE_READ = 'read';
    const TYPE_WRITE = 'write';

    /**
     * @param array $group
     * @param string $context
     */
    public function addReadAcl(array $group, $context);

    /**
     * @param array $group
     * @param string $context
     */
    public function addWriteAcl(array $group, $context);

    /**
     * @param string $context
     * @return void
     */
    public function removeReadAcl($context);

    /**
     * @param string $context
     * @return void
     */
    public function removeWriteAcl($context);

    /**
     * @param string $context
     * @return void
     */
    public function removeAcl($context);

    /**
     * @return void
     */
    public function removeAllReadAcl();

    /**
     * @return void
     */
    public function removeAllWriteAcl();

    /**
     * @return void
     */
    public function removeAllAcl();

    /**
     * @return void
     */
    public function convertAllWriteAclToReadAcl();

    /**
     * @param User $user
     * @param string $type
     * @return bool
     */
    public function isAllowed(User $user = null, $type = self::TYPE_WRITE);

    /**
     * @param User $owner
     * @return bool
     */
    public function isOwner(User $owner);

    /**
     * @param User $owner
     * @return bool
     */
    public function wasOwner(User $owner);

}
