<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */
namespace Rc\Domain\Model\Core\Acl;

use Rc\Domain\Model\ModelInterface;

final class AclContext
{
    /** roles */
    const OWNER = 'owner';
    const AUTHOR = 'author';
    const COAUTHOR = 'coauthor';
    const CONTRIBUTOR = 'contributor';
    const SUPERVISOR = 'supervisor';
    const EDITOR = 'editor';
    const REVIEWER = 'reviewer';
    const DOUBLE_BLIND_REVIEWER = 'double-blind-reviewer';
    const EXTERNAL_READER = 'external-readers';

    /** status */
    const PUBLICATION = 'publication';
    const REVIEW = 'review';

    /** share */
    const ACCESS_LEVEL = 'access-level';

    /** relations */
    const APPLICATION = 'application:%s';

    /**
     * @return string
     */
    public static function OWNER()
    {
        return self::OWNER;
    }

    /**
     * @return string
     */
    public static function AUTHOR()
    {
        return self::AUTHOR;
    }

    /**
     * @return string
     */
    public static function COAUTHOR()
    {
        return self::COAUTHOR;
    }

    /**
     * @return string
     */
    public static function CONTRIBUTOR()
    {
        return self::CONTRIBUTOR;
    }

    /**
     * @return string
     */
    public static function SUPERVISOR()
    {
        return self::SUPERVISOR;
    }

    /**
     * @return string
     */
    public static function EDITOR()
    {
        return self::EDITOR;
    }

    /**
     * @return string
     */
    public static function REVIEWER()
    {
        return self::REVIEWER;
    }

    /**
     * @return string
     */
    public static function DOUBLE_BLIND_REVIEWER()
    {
        return self::DOUBLE_BLIND_REVIEWER;
    }

    /**
     * @return string
     */
    public static function EXTERNAL_READER()
    {
        return self::EXTERNAL_READER;
    }

    /**
     * @return string
     */
    public static function PUBLICATION()
    {
        return self::PUBLICATION;
    }

    /**
     * @return string
     */
    public static function REVIEW()
    {
        return self::REVIEW;
    }

    /**
     * @return string
     */
    public static function ACCESS_LEVEL()
    {
        return self::ACCESS_LEVEL;
    }

    /**
     * @param ModelInterface|int $entity
     * @return string
     */
    public static function APPLICATION($entity)
    {
        return self::format(self::APPLICATION, $entity);
    }

    /**
     * @param string $context
     * @param ModelInterface|int $entity
     * @return string
     */
    protected static function format($context, $entity)
    {
        if ($entity instanceof ModelInterface) {
            return sprintf($context, $entity->getId());
        } else {
            return sprintf($context, $entity);
        }
    }
}
