<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Model\Core;

trait DateTimeTrait
{

    /**
     * @param \DateTime $date
     * @param string|bool $format
     * @return \DateTime|string
     */
    protected function formatDateTime(\DateTime $date = null, $format)
    {
        if (!isset($date)) {
            return null;
        }
        if ($format === false) {
            return $date;
        }
        if ($format === true) {
            $format = \Zend_Registry::get('Zend_Locale')->getDefaultDateFormatForPhp();
        }
        return $date->format($format);
    }

    /**
     * @todo: remove replace after migration
     * @param \DateTime|string|null $date
     * @param bool $replace
     * @return \DateTime|null
     */
    protected function castToDateTime($date = null, $replace = true)
    {
        if (empty($date) || $date == '[]') {
            return null;
        }
        if ($date instanceof \DateTime) {
            return $date;
        }
        if (is_numeric($date) && strlen($date) == 10) {
            return new \DateTime('@' . $date);
        }
        if ($replace) {
            $date = str_replace(['/', '[', ']'], ['.', '', ''], $date);
            if (empty($date)) {
                return null;
            }

            if (preg_match('~^\d{4}$~', $date)) {
                $date = '01.01.' . $date;
            } elseif (preg_match('~^\d{2}\.\d{2}\.\d{2}$~', $date)) {
                $parts = explode('.', $date);
                if ($parts[2] < 20) {
                    $parts[2] = '20' . $parts[2];
                } else {
                    $parts[2] = '19' . $parts[2];
                }
                $date = implode('.', $parts);
            } elseif (preg_match('~^\d{2}\.\d{2}\.0\d{2}$~', $date)) {
                $parts = explode('.', $date);
                $parts[2] = '2' . $parts[2];
                $date = implode('.', $parts);
            } elseif (preg_match('~^.*\d{4}-\d{4}$~', $date)) {
                $date = '01.01.' . explode('-', $date)[1];
            } elseif (preg_match('~^\d{8}$~', $date)) {
                $date = substr($date, 0, 2) . '.' . substr($date, 2, 2) . '.' . substr($date, 4, 4);
            } elseif ($date == '2012.13') {
                $date = '20.12.2013';
            } elseif ($date == '04.0472014') {
                $date = '04.04.2014';
            }
        }
        return new \DateTime($date);
    }
}