<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Model\Core;

use Rc\Domain\Model\Translation;
use Rc\Domain\Security\Text;
use Rc\Utility\Strings;

trait FormValuesTrait
{

    /**
     * @param array $values
     * @return $this
     */
    public function setValues(array $values)
    {
        foreach ($values as $attribute => $value) {
            $method = 'set' . Strings::toCamelCase($attribute, '-', true);
            if (!method_exists($this, $method)) {
                continue;
            }

            $value = $this->stripJavascript($value);
            if (!empty($value) && is_array($value) && empty(array_filter(array_keys($value), 'is_numeric'))) { // multiple languages
                $this->$method(new Translation($value));
            } elseif ($method == 'setTitle') {
                $this->$method(new Translation($value));
            } else {
                $this->$method($value);
            }
        }

        return $this;
    }

    /**
     * @param string|array|null $input
     * @return string|array
     */
    protected function stripJavascript($input)
    {
        if (!isset($input)) {
            return $input;
        }

        if (is_string($input) || is_array($input)) {
            return Text::removeJavascriptTags($input);
        }

        return $input;
    }
}