<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */
namespace Rc\Domain\Model\Core;

use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;
use Rc\Domain\Model\AbstractModel;
use Rc\Domain\Model\ModelInterface;
use Rc\Domain\Model\User;

/**
 * @ODM\Document
 * @ODM\Index(keys={"document._id":"asc"})
 */
class Revision implements ModelInterface
{
    use DateTimeTrait;

    /**
     * @var string
     * @ODM\Field(type="string")
     * @ODM\Id()
     */
    protected $uuid;

    /**
     * @var int
     * @ODM\Field(type="int")
     */
    protected $id;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $label;

    /**
     * @var \DateTime
     * @ODM\Field(type="date")
     */
    protected $created;

    /**
     * @var User
     * @ODM\ReferenceOne(targetDocument="Rc\Domain\Model\User", storeAs="dbRef")
     */
    protected $user;

    /**
     * @var AbstractModel
     * @ODM\EmbedOne()
     */
    protected $document;

    /**
     * @param ModelInterface $document
     * @param User $user
     * @param string $label
     */
    public function __construct(ModelInterface $document, User $user, $label = null)
    {
        $this->created = new \DateTime();
        $this->user = $user;
        $this->document = $document;
        $this->label = $label;
    }

    /**
     * @return string
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @param string $id
     */
    public function setId($id)
    {
        $this->id = $id;
    }

    /**
     * @return string
     */
    public function getUuid()
    {
        return $this->uuid;
    }

    /**
     * @param string $uuid
     */
    public function setUuid($uuid)
    {
        $this->uuid = $uuid;
    }

    /**
     * @return string
     */
    public function getLabel()
    {
        return $this->label;
    }

    /**
     * @param string $label
     */
    public function setLabel($label)
    {
        $this->label = $label;
    }

    /**
     * @param string|bool $format
     * @return \DateTime|string
     */
    public function getCreated($format = true)
    {
        return $this->formatDateTime($this->created, $format);
    }

    /**
     * @param string|bool $format
     * @return \DateTime|string
     */
    public function getModified($format = true)
    {
        return $this->formatDateTime($this->created, $format);
    }

    /**
     * @return User
     */
    public function getUser()
    {
        return $this->user;
    }

    /**
     * @return AbstractModel
     */
    public function getDocument()
    {
        return $this->document;
    }

    /**
     * @deprecated
     * @param array $values
     * @return $this
     */
    public function setValues(array $values)
    {
        throw new \LogicException('unsupprted method call');
    }

    /**
     * @deprecated
     * @return array
     */
    public function getValues()
    {
        throw new \LogicException('unsupprted method call');
    }

    /**
     * @deprecated
     * @return array
     */
    public function getDeleted()
    {
        throw new \LogicException('unsupprted method call');
    }

    /**
     * Executes this method before removing revision instance to avoid side effects!
     */
    public function prepareForDeletion()
    {
        $this->document = null;
    }

    /**
     * @return string
     */
    public function __toString()
    {
        return '';
    }

}