<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */
namespace Rc\Domain\Model\Media;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;
use Rc\Domain\Model\MediaInterface;
use Rc\Domain\Model\Research\MediaContainer;
use Rc\Domain\Model\Research\MediaContainerInterface;

trait MediaAwareTrait
{
    /**
     * @var MediaInterface
     * @ODM\ReferenceOne(targetDocument="Rc\Domain\Model\Media", cascade={"persist"}, storeAs="dbRef")
     */
    protected $thumb;

    /**
     * @var MediaInterface[]|ArrayCollection
     * @ODM\ReferenceMany(targetDocument="Rc\Domain\Model\Media", cascade={"persist"}, limit="1", storeAs="dbRef")
     */
    protected $mediaList;

    /**
     * @return MediaContainerInterface
     */
    public function getMediaContainer()
    {
        return new MediaContainer($this->mediaList->toArray(), $this->thumb);
    }

    /**
     * @return boolean
     */
    public function hasThumb()
    {
        return isset($this->thumb);
    }

    /**
     * @return MediaInterface
     */
    public function getThumb()
    {
        return $this->thumb;
    }

    /**
     * @param MediaInterface $thumb
     */
    public function setThumb(MediaInterface $thumb = null)
    {
        $this->thumb = $thumb;
    }

    /**
     * @return boolean
     */
    public function hasMedia()
    {
        return isset($this->mediaList) && !$this->mediaList->isEmpty();
    }

    /**
     * @return boolean
     */
    public function hasResourceFiles()
    {
        if (!$this->hasMedia()) {
            return false;
        }
        if (!$this->getMedia()->hasMaster() && !$this->getMedia()->hasDerivatives()) {
            return false;
        }

        return true;
    }

    /**
     * @return MediaInterface|null
     */
    public function getMedia()
    {
        return $this->mediaList->first() ?: null;
    }

    /**
     * @param MediaInterface $media
     */
    public function setMedia(MediaInterface $media = null)
    {
        $this->mediaList->clear();
        if (isset($media)) {
            $this->mediaList->add($media);
        }
    }

    /**
     * @return MediaInterface[]
     */
    public function getMediaList()
    {
        return $this->mediaList->toArray();
    }

    /**
     * @param MediaInterface[] $mediaList
     */
    public function setMediaList(array $mediaList)
    {
        $this->mediaList = new ArrayCollection($mediaList);
    }

    /**
     * @return Dimension
     */
    public function getMediaResolution()
    {
        if ($this->hasMedia()) {
            return $this->getMedia()->getResolution();
        }

        if ($this->hasThumb()) {
            return $this->getThumb()->getResolution();
        }

        return null;
    }
}