<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Model\Research;

use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;
use MyLocal;
use Rc\Domain\Model\Portal;
use Rc\Domain\Model\Research;

/**
 * @ODM\Document
 */
class Degree extends Research implements MediaContainerAwareInterface, StatusAwareInterface
{
    use StatusAwareTrait;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $referenceUrl;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $level;

    /**
     * @var \DateTime
     * @ODM\Field(type="date")
     */
    protected $startDate;

    /**
     * @var \DateTime
     * @ODM\Field(type="date")
     */
    protected $conferredDate;

    /**
     * @var array
     * @ODM\Field(type="collection")
     */
    protected $awardingInstitutions = [];

    /**
     * @var Portal
     * @ODM\ReferenceOne(targetDocument="\Rc\Domain\Model\Portal", storeAs="dbRef")
     */
    protected $portal;

    public function __construct()
    {
        parent::__construct();
        $this->status = self::STATUS_PROGRESS;
    }

    /**
     * @return string
     */
    public function getReferenceUrl()
    {
        return $this->referenceUrl;
    }

    /**
     * @param string $referenceUrl
     * @return Degree
     */
    public function setReferenceUrl($referenceUrl)
    {
        $this->referenceUrl = $referenceUrl;
        return $this;
    }

    /**
     * @return string
     */
    public function getLevel()
    {
        return $this->level;
    }

    /**
     * @param string $level
     * @return Degree
     */
    public function setLevel($level)
    {
        $this->level = $level;
        return $this;
    }

    /**
     * @param string|bool $format
     * @return \DateTime|string
     */
    public function getStartDate($format = true)
    {
        return $this->formatDateTime($this->startDate, $format);
    }

    /**
     * @param string $startDate
     * @return Degree
     */
    public function setStartDate($startDate)
    {
        $this->startDate = $this->castToDateTime($startDate);
        return $this;
    }

    /**
     * @param string|bool $format
     * @return \DateTime|string
     */
    public function getConferredDate($format = true)
    {
        return $this->formatDateTime($this->conferredDate, $format);
    }

    /**
     * @param string $conferredDate
     * @return Degree
     */
    public function setConferredDate($conferredDate)
    {
        $this->conferredDate = $this->castToDateTime($conferredDate);
        return $this;
    }

    /**
     * @return Portal
     */
    public function getPortal()
    {
        return $this->portal;
    }

    /**
     * @param Portal $portal
     * @return Degree
     */
    public function setPortal(Portal $portal = null)
    {
        $this->portal = $portal;
        return $this;
    }

    /**
     * @return int
     */
    public function getPortalId()
    {
        return $this->portal->getId();
    }

    /**
     * @param int $portalId
     * @return $this
     */
    public function setPortalId($portalId)
    {
        $this->portal = $this->createReference(Portal::class, $portalId);
        return $this;
    }

    /**
     * @return array
     */
    public function getAwardingInstitutions()
    {
        return $this->awardingInstitutions;
    }

    /**
     * @param array $values
     */
    public function setAwardingInstitutions(array $values)
    {
        $this->awardingInstitutions = $values;
    }

    /**
     * @return string[]
     */
    public function getTranslatedAwardingInstitutions()
    {
        $countryList = MyLocal::getTranslationList('territory', null, 2);

        $retVal = [];
        foreach ($this->getAwardingInstitutions() as $entry) {
            $retVal[] = $entry['institution'] . ' (' . $countryList[$entry['country']] . ')';
        }
        return $retVal;
    }

}
