<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */
namespace Rc\Domain\Model\Research\Exposition\SimpleMedia;

use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;
use Rc\Domain\Model\Media\MediaType;
use Rc\Domain\Model\Research\Exposition\Generic\ThreeDimensionalAwareInterface;
use Rc\Domain\Model\Research\Exposition\SimpleMedia;
use Rc\Domain\Model\Research\MediaContainer;
use Rc\Domain\Model\Research\MediaContainerInterface;

/**
 * @ODM\Document
 */
class ArchiveSimpleMedia extends SimpleMedia implements ThreeDimensionalAwareInterface
{

    /**
     * @var int
     * @ODM\Field(type="int")
     */
    protected $levels;

    /**
     * @var int
     * @ODM\Field(type="int")
     */
    protected $startLevel;

    /**
     * @var boolean
     * @ODM\Field(type="bool")
     */
    protected $autoRotate;

    /**
     * @return MediaContainerInterface
     */
    public function getMediaContainer()
    {
        // for performance reasons we supply only first media
        $mediaContainer = new MediaContainer($this->mediaList->first() ?: null, $this->thumb);
        $mediaContainer->setId($this->getId());
        $mediaContainer->setName($this->getName());
        $mediaContainer->setContentType($this->getMediaType());
        return $mediaContainer;
    }

    /**
     * @return int
     */
    public function getLevels()
    {
        return $this->levels;
    }

    /**
     * @param int $levels
     */
    public function setLevels($levels)
    {
        $this->levels = $levels;
    }

    /**
     * @return int
     */
    public function getStartLevel()
    {
        return $this->startLevel;
    }

    /**
     * @param int $startLevel
     */
    public function setStartLevel($startLevel)
    {
        $this->startLevel = $startLevel;
    }

    /**
     * @return boolean
     */
    public function getAutoRotate()
    {
        return $this->isAutoRotate();
    }

    /**
     * @return boolean
     */
    public function isAutoRotate()
    {
        return $this->autoRotate;
    }

    /**
     * @param boolean $autoRotate
     */
    public function setAutoRotate($autoRotate)
    {
        $this->autoRotate = $autoRotate;
    }

    /**
     * @return string
     */
    public function getMediaType()
    {
        return MediaType::ARCHIVE;
    }

    /**
     * @inheritdoc
     */
    public function getOptions()
    {
        return [
            'simpleMedia' => $this,
            'research'    => $this->research,
            'levels'      => $this->levels,
            'startLevel'  => $this->startLevel,
            'autoRotate'  => $this->autoRotate,
        ];
    }
}