<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Model\Research\Exposition\TableOfContent;

use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;
use Rc\Domain\Model\Research\Exposition\Weave;
use Rc\Domain\Model\Translation;

/**
 * @ODM\EmbeddedDocument()
 */
class Chapter
{

    /**
     * @var Weave
     * @ODM\ReferenceOne()
     */
    protected $weave;

    /**
     * @var Translation
     * @ODM\EmbedOne()
     */
    protected $title;

    /**
     * @var Anchor
     * @ODM\EmbedOne()
     */
    protected $anchor;

    /**
     * @param Weave $weave
     * @param Translation $title
     * @param Anchor $anchor
     */
    public function __construct(Weave $weave, Translation $title, Anchor $anchor)
    {
        $this->weave = $weave;
        $this->title = $title;
        $this->anchor = $anchor;
    }

    /**
     * @return Weave
     */
    public function getWeave()
    {
        return $this->weave;
    }

    /**
     * @return string
     */
    public function getTitle()
    {
        return $this->title;
    }

    /**
     * @return Anchor
     */
    public function getAnchor()
    {
        return $this->anchor;
    }

    /**
     * @return array
     */
    public function toArray()
    {
        return [
                'weaveid' => $this->weave->getId(),
                'title'   => (string)$this->title,
            ] + $this->anchor->toArray();
    }

}