<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Model\Research\Exposition;

use Doctrine\Common\NotifyPropertyChanged;
use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;
use Rc\Doctrine\Common\NotifyPropertyChangedTrait;
use Rc\Domain\Model\AbstractModel;
use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Model\Research\Exposition\Weave\Tool;
use Rc\Domain\Model\Research\Exposition\Weave\ToolInterface;

/**
 * @ODM\Document
 * @ODM\ChangeTrackingPolicy(value="NOTIFY")
 * @ODM\Index(keys={"exposition.$id"="asc"})
 */
class ToolTemplate extends AbstractModel implements NotifyPropertyChanged
{
    use NotifyPropertyChangedTrait;

    /**
     * @var int
     */
    protected $id = false;

    /**
     * @var Tool
     * @ODM\EmbedOne(targetDocument="Rc\Domain\Model\Research\Exposition\Weave\Tool")
     */
    protected $tool;

    /**
     * @var Exposition
     * @ODM\ReferenceOne(targetDocument="Rc\Domain\Model\Research\Exposition", storeAs="dbRef")
     */
    protected $exposition;

    /**
     * @param ToolInterface $tool
     * @param Exposition $exposition
     */
    public function __construct(ToolInterface $tool, Exposition $exposition)
    {
        parent::__construct();
        $this->tool = $tool;
        $this->exposition = $exposition;
    }

    /**
     * @return Tool
     */
    public function getTool()
    {
        return $this->tool;
    }

    /**
     * @return Exposition
     */
    public function getExposition()
    {
        return $this->exposition;
    }

    /**
     * @param array $values
     */
    public function updateToolValues(array $values)
    {
        $oldTool = clone $this->tool;
        $oldTool->setId($oldTool->getId());
        $oldTool->setUuid($oldTool->getUuid());

        $this->tool->setValues($values);

        $this->onPropertyChange('tool', $oldTool, $this->tool);
    }
}