<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */
namespace Rc\Domain\Model\Research\Exposition\Weave\Generic;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;
use Rc\Domain\Model\Research\Exposition\SimpleMedia\AudioSimpleMedia;
use Rc\Domain\Model\Research\Exposition\SimpleMediaInterface;
use Rc\Domain\Model\Research\Exposition\Weave\Tool;

/**
 * @mixin Tool
 */
trait AudioAwareTrait
{

    /**
     * @var ArrayCollection|AudioSimpleMedia[]
     * @ODM\ReferenceMany(strategy="set", storeAs="dbRef")
     */
    protected $audioList;

    /**
     * @param SimpleMediaInterface $parent
     * @param AudioSimpleMedia|null $audio
     */
    public function setAudio(SimpleMediaInterface $parent, AudioSimpleMedia $audio = null)
    {
        if (empty($audio)) {
            $this->audioList->offsetUnset($parent->getId());
        } else {
            $this->audioList->offsetSet($parent->getId(), $audio);
        }
    }

    /**
     * @param SimpleMediaInterface $parent
     * @return bool
     */
    public function hasAudio(SimpleMediaInterface $parent)
    {
        return $this->audioList->offsetExists($parent->getId());
    }

    /**
     * @param SimpleMediaInterface $parent
     * @return AudioSimpleMedia|null
     */
    public function getAudio(SimpleMediaInterface $parent)
    {
        if (!$this->hasAudio($parent)) {
            return null;
        }
        return $this->audioList->offsetGet($parent->getId());
    }

    /**
     * @param SimpleMediaInterface $parent
     * @return null
     */
    public function getAudioId(SimpleMediaInterface $parent)
    {
        $audio = $this->getAudio($parent);
        if (!isset($audio)) {
            return null;
        }

        return $audio->getId();
    }

    /**
     * @return AudioSimpleMedia[]
     */
    public function getAudios()
    {
        return $this->audioList->toArray();
    }

    /**
     * @param SimpleMediaInterface $oldParent
     * @param SimpleMediaInterface $newParent
     */
    protected function removeExternalDependencyForAudio(SimpleMediaInterface $oldParent, SimpleMediaInterface $newParent)
    {
        if (!$this->hasAudio($oldParent)) {
            return;
        }

        $audio = $this->getAudio($oldParent);
        if ($audio->hasWork()) {
            $audio = clone $this->getAudio($oldParent);
            $audio->setResearch($this->weave->getExposition());
        }

        $this->setAudio($oldParent, null);
        $this->setAudio($newParent, $audio);
    }
}