<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */
namespace Rc\Domain\Model\Research\Exposition\Weave\Generic;

use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;
use Rc\Domain\Model\Media\Dimension;

trait DimensionAwareTrait
{
    /**
     * @var int
     * @ODM\Field(type="int")
     */
    protected $width;

    /**
     * @var int
     * @ODM\Field(type="int")
     */
    protected $height;

    /**
     * @return int
     */
    public function getWidth()
    {
        return $this->width;
    }

    /**
     * @param int $width
     * @return $this
     */
    public function setWidth($width)
    {
        $this->width = $width;
        return $this;
    }

    /**
     * @return int
     */
    public function getHeight()
    {
        return $this->height;
    }

    /**
     * @param int $height
     * @return $this
     */
    public function setHeight($height)
    {
        $this->height = $height;
        return $this;
    }

    /**
     * @return float
     */
    public function getAspectRatio()
    {
        if (!$this->hasAspectRatio()) {
            return null;
        }
        return $this->width / $this->height;
    }

    /**
     * @return bool
     */
    public function hasAspectRatio()
    {
        return !empty($this->height);
    }

    /**
     * @return Dimension
     */
    public function getDimension()
    {
        return new Dimension($this->width, $this->height);
    }

    /**
     * @return string
     */
    public function getDisplayAspectRatio()
    {
        $width = $this->width;
        $height = $this->height;

        if ($width === $height) {
            return '1:1';
        }

        // make sure $this->width is always the larger number
        if ($width < $height) {
            $tmp = $width;
            $width = $height;
            $height = $tmp;
        }

        $divisor = $this->getGreatestCommonDivisor($width, $this->height);
        if (isset($tmp)) {
            return ($height / $divisor) . ':' . ($width / $divisor);
        } else {
            return ($width / $divisor) . ':' . ($height / $divisor);
        }
    }

    /**
     * @param int $a
     * @param int $b
     * @return int
     */
    protected function getGreatestCommonDivisor($a, $b)
    {
        if ($b === 0) {
            return $a;
        }
        return $this->getGreatestCommonDivisor($b, $a % $b);
    }
}