<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Model\Research\Exposition\Weave\Tool;

use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;
use Rc\Domain\Model\Media\MediaType;
use Rc\Domain\Model\Research\Exposition\Weave\Tool;
use Rc\Domain\Model\Research\Exposition\Weave\Tool\Context\Context;
use Rc\Domain\Model\Research\Exposition\Weave\Tool\Context\ContextAwareInterface;
use Rc\Domain\Model\Research\Exposition\Weave\Tool\Generic\SimpleAclAwareInterface;
use Rc\Domain\Model\User;

/**
 * @ODM\Document
 */
class CommentTool extends Tool implements SimpleAclAwareInterface, ContextAwareInterface
{
    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $resolution;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $deadline;

    /**
     * @var Context
     * @ODM\EmbedOne()
     */
    protected $context;

    /**
     * @var User
     * @ODM\ReferenceOne(targetDocument="Rc\Domain\Model\User", storeAs="dbRef")
     */
    protected $assignee;

    /**
     * @return Context
     */
    public function getContext()
    {
        return $this->context;
    }

    /**
     * @param Context $context
     */
    public function setContext(Context $context)
    {
        $this->context = $context;
    }

    /**
     * @return bool
     */
    public function hasContext()
    {
        return isset($this->context);
    }

    /**
     * @return array
     */
    public function getSupportedMediaTypes()
    {
        return [
            MediaType::PDF,
        ];
    }

    /**
     * @inheritdoc
     */
    public function getZIndex()
    {
        return $this->zIndex + 1e3;
    }

    /**
     * @param User|null $user
     * @return bool
     */
    public function isVisibleTo(User $user = null)
    {
        if (!isset($this->context)) {
            return parent::isVisibleTo($user);
        }

        return $this->context->isVisibleTo($this, $user);
    }

    /**
     * @param User|null $user
     * @return bool
     */
    public function isEditableBy(User $user = null)
    {
        if (!isset($this->context)) {
            return parent::isEditableBy($user);
        }

        return $this->context->isEditableBy($this, $user);
    }

    /**
     * @param User|null $user
     * @return bool
     */
    public function isMetaEditableBy(User $user = null)
    {
        if (!isset($this->context)) {
            return parent::isMetaEditableBy($user);
        }

        return $this->context->isMetaEditableBy($this, $user);
    }

    /**
     * @return string
     */
    public function getResolution()
    {
        return $this->resolution;
    }

    /**
     * @param string $resolution
     */
    public function setResolution($resolution)
    {
        $this->resolution = $resolution;
    }

    /**
     * @return string
     */
    public function getDeadline()
    {
        return $this->deadline;
    }

    /**
     * @param string $deadline
     */
    public function setDeadline($deadline)
    {
        $this->deadline = $deadline;
    }

    /**
     * @return User
     */
    public function getAssignee()
    {
        return $this->assignee;
    }

    /**
     * @param User $assignee
     */
    public function setAssignee(User $assignee = null)
    {
        $this->assignee = $assignee;
    }

}