<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */
namespace Rc\Domain\Model\Research\Exposition\Weave\Tool;

use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;
use Rc\Domain\Model\Media\MediaType;
use Rc\Domain\Model\Research\Exposition\Weave\Tool;
use Rc\Domain\Model\Research\Exposition\Weave\Tool\Generic\AspectRatioAwareInterface;
use Rc\Domain\Model\Research\Exposition\Weave\Tool\Generic\AspectRatioAwareTrait;

/**
 * @ODM\Document
 */
class PictureTool extends Tool implements AspectRatioAwareInterface
{
    use AspectRatioAwareTrait;

    const HIGH_QUALITY_WIDTH = 1024;
    const HIGH_QUALITY_HEIGHT = 1024;

    /**
     * @var boolean
     * @ODM\Field(type="boolean")
     */
    protected $openPictureInPopup;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $openLink;

    /**
     * @return bool
     */
    public function isOpenPictureInPopup()
    {
        return $this->openPictureInPopup;
    }

    /**
     * @param boolean $openPictureInPopup
     * @return $this
     */
    public function setOpenPictureInPopup($openPictureInPopup)
    {
        $this->openPictureInPopup = $openPictureInPopup;
        return $this;
    }

    /**
     * @todo: check why we replace x/y for the current weave in original method
     * @return string
     */
    public function getOpenLink()
    {
        return $this->openLink;
    }

    /**
     * @todo: check why we replace x/y for the current weave in original method
     * @param string $link
     * @return $this
     */
    public function setOpenLink($link)
    {
        $this->openLink = $link;
        return $this;
    }

    /**
     * @return array
     */
    public function getSupportedMediaTypes()
    {
        return [
            MediaType::IMAGE,
        ];
    }
}

?>