<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Model\Research\Exposition\Weave\Tool;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;
use Rc\Domain\Model\Media\MediaType;
use Rc\Domain\Model\Research\Exposition\SimpleMediaInterface;
use Rc\Domain\Model\Research\Exposition\Weave\Tool;
use Rc\Domain\Model\Research\Exposition\Weave\Tool\Generic\PlayerAwareInterface;

/**
 * @ODM\Document
 */
class PlayTool extends Tool
{
    const MEDIA_LIMIT = 2;

    /**
     * @var Collection|PlayerAwareInterface[]
     * @ODM\ReferenceMany(targetDocument="Rc\Domain\Model\Research\Exposition\Weave\Tool", storeAs="dbRef")
     */
    protected $players;

    /**
     * @var boolean
     * @ODM\Field(type="boolean")
     */
    protected $pauseOtherPlayers;

    /**
     * @return array
     */
    public function getSupportedMediaTypes()
    {
        return [
            MediaType::IMAGE,
            MediaType::HTML,
        ];
    }

    /**
     * @return integer[]
     */
    public function getPlayers()
    {
        return $this->createIdList($this->players);
    }

    /**
     * @param integer[] $players
     */
    public function setPlayers(array $players)
    {
        $this->players = new ArrayCollection($this->createReferenceList(Tool::class, $players));
    }

    /**
     * @return boolean
     */
    public function getPauseOtherPlayers()
    {
        return $this->pauseOtherPlayers;
    }

    /**
     * @param boolean $pauseOtherPlayers
     */
    public function setPauseOtherPlayers($pauseOtherPlayers)
    {
        $this->pauseOtherPlayers = $pauseOtherPlayers;
    }

    /**
     * @return SimpleMediaInterface|null
     */
    public function getMediaForPauseSlide()
    {
        if ($this->mediaList->containsKey(1)) {
            return $this->mediaList->offsetGet(1);
        } else {
            return null;
        }
    }

    /**
     * @return SimpleMediaInterface|null
     */
    public function getMediaForPlaySlide()
    {
        if ($this->mediaList->containsKey(0)) {
            return $this->mediaList->offsetGet(0);
        } else {
            return null;
        }
    }
}