<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Model\Research\Exposition\Weave\Tool;

use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;
use Rc\Domain\Model\Media\MediaType;
use Rc\Domain\Model\Research\Exposition\Weave\Tool;

/**
 * @ODM\Document
 */
class ShapeTool extends Tool
{
    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $fillColor;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $strokeColor;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $strokeWidth;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $strokeDashArray;

    public function __construct()
    {
        parent::__construct();

        if ($this->isNew()) {
            $this->setShapeContent('circle');
            $this->setFillColor('black');
            $this->setStrokeColor('black');
            $this->setStrokeWidth(3);
            $this->setStrokeDashArray('');
        }
    }

    /**
     * @return array
     */
    public static function getStrokeDashArrays()
    {
        return [
            ''    => 'css.border.solid',
            '3,3' => 'css.border.dotted',
            '7,3' => 'css.border.dashed',
        ];
    }

    /**
     * @return array
     */
    public function getSupportedMediaTypes()
    {
        return [
            MediaType::SHAPE,
        ];
    }

    /**
     * @param null $shapeId
     * @return array
     */
    public function getValidOptions($shapeId = null)
    {
        if ($shapeId === null) {
            $shapeId = $this->getShapeContent();
        }

        switch ($shapeId) {
            case 'line':
            case 'verticalLine':
            case 'arrowLeft':
            case 'arrowUp':
            case 'arrowRight':
            case 'arrowDown':
                return [
                    'stroke',
                ];
                break;

            default:
                return [
                    'fill',
                    'stroke',
                ];
        }
    }

    /**
     * @return array
     */
    public function getStroke()
    {
        return [
            'color'     => $this->getStrokeColor(),
            'width'     => $this->getStrokeWidth(),
            'dashArray' => $this->getStrokeDashArray(),
        ];
    }

    /**
     * @return string
     */
    public function getStrokeDashArray()
    {
        return $this->strokeDashArray;
    }

    /**
     * @param string $strokeDashArray
     */
    public function setStrokeDashArray($strokeDashArray)
    {
        $this->strokeDashArray = $strokeDashArray;
    }

    /**
     * @return string
     */
    public function getStrokeWidth()
    {
        return $this->strokeWidth;
    }

    /**
     * @param string $strokeWidth
     */
    public function setStrokeWidth($strokeWidth)
    {
        $this->strokeWidth = $strokeWidth;
    }

    /**
     * @return string
     */
    public function getStrokeColor()
    {
        return $this->strokeColor;
    }

    /**
     * @param string $strokeColor
     */
    public function setStrokeColor($strokeColor)
    {
        $this->strokeColor = $strokeColor;
    }

    /**
     * @return string
     */
    public function getFillColor()
    {
        return $this->fillColor;
    }

    /**
     * @param string $fillColor
     */
    public function setFillColor($fillColor)
    {
        $this->fillColor = $fillColor;
    }
}