<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */
namespace Rc\Domain\Model\Research\Exposition\Weave\Tool;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;
use Rc\Domain\Model\Media\MediaType;
use Rc\Domain\Model\Research\Exposition\Weave\Tool;

/**
 * @ODM\Document
 */
class SlideshowTool extends Tool
{
    const HIGH_QUALITY_WIDTH = 1024;
    const HIGH_QUALITY_HEIGHT = 1024;
    const MEDIA_LIMIT = 1e3;
    const PLAY_OPTION_NONE = '';
    const PLAY_OPTION_AUTO = 'auto';
    const PLAY_OPTION_ONCLICK = 'onclick';

    /**
     * @var int
     * @ODM\Field(type="int")
     */
    protected $crossfade = 0;

    /**
     * @var int
     * @ODM\Field(type="int")
     */
    protected $timeoutDuration = 750;

    /**
     * @var int
     * @ODM\Field(type="int")
     */
    protected $fadeDuration = 350;

    /**
     * @var int
     * @ODM\Field(type="int")
     */
    protected $defaultSlideSpeed;

    /**
     * @var array
     * @ODM\Field(type="hash")
     */
    protected $slidesSpeed = [];

    /**
     * @var boolean
     * @ODM\Field(type="boolean")
     */
    protected $loop;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $playOption;

    /**
     * @var boolean
     * @ODM\Field(type="boolean")
     */
    protected $showNavbar = true;

    /**
     * @var boolean
     * @ODM\Field(type="boolean")
     */
    protected $openPictureInPopup;

    /**
     * @return array
     */
    public static function getPlayOptions()
    {
        return [
            self::PLAY_OPTION_NONE    => 'play.option.none',
            self::PLAY_OPTION_AUTO    => 'play.option.auto',
            self::PLAY_OPTION_ONCLICK => 'play.option.onclick',
        ];
    }

    /**
     * @return array
     */
    public function getSupportedMediaTypes()
    {
        return [
            MediaType::IMAGE,
        ];
    }

    /**
     * @return boolean
     */
    public function isOpenPictureInPopup()
    {
        return $this->openPictureInPopup;
    }

    /**
     * @param boolean $openPictureInPopup
     */
    public function setOpenPictureInPopup($openPictureInPopup)
    {
        $this->openPictureInPopup = $openPictureInPopup;
    }

    /**
     * @return boolean
     */
    public function isShowNavbar()
    {
        return $this->showNavbar;
    }

    /**
     * @param boolean $showNavbar
     */
    public function setShowNavbar($showNavbar)
    {
        $this->showNavbar = $showNavbar;
    }

    /**
     * @return string
     */
    public function getPlayOption()
    {
        return $this->playOption;
    }

    /**
     * @param string $playOption
     */
    public function setPlayOption($playOption)
    {
        $this->playOption = $playOption;
    }

    /**
     * @return boolean
     */
    public function isLoop()
    {
        return $this->loop;
    }

    /**
     * @param boolean $loop
     */
    public function setLoop($loop)
    {
        $this->loop = $loop;
    }

    /**
     * @param bool $asString
     * @return array
     */
    public function getSlidesSpeed($asString = true)
    {
        if ($asString) {
            return json_encode($this->slidesSpeed);
        }
        return $this->slidesSpeed ?: [];
    }

    /**
     * @param array $slidesSpeed
     */
    public function setSlidesSpeed($slidesSpeed)
    {
        if (is_string($slidesSpeed)) {
            $slidesSpeed = json_decode($slidesSpeed, true);
        }
        $this->slidesSpeed = $slidesSpeed;
    }

    /**
     * @return int
     */
    public function getDefaultSlideSpeed()
    {
        return $this->defaultSlideSpeed;
    }

    /**
     * @param int $defaultSlideSpeed
     */
    public function setDefaultSlideSpeed($defaultSlideSpeed)
    {
        $this->defaultSlideSpeed = $defaultSlideSpeed;
    }

    /**
     * @return int
     */
    public function getFadeDuration()
    {
        return $this->fadeDuration;
    }

    /**
     * @param int $fadeDuration
     */
    public function setFadeDuration($fadeDuration)
    {
        $this->fadeDuration = $fadeDuration;
    }

    /**
     * @return int
     */
    public function getTimeoutDuration()
    {
        return $this->timeoutDuration;
    }

    /**
     * @param int $timeoutDuration
     */
    public function setTimeoutDuration($timeoutDuration)
    {
        $this->timeoutDuration = $timeoutDuration;
    }

    /**
     * @return int
     */
    public function getCrossfade()
    {
        return $this->crossfade;
    }

    /**
     * @param int $crossfade
     */
    public function setCrossfade($crossfade)
    {
        $this->crossfade = $crossfade;
    }

    /**
     * @param int $mediaId
     * @param int $position
     * @return $this
     */
    public function setResourcePosition($mediaId, $position)
    {
        $simpleMediaList = $this->mediaList->toArray();
        $indexToMove = array_search($mediaId, $simpleMediaList);
        if ($indexToMove !== false) {
            unset($simpleMediaList[$indexToMove]);
            $simpleMediaList = array_merge(
                array_slice($simpleMediaList, 0, $position),
                [$mediaId],
                array_slice($simpleMediaList, $position)
            );
            $this->mediaList = new ArrayCollection($simpleMediaList);
        }
        return $this;
    }

}