<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Model\Research\Exposition\Weave;

use Rc\Domain\Model\Core\Acl\AclDelegatorAwareInterface;
use Rc\Domain\Model\Core\Revision\RevisionAwareInterface;
use Rc\Domain\Model\ModelInterface;
use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Model\Research\Exposition\SimpleMediaInterface;
use Rc\Domain\Model\Research\Exposition\Weave\Generic\AudioAwareInterface;
use Rc\Domain\Model\Research\Exposition\Weave\Generic\BackgroundAwareInterface;
use Rc\Domain\Model\Research\Exposition\Weave\Generic\ImageAwareInterface;
use Rc\Domain\Model\Research\Exposition\Weave\Generic\StyleAwareInterface;
use Rc\Domain\Model\Research\Exposition\Weave\Tool\Generic\SimpleAclAwareInterface;
use Rc\Domain\Model\Research\MediaContainerAwareInterface;
use Rc\Domain\Model\Research\MediaContainerInterface;
use Rc\Domain\Model\Research\SimpleMediaAwareInterface;
use Rc\Domain\Model\Translation;
use Rc\Domain\Model\User;

interface ToolInterface extends
    ModelInterface,
    AudioAwareInterface,
    BackgroundAwareInterface,
    MediaContainerAwareInterface,
    StyleAwareInterface,
    RevisionAwareInterface,
    AclDelegatorAwareInterface,
    SimpleAclAwareInterface,
    SimpleMediaAwareInterface,
    ImageAwareInterface
{

    const SIZE_COVER = 'cover';
    const SIZE_CONTAIN = 'contain';
    const SIZE_FIT_WIDTH = 'fit-width';
    const SIZE_FIT_HEIGHT = 'fit-height';
    const SIZE_FIT = 'fit';
    const SIZE_AUTO = 'auto';
    const SIZE_HORIZONTAL = 0;
    const SIZE_VERTICAL = 1;
    const POSITION_CENTER_CENTER = 'center center';
    const MEDIA_LIMIT = 1;

    /**
     * @return string
     */
    public function getName();

    /**
     * @return User
     */
    public function getOwner();

    /**
     * @return Translation
     */
    public function getTitle();

    /**
     * @param Translation $title
     * @return $this
     */
    public function setTitle(Translation $title);

    /**
     * @param User $owner
     * @return $this
     */
    public function setOwner(User $owner);

    /**
     * @param User $owner
     * @return bool
     */
    public function isOwner(User $owner);

    /**
     * @return Translation
     */
    public function getDescription();

    /**
     * @param Translation $description
     * @return $this
     */
    public function setDescription(Translation $description);

    /**
     * @return string|\DateTime
     */
    public function getDate();

    /**
     * @param string $date
     * @return void
     */
    public function setDate($date);

    /**
     * @return string
     */
    public function getTextContent();

    /**
     * @param string $text
     */
    public function setTextContent($text);

    /**
     * @return bool
     */
    public function hasTextContent();

    /**
     * @return User
     */
    public function getTextAuthor();

    /**
     * @param User $user
     */
    public function setTextAuthor(User $user);

    /**
     * @param User $user
     * @return bool
     */
    public function isTextAuthor(User $user);

    /**
     * @return string
     */
    public function getTextModified();

    /**
     * @param string $date
     */
    public function setTextModified($date);

    /**
     * @return GraphicalWeave
     */
    public function getWeave();

    /**
     * @param GraphicalWeave $weave
     * @return $this
     */
    public function setWeave(GraphicalWeave $weave);

    /**
     * @return bool
     */
    public function hasWeave();

    /**
     * @return int
     */
    public function getLeft();

    /**
     * @param int $left
     * @return $this
     */
    public function setLeft($left);

    /**
     * @return int
     */
    public function getTop();

    /**
     * @param int $top
     * @return $this
     */
    public function setTop($top);

    /**
     * @return int
     */
    public function getOuterWidth();

    /**
     * @return int
     */
    public function getOuterHeight();

    /**
     * @return null|string
     */
    public function getMediaType();

    /**
     * @param string $mediaType
     */
    public function setMediaType($mediaType);

    /**
     * manual modified flag, e.g. for research ordering
     *
     * @return int
     */
    public function getLastModified();

    /**
     * manual modified flag, e.g. for research ordering
     *
     * @param integer|null $timestamp
     */
    public function setLastModified($timestamp = null);

    /**
     * @param int $sourceId
     */
    public function setSourceId($sourceId);

    /**
     * @return int|null
     */
    public function getBackgroundImageFileId();

    /**
     * @param int $fileId
     */
    public function setBackgroundImageFileId($fileId);

    /**
     * @return int
     */
    public function getOpacity();

    /**
     * @param int $opacity
     * @return $this
     */
    public function setOpacity($opacity);

    /**
     * @param int $zIndex
     * @return $this
     */
    public function setZIndex($zIndex);

    /**
     * @return string
     */
    public function getPosition();

    /**
     * @param string $position
     * @return $this
     */
    public function setPosition($position);

    /**
     * @param bool $reviewing
     * @return $this
     */
    public function setReviewing($reviewing);

    /**
     * @return boolean
     */
    public function hasMedia();

    /**
     * @return SimpleMediaInterface|null
     */
    public function getMedia();

    /**
     * @return SimpleMediaInterface[]
     */
    public function getMediaList();

    /**
     * @param SimpleMediaInterface $simpleMedia
     * @return MediaContainerInterface
     */
    public function getSimpleMediaContainer(SimpleMediaInterface $simpleMedia);

    /**
     * @param SimpleMediaInterface|null $media
     */
    public function setMedia(SimpleMediaInterface $media = null);

    /**
     * @return Exposition
     */
    public function getExposition();

    /**
     * @return void
     */
    public function removeExternalDependencies();

    /**
     * @return int
     */
    public function getResourceLimit();

    /**
     * @return void
     */
    public function removeMissingResources();
}