<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */
namespace Rc\Domain\Model\Research;

use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;
use Rc\Domain\Model\Research;
use Rc\Domain\Model\User;

/**
 * @ODM\Document
 */
class Project extends Research implements MediaContainerAwareInterface
{

    const VAL_DESCRIPTION_LENGTH_MIN = 10;
    const VAL_DESCRIPTION_LENGTH_MAX = 600;

    const VAL_GENRE_LENGTH_MIN = 0;
    const VAL_GENRE_LENGTH_MAX = 50;
    const VAL_AREA_LENGTH_MIN = 0;
    const VAL_AREA_LENGTH_MAX = 50;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $mainFieldOfStudy;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $mainFieldOfStudyType;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $area;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $ects;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $genre;

    /**
     * @var \DateTime
     * @ODM\Field(type="date")
     */
    protected $startDate;

    /**
     * @var \DateTime
     * @ODM\Field(type="date")
     */
    protected $endDate;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $referenceUrl;

    /**
     * @var User
     * @ODM\ReferenceOne(targetDocument="\Rc\Domain\Model\User", storeAs="dbRef")
     */
    protected $principalInvestigator;

    /**
     * @return string
     */
    public function getMainFieldOfStudy()
    {
        return $this->mainFieldOfStudy;
    }

    /**
     * @param string $mainFieldOfStudy
     * @return Project
     */
    public function setMainFieldOfStudy($mainFieldOfStudy)
    {
        $this->mainFieldOfStudy = $mainFieldOfStudy;
        return $this;
    }

    /**
     * @return string
     */
    public function getMainFieldOfStudyType()
    {
        return $this->mainFieldOfStudyType;
    }

    /**
     * @param string $mainFieldOfStudyType
     * @return Project
     */
    public function setMainFieldOfStudyType($mainFieldOfStudyType)
    {
        $this->mainFieldOfStudyType = $mainFieldOfStudyType;
        return $this;
    }

    /**
     * @return string
     */
    public function getArea()
    {
        return $this->area;
    }

    /**
     * @param string $area
     * @return Project
     */
    public function setArea($area)
    {
        $this->area = $area;
        return $this;
    }

    /**
     * @return string
     */
    public function getEcts()
    {
        return $this->ects;
    }

    /**
     * @param string $ects
     * @return Project
     */
    public function setEcts($ects)
    {
        $this->ects = $ects;
        return $this;
    }

    /**
     * @return string
     */
    public function getGenre()
    {
        return $this->genre;
    }

    /**
     * @param string $genre
     * @return Project
     */
    public function setGenre($genre)
    {
        $this->genre = $genre;
        return $this;
    }

    /**
     * @param string|bool $format
     * @return \DateTime|string
     */
    public function getStartDate($format = true)
    {
        return $this->formatDateTime($this->startDate, $format);
    }

    /**
     * @param string $startDate
     * @return Project
     */
    public function setStartDate($startDate)
    {
        $this->startDate = $this->castToDateTime($startDate);
        return $this;
    }

    /**
     * @param string|bool $format
     * @return \DateTime|string
     */
    public function getEndDate($format = true)
    {
        return $this->formatDateTime($this->endDate, $format);
    }

    /**
     * @param string $endDate
     * @return Project
     */
    public function setEndDate($endDate)
    {
        $this->endDate = $this->castToDateTime($endDate);
        return $this;
    }

    /**
     * @return string
     */
    public function getReferenceUrl()
    {
        return $this->referenceUrl;
    }

    /**
     * @param string $referenceUrl
     * @return Project
     */
    public function setReferenceUrl($referenceUrl)
    {
        $this->referenceUrl = $referenceUrl;
        return $this;
    }

    /**
     * @return User
     */
    public function getPrincipalInvestigator()
    {
        return $this->principalInvestigator ?: $this->getOwner();
    }

    /**
     * @param User $user
     * @return $this
     */
    public function setPrincipalInvestigator(User $user)
    {
        $this->principalInvestigator = $user;
        return $this;
    }

    /**
     * @return int[]
     */
    public function getPrincipalInvestigators()
    {
        return [$this->getPrincipalInvestigator()->getId()];
    }
}
