<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Model\Research;

use Rc\Domain\Model\Core\Acl\AclAwareTrait;

/**
 * @mixin AclAwareTrait
 */
trait StatusAwareTrait
{

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $status = self::STATUS_PROGRESS;

    /**
     * @param int $status
     * @return bool
     */
    public function hasStatus($status)
    {
        return $this->getStatus() == $status;
    }

    /**
     * @return string
     */
    public function getStatus()
    {
        return $this->status;
    }

    /**
     * @param string $status
     */
    public function setStatus($status)
    {
        $this->status = $status;
    }

    /**
     * @internal
     * @deprecated
     * @param string $value
     */
    public function setStatusValue($value)
    {
        $this->status = $value;
    }

    /**
     * @param string $status
     * @return bool
     */
    public function isStatus($status)
    {
        return $this->getStatus() == $status;
    }

    /**
     * @return bool
     */
    public function isPublished()
    {
        return $this->isStatus(StatusAwareInterface::STATUS_PUBLISHED) || $this->isStatus(StatusAwareInterface::STATUS_REVIEW_PUBLISHED);
    }

    /**
     * @return bool
     */
    public function isInReview()
    {
        return $this->isStatus(StatusAwareInterface::STATUS_REVIEW) || $this->isStatus(StatusAwareInterface::STATUS_PRE_REVIEW);
    }

    /**
     * @return bool
     */
    public function isInProgress()
    {
        return $this->isStatus(StatusAwareInterface::STATUS_PROGRESS) || $this->isStatus(StatusAwareInterface::STATUS_REWORK);
    }

    /**
     * Should be overwritten in classes implementing StatusAwareTrait
     */
    public function removeExternalDependencies()
    {
    }
}
