<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */
namespace Rc\Domain\Model\Research;

use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;
use JMS\Serializer\Annotation as ODS;
use Rc\Domain\Model\Research;
use Rc\Domain\Model\User;
use Rc\Domain\Repository\UserRepository;

/**
 * @ODM\Document
 */
class Work extends Research implements MediaContainerAwareInterface, ObjectShareAwareInterface
{
    use ObjectShareAwareTrait;

    /**
     * @var array
     * @ODS\Exclude()
     */
    protected static $genreMapping = [
        'publication'    => 'publication',
        'paper'          => 'publication',
        'catalogue'      => 'publication',
        'article'        => 'publication',
        'book'           => 'publication',
        'broadcast'      => 'publication',
        'cd'             => 'publication',
        'dvd'            => 'publication',
        'event'          => 'event',
        'exhibition'     => 'event',
        'screening'      => 'event',
        'concert'        => 'event',
        'performance'    => 'event',
        'festival'       => 'event',
        'seminar'        => 'event',
        'conference'     => 'event',
        'presentation'   => 'event',
        'workshop'       => 'event',
        'art'            => 'art',
        'installation'   => 'art',
        'scenery'        => 'art',
        'piece'          => 'art',
        'design'         => 'art',
        'screenplay'     => 'art',
        'sound'          => 'art',
        'photograph'     => 'art',
        'painting'       => 'art',
        'scalemodel'     => 'art',
        'digitalartwork' => 'art',
        'visualisation'  => 'art',
        'illustration'   => 'art',
        'ceramic'        => 'art',
        'print'          => 'art',
        'construction'   => 'art',
        'drawing'        => 'art',
        'video'          => 'art',
        'composition'    => 'art',
        'movie'          => 'art',
    ];

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $copyright;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $genre;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $referenceUrl;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $place;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $country;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $material;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $technique;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $format;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $publisher;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $reference;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $freeTextAuthor;

    /**
     * @return string
     */
    public function getCopyrightHolder()
    {
        return $this->copyright;
    }

    /**
     * @param string $copyright
     * @return $this
     */
    public function setCopyrightHolder($copyright)
    {
        $this->copyright = $copyright;
        return $this;
    }

    /**
     * @return string|null
     */
    public function getMainGenre()
    {
        if (!isset(self::$genreMapping[$this->getGenre()])) {
            return null;
        }
        return self::$genreMapping[$this->getGenre()];
    }

    /**
     * @return string
     */
    public function getGenre()
    {
        return $this->genre;
    }

    /**
     * @param string $genre
     * @return $this
     */
    public function setGenre($genre)
    {
        $this->genre = $genre;
        return $this;
    }

    /**
     * @return string
     */
    public function getReferenceUrl()
    {
        return $this->referenceUrl;
    }

    /**
     * @param string $referenceUrl
     * @return $this
     */
    public function setReferenceUrl($referenceUrl)
    {
        $this->referenceUrl = $referenceUrl;
        return $this;
    }

    /**
     * @return string
     */
    public function getPlace()
    {
        return $this->place;
    }

    /**
     * @param string $place
     * @return $this
     */
    public function setPlace($place)
    {
        $this->place = $place;
        return $this;
    }

    /**
     * @return string
     */
    public function getCountry()
    {
        return $this->country;
    }

    /**
     * @param string $country
     * @return $this
     */
    public function setCountry($country)
    {
        $this->country = $country;
        return $this;
    }

    /**
     * @return string
     */
    public function getMaterial()
    {
        return $this->material;
    }

    /**
     * @param string $material
     * @return $this
     */
    public function setMaterial($material)
    {
        $this->material = $material;
        return $this;
    }

    /**
     * @return string
     */
    public function getTechnique()
    {
        return $this->technique;
    }

    /**
     * @param string $technique
     * @return $this
     */
    public function setTechnique($technique)
    {
        $this->technique = $technique;
        return $this;
    }

    /**
     * @return string
     */
    public function getFormat()
    {
        return $this->format;
    }

    /**
     * @param string $format
     * @return $this
     */
    public function setFormat($format)
    {
        $this->format = $format;
        return $this;
    }

    /**
     * @return string
     */
    public function getPublisher()
    {
        return $this->publisher;
    }

    /**
     * @param string $publisher
     * @return $this
     */
    public function setPublisher($publisher)
    {
        $this->publisher = $publisher;
        return $this;
    }

    /**
     * @return string
     */
    public function getReference()
    {
        return $this->reference;
    }

    /**
     * @param string $reference
     * @return $this
     */
    public function setReference($reference)
    {
        $this->reference = $reference;
        return $this;
    }

    /**
     * @return User|null
     */
    public function getAuthor()
    {
        return $this->author ?: $this->owner;
    }

    /**
     * @return string
     */
    public function getFreeTextAuthor()
    {
        return $this->freeTextAuthor;
    }

    /**
     * @param string $author
     * @return $this
     */
    public function setFreeTextAuthor($author)
    {
        $this->freeTextAuthor = $author;
        return $this;
    }

    /**
     * @return bool
     */
    public function hasFreeTextAuthor()
    {
        return isset($this->freeTextAuthor);
    }

    /**
     * @param array $values
     * @return $this
     */
    public function setValues(array $values)
    {
        parent::setValues($values);

        if (!$this->hasFreeTextAuthor() && !empty($values['rcauthors'])) {
            $this->setAuthor(UserRepository::findById(reset($values['rcauthors'])));
        } else {
            $this->setAuthor(null);
        }
        return $this;
    }
}
