<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Model;

use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;

/**
 * @ODM\EmbeddedDocument
 */
class Translation
{
    const DEFAULT_LANGUAGE = 'en';

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $en;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $nl;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $et;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $fi;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $fr;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $de;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $no;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $sv;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $es;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $pt;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $it;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $da;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $lt;

    /**
     * @param array|string $list
     */
    public function __construct($list)
    {
        if (is_string($list)) {
            $list = [self::DEFAULT_LANGUAGE => $list];
        }

        foreach ($list as $key => $value) {
            $this->{$key} = $value;
        }
    }

    /**
     * @param string $language
     * @param string $value
     * @return Translation
     */
    public function set($language, $value)
    {
        $list = $this->toArray();
        $list[$language] = $value;

        return new self($list);
    }

    /**
     * @return array
     */
    public function toArray()
    {
        return array_filter(get_object_vars($this), function ($value) {
            return isset($value) && $value !== '';
        });
    }

    /**
     * @return string
     */
    public function __toString()
    {
        if (isset($this->{self::DEFAULT_LANGUAGE})) {
            return $this->{self::DEFAULT_LANGUAGE};
        }

        $list = $this->toArray();
        return reset($list) ?: '';
    }
}