<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Repository\Application;

use DateTime;
use Rc\Domain\Model\Application\ApplicationProgram;
use Rc\Domain\Model\Portal;
use Rc\Domain\Model\User;
use Rc\Domain\Repository\MongoRepository;

/**
 * @method static ApplicationProgram findById($id)
 */
class ApplicationProgramRepository extends MongoRepository
{

    /**
     * @param Portal $portal
     * @param array $options
     * @return ApplicationProgram[]
     */
    public static function findByPortal(Portal $portal, array $options = [])
    {
        return self::find([
            'portal.uuid' => (string)$portal->getUuid(),
        ], $options);
    }

    /**
     * @param User $user
     * @param array $options
     * @return ApplicationProgram[]
     */
    public static function findEligibleByUser(User $user, array $options = [])
    {
        return self::find([
            '$and' => [
                [
                    '$or' => [
                        [
                            'accessGroup' => ApplicationProgram::ACCESS_GROUP_RC_USER,
                        ],
                        [
                            'accessGroup' => ApplicationProgram::ACCESS_GROUP_PORTAL_USER,
                            'portal.uuid' => ['$in' => self::castListToUuidList($user->getPortals())],
                        ],
                        [
                            'accessGroup'          => ApplicationProgram::ACCESS_GROUP_APPLICANT_USER,
                            'applicantPortals.uuid' => ['$in' => self::castListToUuidList($user->getPortals())],
                        ],
                    ],
                ],
                [
                    '$or' => [
                        [
                            'status' => ApplicationProgram::STATUS_OPEN,
                            '$or'    => [
                                ['startDate' => ['$exists' => false]],
                                ['startDate' => ['$lte' => new DateTime()]],
                            ],
                            '$and'   => [
                                [
                                    '$or' => [
                                        ['endDate' => ['$gte' => new DateTime()]],
                                        ['endDate' => ['$exists' => false]],
                                    ],
                                ],
                            ],
                        ],
                        [
                            'status'      => ApplicationProgram::STATUS_TESTING,
                            'admins.uuid' => ['$in' => [(string)$user->getUuid()]],
                        ],
                    ],
                ],
            ],
        ], $options);
    }

    /**
     * @param string $status
     * @param array $options
     * @return ApplicationProgram[]
     */
    public static function findByStatus($status, array $options = [])
    {
        return self::find([
            'status' => (string)$status,
        ], $options);
    }

    /**
     * @param string $field
     * @return bool
     */
    protected static function isMultiLanguageField($field)
    {
        return false;
    }
}