<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Repository\Core;

use Rc\Domain\Model\Core\Revision;
use Rc\Domain\Model\ModelInterface;
use Rc\Domain\Model\Research\Exposition\Weave;
use Rc\Domain\Repository\MongoRepository;

/**
 * @method static Revision|null findById($id))
 * @method static Revision getById($id)
 * @method static Revision|null findOne(array $query = [], array $options = [])
 * @method static Revision[] findByIds(array $id, array $options = [])
 * @method static Revision[] findAll(array $options = [])
 */
class RevisionRepository extends MongoRepository
{

    /**
     * @param Weave $weave
     * @param array $options
     * @return Revision[]
     */
    public static function findByWeave(Weave $weave, $options = [])
    {
        return self::findByWeaveId($weave->getUuid(), $options);
    }

    /**
     * @param string $uuid
     * @param array $options
     * @return Revision[]
     */
    public static function findByWeaveId($uuid, $options = [])
    {
        $options['sort'] = ['created' => self::SORT_DESC];

        return self::find([
            'document.weave.$id' => new \MongoId($uuid),
        ], $options);
    }

    /**
     * @param ModelInterface $document
     * @param array $options
     * @return Revision[]
     */
    public static function findByDocument(ModelInterface $document, $options = [])
    {
        return self::findByDocumentId($document->getUuid(), $options);
    }

    /**
     * @param string $uuid
     * @param array $options
     * @return Revision[]
     */
    public static function findByDocumentId($uuid, $options = [])
    {
        $options['sort'] = ['created' => self::SORT_DESC];

        return self::find([
            'document._id' => new \MongoId($uuid),
        ], $options);
    }

    /**
     * @param Weave $weave
     * @param array $options
     * @return Revision[]
     */
    public static function findDeletedToolsByWeave(Weave $weave, $options = [])
    {
        $options['sort'] = ['created' => self::SORT_DESC];

        return self::find([
            'label'              => 'delete',
            'document.weave.$id' => new \MongoId((string)$weave->getUuid()),
        ], $options);
    }

    /**
     * @param Revision $revision
     */
    public static function remove($revision)
    {
        $revision->prepareForDeletion();
        parent::remove($revision);
    }
}