<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Repository;

use Rc\Domain\Model\ObjectShare;
use Rc\Domain\Model\Research;

/**
 * @method static ObjectShare|null findById($id)
 * @method static ObjectShare getById($id)
 * @method static ObjectShare[] findByIds(array $id, array $options = [])
 * @method static ObjectShare[] findAll(array $options = [])
 */
class ObjectShareRepository extends MongoRepository
{

    /**
     * @param Research $research
     * @param array $options
     * @return ObjectShare|null
     */
    public static function findByEntity(Research $research, array $options = []): ?ObjectShare
    {
        return self::findOne([
            'entityId' => $research->getId(),
        ], $options);
    }

    /**
     * @param string $secret
     * @param array $options
     * @return null|ObjectShare
     */
    public static function findBySecret(string $secret, array $options = []): ?ObjectShare
    {
        return self::findOne([
            'secret' => $secret,
        ], $options);
    }

}