<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */
namespace Rc\Domain\Repository\Research\Exposition;

use Rc\Domain\Model\Research\Exposition\Popover;
use Rc\Domain\Model\Research\Exposition\Weave;
use Rc\Domain\Repository\MongoRepository;

/**
 * @method static Popover|null findById($id)
 * @method static Popover getById($id)
 * @method static Popover[] findByIds(array $id, array $options = [])
 * @method static Popover[] findAll(array $options = [])
 */
class PopoverRepository extends MongoRepository
{

    /**
     * @param Weave $weave
     * @param array $options
     * @return Popover[]
     */
    public static function findByWeave(Weave $weave, array $options = [])
    {
        return self::find([
            'weave.$id' => new \MongoId($weave->getUuid()),
            'deleted'   => false,
        ], $options + ['sort' => ['creationDate' => self::SORT_ASC]]);
    }

    /**
     * @param Weave $weave
     * @param array $options
     * @return Popover[]
     */
    public static function findDeletedByWeave(Weave $weave, array $options = [])
    {
        return self::find([
            'weave.uuid' => (string)$weave->getUuid(),
            'deleted'    => true,
        ], $options);
    }

    /**
     * @param int $id
     * @param array $options
     * @return Popover
     */
    public static function findDeletedById($id, array $options = [])
    {
        return self::findOne([
            'weave.id' => (int)$id,
            'deleted'  => true,
        ], $options);
    }
}