<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */
namespace Rc\Domain\Repository\Research\Exposition;

use Identity;
use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Model\Research\Exposition\SimpleMedia;
use Rc\Domain\Model\Research\Exposition\Weave\Tool;
use Rc\Domain\Model\User;
use Rc\Domain\Repository\MongoRepository;

/**
 * @method static SimpleMedia|null findById($id)
 * @method static SimpleMedia|null findOne(array $query = [], array $options = [])
 * @method static SimpleMedia getById($id)
 * @method static SimpleMedia[] findByIds(array $id, array $options = [])
 * @method static SimpleMedia[] findAll(array $options = [])
 */
class SimpleMediaRepository extends MongoRepository
{

    /**
     * @param Exposition $exposition
     * @param array $options
     * @return SimpleMedia[]
     */
    public static function findByResearch(Exposition $exposition, array $options = [])
    {
        return self::find([
            'research.uuid' => (string)$exposition->getUuid(),
            'hidden'        => false,
        ], $options);
    }

    /**
     * @param Tool $tool
     * @param Exposition $exposition
     * @param array $options
     * @return SimpleMedia[]
     */
    public static function findByTool(Tool $tool, Exposition $exposition, array $options = [])
    {
        $researchIds = [
            $exposition->getUuid(),
        ];
        foreach ($exposition->getShares() as $share) {
            $researchIds[] = $share->getUuid();
        }
        foreach (Identity::getUser()->getWorks() as $work) {
            $researchIds[] = $work->getUuid();
        }
        return self::find([
            'research.uuid' => ['$in' => self::castListToString($researchIds)],
            '_class'        => ['$in' => $tool->getSupportedSimpleMedias()],
            'hidden'        => false,
        ], $options);
    }

    /**
     * @param User $user
     * @param array $options
     * @return SimpleMedia[]
     */
    public static function findByUser(User $user, array $options = [])
    {
        return self::find([
            'owner.uuid' => (string)$user->getUuid(),
        ], $options);
    }
}