<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Repository\Research\Exposition\Weave;

use Doctrine\ODM\MongoDB\MongoDBException;
use MongoException;
use MongoId;
use Rc\Domain\Model\ModelInterface;
use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Model\Research\Exposition\SimpleMedia;
use Rc\Domain\Model\Research\Exposition\SimpleMediaInterface;
use Rc\Domain\Model\Research\Exposition\Weave;
use Rc\Domain\Model\Research\Exposition\Weave\ToolInterface;
use Rc\Domain\Repository\Exception;
use Rc\Domain\Repository\MongoRepository;
use ToolFactory;

/**
 * @method static ToolInterface|null findById($id)
 * @method static ToolInterface getById($id)
 * @method static ToolInterface[] findByIds(array $id, array $options = [])
 * @method static ToolInterface[] findAll(array $options = [])
 * @method static ToolInterface merge(ModelInterface $object)
 */
class ToolRepository extends MongoRepository
{

    /**
     * @param SimpleMediaInterface $simpleMedia
     * @param array $options
     * @return ToolInterface[]
     * @throws Exception
     */
    public static function findBySimpleMedia(SimpleMediaInterface $simpleMedia, $options = [])
    {
        return self::find([
            '$or' => [
                ['thumb.uuid' => (string)$simpleMedia->getUuid()],
                ['mediaList.uuid' => (string)$simpleMedia->getUuid()],
            ],
        ], $options);
    }

    /**
     * @param string|array $genre
     * @param Weave|null $weave
     * @param array $options
     * @return ToolInterface[]
     * @throws Exception
     * @throws MongoException
     */
    public static function findByGenreAndWeave($genre, Weave $weave = null, array $options = [])
    {
        return self::find([
            'weave.$id' => new MongoId($weave->getUuid()),
            'deleted'   => false,
            '_class'    => [
                '$in' => array_map(function ($name) {
                    return ToolFactory::getObjectName($name);
                }, (array)$genre),
            ],
        ], $options);
    }

    /**
     * @param Exposition $exposition
     * @param array $options
     * @return ToolInterface[]
     * @throws Exception
     */
    public static function findByExposition(Exposition $exposition, array $options = [])
    {
        return self::find([
            'weave.uuid' => [
                '$in' => array_map(function (Weave $weave) {
                    return $weave->getUuid();
                }, $exposition->getWeaves()),
            ],
        ], $options);
    }

    /**
     * @param SimpleMedia[] $simpleMediaList
     * @param array $options
     * @return ToolInterface[]
     * @throws Exception
     */
    public static function findBySimpleMedias(array $simpleMediaList, array $options = [])
    {
        $uuidList = self::castListToUuidList($simpleMediaList);
        return self::find([
            '$or' => [
                ['thumb.uuid' => $uuidList],
                ['mediaList.uuid' => $uuidList],
            ],
        ], $options);
    }

    /**
     * @param ToolInterface $tool
     * @param string $newClassName
     * @return ToolInterface
     * @throws MongoDBException
     */
    public static function convert(ToolInterface $tool, $newClassName)
    {
        return self::getRepository()
            ->createQueryBuilder()
            // find
            ->findAndUpdate()
            ->field('_id')->equals((string)$tool->getUuid())
            // update
            ->field('_class')->set((string)$newClassName)
            ->returnNew(true)
            ->getQuery()
            ->execute();
    }
}