<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */
namespace Rc\Domain\Repository\Research;

use Rc\Domain\Model\Research\Work;
use Rc\Domain\Model\User;
use Rc\Domain\Repository\Generic\SearchResult;
use Rc\Domain\Repository\IdMappingRepository;
use Rc\Domain\Repository\ResearchRepository;
use Rc\Domain\Repository\SingleCollectionInheritanceInterface;

/**
 * @method static Work|null findById($id)
 * @method static Work getById($id)
 * @method static Work[] findByIds(array $id, array $options = [])
 * @method static Work[] findAll(array $options = [])
 */
class WorkRepository extends ResearchRepository implements SingleCollectionInheritanceInterface
{

    /**
     * @param array $arguments
     * @param array $options
     * @return Work[]|SearchResult
     */
    public static function search(array $arguments, array $options = [])
    {
        $queryBuilder = self::createQueryBuilder([], $options);

        // title
        if (isset($arguments['title'])) {
            $queryBuilder->addAnd(self::buildSearchRegexForTranslationList('title', $arguments['title']));
        }

        // genre
        if (isset($arguments['genre'])) {
            $queryBuilder->addAnd(['genre' => (string)$arguments['genre']]);
        }

        // keywords
        if (isset($arguments['keyword'])) {
            $queryBuilder->addAnd(['keywords' => ['$in' => [(string)$arguments['keyword']]]]);
        }

        // author
        if (isset($arguments['author'])) {
            $authorUuid = IdMappingRepository::getByTypeAndReferenceId(User::class, $arguments['author'])->getReferenceUuid();
            $queryBuilder->addAnd(['$or' => [
                ['author.uuid' => $authorUuid],
                ['coAuthors.uuid' => $authorUuid],
            ]]);
        }

        // fulltext
        if (isset($arguments['fulltext'])) {
            $queryBuilder->text((string)$arguments['fulltext']);
            $queryBuilder->sortMeta('score', 'textScore');
        }

        // accessible condition
        if (!empty($arguments['accessible'])) {
            $queryBuilder->addAnd(self::getIsAccessibleCondition($arguments['accessible'][0], $arguments['accessible'][1]));
        } else {
            $queryBuilder->addAnd(self::getIsAccessibleCondition());
        }

        return new SearchResult($queryBuilder);
    }

}