<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Serializer\Handler;

use Doctrine\Common\Util\ClassUtils;
use JMS\Serializer\DeserializationContext;
use JMS\Serializer\GraphNavigator;
use JMS\Serializer\Handler\SubscribingHandlerInterface;
use JMS\Serializer\SerializationContext;
use JMS\Serializer\VisitorInterface;
use Rc\Domain\Model\Core\OrmAwareTrait;

class Uuid implements SubscribingHandlerInterface
{
    use OrmAwareTrait;

    /**
     * Return format:
     *      array(
     *          array(
     *              'direction' => GraphNavigator::DIRECTION_SERIALIZATION,
     *              'format' => 'json',
     *              'type' => 'DateTime',
     *              'method' => 'serializeDateTimeToJson',
     *          ),
     *      )
     * The direction and method keys can be omitted.
     *
     * @return array
     */
    public static function getSubscribingMethods()
    {
        $methods = [];
        $formats = ['json', 'xml', 'yml'];

        foreach ($formats as $format) {
            $methods[] = [
                'direction' => GraphNavigator::DIRECTION_SERIALIZATION,
                'type'      => \Rc\Serializer\Type\Uuid::class,
                'format'    => $format,
                'method'    => 'serialize',
            ];

            $methods[] = [
                'direction' => GraphNavigator::DIRECTION_DESERIALIZATION,
                'type'      => \Rc\Serializer\Type\Uuid::class,
                'format'    => $format,
                'method'    => 'deserialize',
            ];
        }

        return $methods;
    }

    /**
     * @param VisitorInterface $visitor
     * @param string $uuid
     * @param array $type
     * @param SerializationContext $context
     * @return array
     */
    public function serialize(VisitorInterface $visitor, $uuid, array $type, SerializationContext $context)
    {
        return [
            '$uuid'  => $uuid,
            '$class' => ClassUtils::getClass($context->getObject()),
        ];
    }

    /**
     * @param VisitorInterface $visitor
     * @param array $data
     * @param array $type
     * @param DeserializationContext $context
     * @return mixed|object
     */
    public function deserialize(VisitorInterface $visitor, array $data, array $type, DeserializationContext $context)
    {
        return $data['$uuid'];
    }

}
