<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Serializer\Listener;

use JMS\Serializer\EventDispatcher\EventSubscriberInterface;
use JMS\Serializer\EventDispatcher\PreSerializeEvent;
use JMS\Serializer\SerializationContext;
use Rc\Domain\Model\ModelInterface;

class AllowCycles implements EventSubscriberInterface
{

    /**
     * Returns the events to which this class has subscribed.
     * Return format:
     *     array(
     *         array('event' => 'the-event-name', 'method' => 'onEventName', 'class' => 'some-class', 'format' => 'json'),
     *         array(...),
     *     )
     * The class may be omitted if the class wants to subscribe to events of all classes.
     * Same goes for the format key.
     *
     * @return array
     */
    public static function getSubscribedEvents()
    {
        return [
            ['event' => 'serializer.pre_serialize', 'method' => 'onPreSerialize'],
        ];
    }

    /**
     * Allow cycles, because we don't know on which entity doctrine stores the relation information. Because all
     * entities are exported as relation we cannot have cycles.
     *
     * @param PreSerializeEvent $event
     */
    public function onPreSerialize(PreSerializeEvent $event)
    {
        /** @var SerializationContext $context */
        $context = $event->getContext();

        if ($context->getObject() instanceof ModelInterface) {
            $context->getVisitingSet()->detach($context->getObject());
        }
    }
}