<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Serializer\Visitor;

class JsonDeserializationVisitor extends \JMS\Serializer\JsonDeserializationVisitor

{
    /**
     * @param mixed $data
     * @return mixed
     */
    public function prepare($data)
    {
        $data = parent::prepare($data);

        $data = $this->replaceUuids($data);

        return $data;
    }

    /**
     * @param array $data
     * @return array
     */
    protected function replaceUuids(array $data)
    {
        $mapping = $this->buildUuidMapping($data);

        array_walk_recursive($data, function (&$value, $key) use ($mapping) {
            if ($key !== '$uuid' && $key !== '$reference') {
                return;
            }

            if (!isset($mapping[$value])) {
                return;
            }

            $value = $mapping[$value];
        });

        return $data;
    }

    /**
     * @param array $data
     * @return array
     */
    protected function findEntityUuids(array $data)
    {
        if (isset($data['$uuid'])) {
            return [$data['$uuid']];
        }

        $uuids = [];
        foreach ($data as $key => $value) {
            if (!isset($value) || !is_array($value)) {
                continue;
            }

            $result = $this->findEntityUuids($value);
            if (empty($result)) {
                continue;
            }

            $uuids = array_merge($uuids, $result);
        }
        return $uuids;
    }

    /**
     * @param array $data
     * @return array
     */
    protected function findReferencedUuids(array $data)
    {
        if (isset($data['$reference'])) {
            return [$data['$reference']];
        }

        $uuids = [];
        foreach ($data as $key => $value) {
            if (!isset($value) || !is_array($value)) {
                continue;
            }

            $result = $this->findReferencedUuids($value);
            if (empty($result)) {
                continue;
            }

            $uuids = array_merge($uuids, $result);
        }
        return $uuids;
    }

    /**
     * @param array $data
     * @return array
     */
    protected function buildUuidMapping(array $data)
    {
        $mapping = [];

        $entityUuids = $this->findEntityUuids($data);

        foreach ($entityUuids as $uuid) {
            $mapping[$uuid] = $this->generateUuid();
        }

        foreach (array_intersect($entityUuids, $this->findReferencedUuids($data)) as $uuid) {
            $mapping[$uuid] = $this->generateUuid();
        }

        return $mapping;
    }

    /**
     * @return string
     */
    protected function generateUuid()
    {
        return (string)new \MongoId();
    }
}