<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */
namespace Rc\Service\Ingest;

use Rc\Domain\Model\Media\Resource\PdfResource;
use Rc\Domain\Model\Media\Resource\Type\RoleType;

class PdfIngestService extends ImageIngestService
{

    /**
     * @param \Rc\Domain\Model\Media\Resource $master
     * @return \Rc\Domain\Model\Media\Resource
     */
    public static function processSplashScreen($master)
    {
        $target = tempnam(temp_path(), 'ingest_') . '.png';
        $command = str_replace(
            ['SOURCE', 'PARAM', 'ADDITIONAL', 'TARGET'],
            [escapeshellarg($master->getUri() . '[0]'), '', '', escapeshellarg($target)],
            config('ingest.convert')
        );

        exec($command);
        if (!file_exists($target)) {
            return null;
        }

        return ImageIngestService::processResource($target, new RoleType(RoleType::SPLASH_SCREEN));
    }

    /**
     * @param \Rc\Domain\Model\Media\Resource $master
     * @return \Rc\Domain\Model\Media\Resource[]
     */
    public static function processDerivativeList($master)
    {
        return [];
    }

    /**
     * @return PdfResource
     */
    public static function getResourcePrototype()
    {
        return new PdfResource();
    }

    /**
     * @param PdfResource $resource
     */
    protected static function enrichResource($resource)
    {
        parent::enrichResource($resource);
        $resource->setWidth($resource->getWidth() * 2);
        $resource->setHeight($resource->getHeight() * 2);

        $command = str_replace('SOURCE', $resource->getUri(), config('ingest.pdf_info'));
        exec($command, $result);

        foreach ($result as $row) {
            if (preg_match('/^PDF version: (.*)$/', $row, $matches)) {
                trim($matches[1]); // pdf version
            } elseif (preg_match('/^Page size: (.*)$/', $row, $matches)) {
                trim($matches[1]); // page size
            } elseif (preg_match('/^Pages: (.*)$/', $row, $matches)) {
                $resource->setPages(intval(trim($matches[1])));
            }
        }
    }

}