<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Service\Madek;

use Rc\Domain\Model\Core\Acl\AclAwareInterface;
use Rc\Domain\Model\Core\DateTimeTrait;
use Rc\Domain\Model\Media\Dimension;
use Rc\Domain\Model\MediaInterface;
use Rc\Domain\Model\Research;
use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Model\Research\Exposition\SimpleMediaInterface;
use Rc\Domain\Model\Research\MediaContainerInterface;
use Rc\Domain\Model\User;

class MadekMedia implements SimpleMediaInterface
{
    use DateTimeTrait;

    /**
     * @var array
     */
    protected $data;

    /**
     * @param array $data
     */
    public function __construct(array $data)
    {
        $this->data = $data;
    }

    /**
     * @return AclAwareInterface
     */
    public function getAclEntity()
    {
        // TODO: Implement getAclEntity() method.
    }

    /**
     * @return boolean
     */
    public function hasThumb()
    {
        // TODO: Implement hasThumb() method.
    }

    /**
     * @return MediaInterface
     */
    public function getThumb()
    {
        // TODO: Implement getThumb() method.
    }

    /**
     * @param MediaInterface $thumb
     */
    public function setThumb(MediaInterface $thumb = null)
    {
        // TODO: Implement setThumb() method.
    }

    /**
     * @return boolean
     */
    public function hasMedia()
    {
        // TODO: Implement hasMedia() method.
    }

    /**
     * @return MediaInterface
     */
    public function getMedia()
    {
        // TODO: Implement getMedia() method.
    }

    /**
     * @param MediaInterface $media
     */
    public function setMedia(MediaInterface $media = null)
    {
        // TODO: Implement setMedia() method.
    }

    /**
     * @param MediaInterface[] $mediaList
     */
    public function setMediaList(array $mediaList)
    {
        // TODO: Implement setMediaList() method.
    }

    /**
     * @return MediaInterface[]
     */
    public function getMediaList()
    {
        // TODO: Implement getMediaList() method.
    }

    /**
     * @return Dimension
     */
    public function getMediaResolution()
    {
        // TODO: Implement getMediaResolution() method.
    }

    /**
     * @return MediaContainerInterface
     */
    public function getMediaContainer()
    {
        // TODO: Implement getMediaContainer() method.
    }

    /**
     * @return string
     */
    public function getUuid()
    {
        return $this->data('id');
    }

    /**
     * @return void
     */
    public function getId()
    {
        throw new \BadMethodCallException('Not implemented!');
    }

    /**
     * @param string|bool $format
     * @return \DateTime|string
     */
    public function getCreated($format = true)
    {
        return $this->formatDateTime(new \DateTime($this->data('created_at')), $format);
    }

    /**
     * @param string|bool $format
     * @return \DateTime|string
     */
    public function getModified($format = true)
    {
        return $this->getCreated($format);
    }

    /**
     * @return boolean
     */
    public function getDeleted()
    {
        return false;
    }

    /**
     * @return string
     */
    public function __toString()
    {
        return get_called_class() . '#' . $this->getUuid();
    }

    /**
     * @param array $values
     * @return $this
     */
    public function setValues(array $values)
    {
        throw new \BadMethodCallException('Not implemented!');
    }

    /**
     * @deprecated
     * @return array
     */
    public function getValues()
    {
        throw new \BadMethodCallException('Not implemented!');
    }

    /**
     * @return User
     */
    public function getOwner()
    {
        throw new \BadMethodCallException('Not implemented!');
    }

    /**
     * @param User $owner
     * @return $this
     */
    public function setOwner(User $owner)
    {
        throw new \BadMethodCallException('Not implemented!');
    }

    /**
     * @param User $owner
     * @return bool
     */
    public function isOwner(User $owner)
    {
        return false;
    }

    /**
     * @return Research
     */
    public function getResearch()
    {
        throw new \BadMethodCallException('Not implemented!');
    }

    /**
     * @param Research $research
     */
    public function setResearch(Research $research)
    {
        throw new \BadMethodCallException('Not implemented!');
    }

    /**
     * @return Exposition
     */
    public function hasExposition()
    {
        throw new \BadMethodCallException('Not implemented!');
    }

    /**
     * @return bool
     */
    public function hasWork()
    {
        throw new \BadMethodCallException('Not implemented!');
    }

    /**
     * @return string
     */
    public function getName()
    {
        return trim($this->data('madek_core:title') . ' ' . $this->data('madek_core:subtitle'));
    }

    /**
     * @param string $name
     * @return void
     */
    public function setName($name)
    {
        throw new \BadMethodCallException('Not implemented!');
    }

    /**
     * @deprecated
     * @return string
     */
    public function getTitle()
    {
        return $this->getName();
    }

    /**
     * @deprecated
     * @param string $title
     */
    public function setTitle($title)
    {
        throw new \BadMethodCallException('Not implemented!');
    }

    /**
     * @return string
     */
    public function getCopyright()
    {
        return $this->data('madek_core:copyright_notice');
    }

    /**
     * @param string $copyright
     * @return void
     */
    public function setCopyright($copyright)
    {
        throw new \BadMethodCallException('Not implemented!');
    }

    /**
     * @deprecated
     * @return string
     */
    public function getCopyrightHolder()
    {
        return $this->getCopyright();
    }

    /**
     * @deprecated
     * @param string $copyright
     */
    public function setCopyrightHolder($copyright)
    {
        throw new \BadMethodCallException('Not implemented!');
    }

    /**
     * @return string
     */
    public function getDescription()
    {
        return $this->data('madek_core:description');
    }

    /**
     * @param string $description
     * @return void
     */
    public function setDescription($description)
    {
        throw new \BadMethodCallException('Not implemented!');
    }

    /**
     * @return bool
     */
    public function isHidden()
    {
        return false;
    }

    /**
     * @param bool $hidden
     */
    public function setHidden($hidden)
    {
        throw new \BadMethodCallException('Not implemented!');
    }

    /**
     * @return string|null
     */
    public function getMediaType()
    {
        return $this->data('media_type');
    }

    /**
     * @param string $mediaType
     * @return bool
     */
    public function hasMediaType($mediaType)
    {
        return $this->getMediaType() == $mediaType;
    }

    /**
     * @return array
     */
    public function getMediaVersions()
    {
        // TODO: Implement getMediaVersions() method.
    }

    /**
     * @return array
     */
    public function getOptions()
    {
        // TODO: Implement getOptions() method.
    }

    /**
     * @return bool
     */
    public function hasResourceFiles()
    {
        return $this->getMediaType() !== null;
    }

    /**
     * @param string $key
     * @return mixed|null
     */
    protected function data($key)
    {
        if (!isset($this->data[$key])) {
            return null;
        }

        return $this->data[$key];
    }
}